
/*
Copyright 2007-2011 Kim A. Brandt <kimabrandt@gmx.de>

This file is part of yarip.

Yarip is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Yarip is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with yarip; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

function YaripMonitorDialog()
{
    this.id = -1;
    this.hidden = true;
    this.stringbundle = null;
    this.tree = null;
    this.doScroll = true;
    this.known = {};

    this.openLocation = function()
    {
        if (this.tree.currentIndex < 0) return;

        var item = yaripMonitorTreeView.getItem(this.tree.currentIndex);
        if (item.content) {
            yarip.openLocation(item.content.spec);
        }
    }

    this.copyLocation = function()
    {
        if (this.tree.currentIndex < 0) return;

        var item = yaripMonitorTreeView.getItem(this.tree.currentIndex);
        if (item.content) {
            var clipboardHelper = Components.classes["@mozilla.org/widget/clipboardhelper;1"].getService(Components.interfaces.nsIClipboardHelper);
//            clipboardHelper.copyString(item.content.asciiSpec);
            clipboardHelper.copyString(item.content.spec);
        }
    }

    this.whitelistContent = function()
    {
        if (this.tree.currentIndex < 0) return;

        var item = yaripMonitorTreeView.getItem(this.tree.currentIndex);
        var pageName = yarip.getFirstAddress(item.location.asciiHref);
        if (!pageName) {
//            pageName = yarip.getPageName(item.location);
            pageName = item.location.pageName;
            if (!pageName) return;
        }

        var regExp = yarip.generateRegExp(item.content.asciiSpec);
        if (!regExp) return;

        var obj = {
            pageName: pageName,
            itemLocation: item.location,
            content: new YaripContentWhitelistItem(regExp),
            itemContent: item.content
        }

        window.openDialog("chrome://yarip/content/whitelistcontentdialog.xul", "whitelistcontentdialog", "chrome,modal,resizable", obj);

        if (!obj.pageName) return;
        if (!obj.content) return;

        yarip.whitelistContent(obj.pageName, obj.content.getRegExp(), obj.content, item.location);
    }

    this.blacklistContent = function()
    {
        if (this.tree.currentIndex < 0) return;

        var item = yaripMonitorTreeView.getItem(this.tree.currentIndex);
        var pageName = yarip.getFirstAddress(item.location.asciiHref);
        if (!pageName) {
//            pageName = yarip.getPageName(item.location);
            pageName = item.location.pageName;
            if (!pageName) return;
        }

        var regExp = yarip.generateRegExp(item.content.asciiSpec);
        if (!regExp) return;

        var obj = {
            pageName: pageName,
            itemLocation: item.location,
            content: new YaripContentBlacklistItem(regExp),
            itemContent: item.content
        }

        window.openDialog("chrome://yarip/content/blacklistcontentdialog.xul", "blacklistcontentdialog", "chrome,modal,resizable", obj);

        if (!obj.pageName) return;
        if (!obj.content) return;

        yarip.blacklistContent(obj.pageName, obj.content.getRegExp(), obj.content, item.location);
    }

    this.extendPage = function()
    {
        if (this.tree.currentIndex < 0) return;

        var item = yaripMonitorTreeView.getItem(this.tree.currentIndex);
        var pageName = yarip.getFirstAddress(item.location.asciiHref);
        if (!pageName) {
//            pageName = yarip.getPageName(item.location);
            pageName = item.location.pageName;
            if (!pageName) return;
        }

        var contentAddress = yarip.getFirstAddress(item.content.asciiSpec);
        if (!contentAddress) {
            contentAddress = yarip.getPageName(yarip.getLocationFromContentLocation(item.content));
//            contentAddress = yarip.getPageName(item.content);
            if (!contentAddress) return;
        }

        var obj = {
            pageName: pageName,
            pageLocation: item.location,
            contentAddress: contentAddress,
            contentLocation: item.content
        }

        window.openDialog("chrome://yarip/content/extendpagedialog.xul", "extendpagedialog", "chrome,modal,resizable", obj);

        if (!obj.pageName) return;
        if (!obj.contentAddress) return;

        yarip.extendPage(obj.pageLocation, obj.pageName, obj.contentLocation, obj.contentAddress);
    }

    this.createPage = function()
    {
        if (this.tree.currentIndex < 0) return;

        var item = yaripMonitorTreeView.getItem(this.tree.currentIndex);
        var pageName = yarip.getFirstAddress(item.location.asciiHref);
        if (!pageName) {
//            pageName = yarip.getPageName(item.location);
            pageName = item.location.pageName;
            if (!pageName) return;
        }

        var obj = {
            pageName: pageName,
            pageLocation: item.location,
            contentLocation: item.content
        }

        window.openDialog("chrome://yarip/content/createpagedialog.xul", "createpagedialog", "chrome,modal,resizable", obj);

        if (!obj.pageName) return;

        var page = yarip.createPage(obj.pageLocation, obj.pageName);
        yaripOverlay.managePages(page.getName());
    }

    this.getPageLocation = function()
    {
        if (this.tree.currentIndex < 0) return;

        var item = yaripMonitorTreeView.getItem(this.tree.currentIndex);
        return item.location;
    }

    this.getPageName = function()
    {
        if (this.tree.currentIndex < 0) return;

        var item = yaripMonitorTreeView.getItem(this.tree.currentIndex);
        return yarip.getFirstAddress(item.location.asciiHref);
    }

    this.copyRegExp = function()
    {
        if (this.tree.currentIndex < 0) return;

        var item = yaripMonitorTreeView.getItem(this.tree.currentIndex);
        var regExp = yarip.generateRegExp(item.content.asciiSpec);
        if (!regExp) return;

        var clipboardHelper = Components.classes["@mozilla.org/widget/clipboardhelper;1"].getService(Components.interfaces.nsIClipboardHelper);
        clipboardHelper.copyString(regExp);
    }

    this.logContentLocation = function(status, location, contentLocation, date, contentType)
    {
        if (!location || !contentLocation) return;
        if (this.hidden && !yarip.logWhenClosed) return;

        var item = {
            location: yarip.getLocationFromLocation(location),
            content: yarip.getContentLocationFromContentLocation(contentLocation)
        };
//        var key = item.location.asciiHref + " " + item.content.asciiSpec;
        var key = item.location.pageName + " " + item.content.asciiSpec;
        var logObj = key in this.known ? this.known[key] : null;
        var time = Date.now();
        if (logObj) {
//            if (time - logObj[1] < 10000) { // < 10 sec
//            if (status == logObj[0] && time - logObj[1] < 10000) { // < 10 sec
//            if (status == logObj[0] && time - logObj[1] < yarip.contentRepeatThreshold) {
            if (!yarip.contentRecurrence && status == logObj[0]) {
                logObj[1] = time;
                return;
            }
        }

        var hours = date.getHours();
        var minutes = date.getMinutes();
        var seconds = date.getSeconds();
        var milliseconds = date.getMilliseconds();
        item.time = (hours < 10 ? "0" : "") + hours + ":" + (minutes < 10 ? "0" : "") + minutes + ":" + (seconds < 10 ? "0" : "") + seconds + "." + (milliseconds < 10 ? "00" : (milliseconds < 100 ? "0" : "")) + milliseconds;
        item.key = key;
        item.status = status;
        item.page = item.location.href;
        item.asciiPage = item.location.asciiHref;
        item.contentLocation = item.content.spec;
        item.asciiContentLocation = item.content.asciiSpec;
        item.contentType = contentType ? "(guess) " + contentType : null;
        yaripMonitorTreeView.appendItem(item, this.doScroll, this.hidden);
        this.known[key] = [status, time];
//        this.known[key] = [status];
    }

    this.updateContentType = function(_status, location, contentLocation, contentType, statusCode)
    {
        if (!location || !contentLocation || !contentType && !statusCode) return;

        contentType = contentType != "application/x-unknown-content-type" ? contentType : null;
//        yaripMonitorTreeView.updateContentType(url, contentType, statusCode);
        yaripMonitorTreeView.updateContentType(location, contentLocation, contentType, statusCode, this.hidden);
    }

    this.clear = function()
    {
        document.getElementById("yarip-filter-textbox").value = "";
        yaripMonitorTreeView.clearItems();
        this.known = {};
    }

    this.handleEvent = function(event)
    {
        if (!event) return;

        switch(event.target.id) {
        case "yarip-monitor-menupopup":
            switch(event.type) {
            case "popupshowing":
                var item = yaripMonitorTreeView.getItem(this.tree.currentIndex);
                document.getElementById("yarip-openLocation-menuitem").disabled = !item;
                document.getElementById("yarip-copyLocation-menuitem").disabled = !item;
                document.getElementById("yarip-copyLocation-menuseparator").disabled = !item;
                document.getElementById("yarip-whitelistContent-menuitem").disabled = !item;
                document.getElementById("yarip-blacklistContent-menuitem").disabled = !item;
                document.getElementById("yarip-copyRegExp-menuitem").disabled = !item;
                document.getElementById("yarip-copyRegExp-menuseparator").disabled = !item;
                break;
            }
            break;
        case "yarip-content-tree":
            this.doScroll = yaripMonitorTreeView.rowCount == this.tree.currentIndex + 1;
            break;
        }
    }

    this.load = function()
    {
        this.stringbundle = document.getElementById("monitor-dialog-stringbundle");
        this.tree = document.getElementById("yarip-content-tree");
        this.tree.view = yaripMonitorTreeView;
        this.tree.addEventListener("select", this, false);
        yarip.addMonitorDialog(this);
    }

    this.toggleShow = function()
    {
        if (this.hidden) {
            this.show();
        } else {
            this.hide();
        }
    }

    this.unload = function()
    {
        this.hide();
        this.tree.removeEventListener("select", this, false);
        yarip.removeMonitorDialog(this);
    }

    this.show = function()
    {
        if (this.hidden)
        {
            document.getElementById('yarip-monitor-splitter').hidden = false;
            document.getElementById('yarip-monitor').hidden = false;
//            yarip.addMonitorDialog(this);
            this.hidden = false;
        }
    }

    this.hide = function()
    {
        if (!this.hidden)
        {
            document.getElementById('yarip-monitor-splitter').hidden = true;
            document.getElementById('yarip-monitor').hidden = true;
//            yarip.removeMonitorDialog(this);
            this.hidden = true;
        }
    }

    this.filterContent = function(event)
    {
        yaripMonitorTreeView.applyFilterString(event.target.value, this.tree.currentIndex);
    }

    this.filterStatus = function(status)
    {
        yaripMonitorTreeView.applyFilterStatus(status, this.tree.currentIndex);
    }

    this.tabSelect = function(browser)
    {
        //lastURI=
        //currentURI=
        //contentDocument=
        //ownerDocument=
        //parentNode=
//        yaripMonitorTreeView.key = currentURI.URI.asciiSpec;
//        yaripMonitorTreeView.applyFilterString();
    }
}

var yaripMonitorTreeView = {
    treebox: null,
    childData: [],
    childDataObj: {},
    visibleData: [],
    whitelisted: Components.classes["@mozilla.org/atom-service;1"].getService(Components.interfaces.nsIAtomService).getAtom("whitelisted"),
    blacklisted: Components.classes["@mozilla.org/atom-service;1"].getService(Components.interfaces.nsIAtomService).getAtom("blacklisted"),
    redirected: Components.classes["@mozilla.org/atom-service;1"].getService(Components.interfaces.nsIAtomService).getAtom("redirected"),
    filterStatus: -1,
    filterString: null,
    filterRegExp: null,
    filterError: false,

    get rowCount() { return this.visibleData.length; },

    cycleHeader: function(col) {},
    getCellProperties: function(row, col, properties)
    {
        if (col.id != "status") return;
        if (row < 0 || row >= this.visibleData.length) return;

        switch (this.visibleData[row].status) {
        case STATUS_WHITELISTED:
            properties.AppendElement(this.whitelisted);
            break;
        case STATUS_BLACKLISTED:
            properties.AppendElement(this.blacklisted);
            break;
        case STATUS_REDIRECTED:
            properties.AppendElement(this.redirected);
            break;
        }
    },
    getCellText: function(row, col)
    {
        var colid = col.id;
        if (colid == "status") return "";
        if (row < 0 || row >= this.visibleData.length) return "";

        var cellText = this.visibleData[row][colid];
        return cellText ? cellText : null;
    },
    canDrop: function(index, orientation) { return false; },
    cycleCell: function(row, col) {},
    drop: function(row, orientation) { return 0; },
    getCellValue: function(row, col) { return ""; },
    getColumnProperties: function(col, properties) {},
    getImageSrc: function(row, col) { return ""; },
    getLevel: function(index) { return 0; },
    getParentIndex: function(rowIndex) { return -1; },
    getProgressMode: function(row, col) { return false; },
    getRowProperties: function(index, properties) {},
    hasNextSibling: function(rowIndex, afterIndex) { return false; },
    isContainerEmpty: function(index) { return false; },
    isContainer: function(index) { return false; },
    isContainerOpen: function(index) { return false; },
    isEditable: function(row, col) { return false; },
    isSelectable: function(row, col) { return false; },
    isSeparator: function(index) { return false; },
    isSorted: function() { return false; },
    performAction: function(action) {},
    performActionOnCell: function(action, row, col) {},
    performActionOnRow: function(action, row) {},
    selectionChanged: function() {},
    setCellText: function(row, col, value) {},
    setCellValue: function(row, col, value) {},
    setTree: function(treebox) { this.treebox = treebox; },
    toggleOpenState: function(index) {},

    appendItem: function(item, doScroll, hidden)
    {
//        if (!this.treebox) return;

        if (this.childDataObj[item.key]) {
            this.childDataObj[item.key].push(item);
        } else {
            this.childDataObj[item.key] = [item];
        }

        this.childData.push(item);
        if (this.allowItem(item)) {
            this.visibleData.push(item);
            var index = this.visibleData.length - 1;
            item.index = index;
            if (this.treebox && !hidden) {
                try { this.treebox.rowCountChanged(index, 1); } catch (e) {}
                try { if (doScroll) this.treebox.ensureRowIsVisible(index); } catch (e) {}
                try { this.treebox.invalidateRow(item.index); } catch (e) {}
            }
        }
    },

    getItem: function(index)
    {
        return this.visibleData[index];
    },

    updateContentType: function(location, contentLocation, contentType, statusCode, hidden)
    {
//        if (!this.treebox) return;

//        var key = location.asciiHref + " " + contentLocation.asciiSpec;
        var key = location.pageName + " " + contentLocation.asciiSpec;
//        var items = this.childDataObj[url];
        var items = this.childDataObj[key];
        if (!items) return;

        for (var i = 0; i < items.length; i++) {
            var item = items[i];
            if (!item.statusCode && item.status != STATUS_REDIRECTED) {
                if (contentType) item.contentType = contentType;
                if (statusCode) item.statusCode = statusCode;
                if (this.treebox && !hidden) {
                    this.treebox.invalidateRow(item.index);
                }
                return;
            }
        }
    },

    clearItems: function()
    {
        if (!this.treebox) return;

        var oldRows = this.rowCount;
        this.childData = [];
        this.visibleData = [];
        this.childDataObj = {};
        this.treebox.rowCountChanged(0, -oldRows);
//        this.filterStatus = -1;
        this.filterString = null;
        this.filterRegExp = null;
        this.filterError = false;
//        document.getElementById("yarip-status-menulist").selectedIndex = 0;
    },

    applyFilterString: function(value, currentIndex, treeView)
    {
        var tv = treeView ? treeView : this;

        var prevRowCount = tv.rowCount;
        var currentItem = tv.visibleData[currentIndex];
        tv.filterString = value != "" ? value : null;
        tv.filterRegExp = null;
        tv.filterError = false;
        tv.visibleData = [];
        if (tv.filterString) {
            try {
                tv.filterRegExp = new RegExp(tv.filterString, "i");
            } catch (e) {
                tv.filterError = true;
            }
        }
        for (var i = 0; i < tv.childData.length; i++) {
            var item = tv.childData[i];
            if (tv.allowItem(item, tv)) {
                tv.visibleData.push(item);
                item.index = tv.visibleData.length - 1;
            } else {
                item.index = -1;
            }
        }
        tv.treebox.rowCountChanged(0, tv.rowCount - prevRowCount);
        if (currentItem) {
            for (var i = currentItem.index; i < tv.visibleData.length; i++) {
                if (tv.visibleData[i] && tv.visibleData[i].index > -1) {
                    tv.selection.select(tv.visibleData[i].index);
                    tv.treebox.ensureRowIsVisible(tv.visibleData[i].index);
                    break;
                }
            }
        }
        tv.treebox.invalidate();
    },

    applyFilterStatus: function(status, currentIndex)
    {
        switch (status) {
        case -1: // all
        case 0: // unknown
        case 1: // whitelisted
        case 2: // blacklisted
        case 3: // w/b
            this.filterStatus = status;
            break;
        default:
            this.filterStatus = -1;
            break;
        }
        this.applyFilterString(this.filterString, currentIndex);
    },

    allowItem: function(item, treeView)
    {
        var tv = treeView ? treeView : this;
        return !tv.filterError
            && (
                !tv.filterRegExp
                || tv.filterRegExp.test(item.time)
                || tv.filterRegExp.test(item.asciiPage)
                || tv.filterRegExp.test(item.asciiContentLocation)
                || (item.contentType && tv.filterRegExp.test(item.contentType))
                || (item.statusCode && tv.filterRegExp.test("" + item.statusCode))
            ) && (
                tv.filterStatus == -1 // all
                || (tv.filterStatus == 0 && item.status == 0) // unknown
                || (tv.filterStatus == 1 && item.status == STATUS_WHITELISTED) // whitelisted
                || (tv.filterStatus == 2 && item.status == STATUS_BLACKLISTED) // blacklisted
//                || (tv.filterStatus == 3 && item.status == STATUS_REDIRECTED) // redirected
                || (tv.filterStatus == 3 && (item.status == 1 || item.status == 2)) // w/b
           );
    }
}
