
/*
Copyright 2007-2011 Kim A. Brandt <kimabrandt@gmx.de>

This file is part of yarip.

Yarip is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Yarip is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with yarip; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

Cu.import("resource://yarip/uri.jsm");

function YaripPageDialog()
{
    this.sb = null;
    this.treePages = null;
    this.view = null;
    this.dialogheaderPage = null;
    this.tabbox = null;
    this.elementTabbox = null;
    this.contentTabbox = null;
    this.pageTabbox = null;
    this.editMenupopup = null;
    this.filterString = null;

    this.tab = null;
    this.tabs = null;
    this.userSelect = false;

    this.page = null;
    this.doReload = true;

    this.updateTab = function()
    {
        if (!this.tabbox) return;

        switch (this.tabbox.selectedIndex) {
        case 0:
            switch (this.elementTabbox.selectedIndex) {
            case 0: this.tab = "elementWhitelist"; break;
            case 1: this.tab = "elementBlacklist"; break;
            case 2: this.tab = "elementAttributeList"; break;
            case 3: this.tab = "elementScriptList"; break;
            }
            break;
        case 1:
            switch (this.contentTabbox.selectedIndex) {
            case 0: this.tab = "contentWhitelist"; break;
            case 1: this.tab = "contentBlacklist"; break;
            case 2:
                switch (this.contentHeaderTabbox.selectedIndex) {
                case 0: this.tab = "contentRequestHeaderList"; break;
                case 1: this.tab = "contentResponseHeaderList"; break;
                }
                break;
            case 3: this.tab = "contentRedirectList"; break;
            }
            break;
        case 2:
            switch (this.pageTabbox.selectedIndex) {
            case 0: this.tab = "pageStyleList"; break;
            case 1: this.tab = "pageScriptList"; break;
            case 2:
                switch (this.pageHeaderTabbox.selectedIndex) {
                case 0: this.tab = "pageRequestHeaderList"; break;
                case 1: this.tab = "pageResponseHeaderList"; break;
                }
                break;
            case 3: this.tab = "pageRedirectList"; break;
            case 4: this.tab = "pageExtensionList"; break;
            case 5: this.tab = "pageExtendedByList"; break;
            }
            break;
        }
    }

    this.setUserSelect = function(value)
    {
        this.userSelect = value != false;
    }

    this.getPageByIndex = function(index)
    {
        index = index >= 0 ? index : this.treePages.currentIndex;
        if (index < 0) return null;

        return this.view.getPageByIndex(index);
    }

    this.getPageByName = function(pageName)
    {
        if (!pageName) return null;

        return yarip.map.get(pageName);
    }

    this.setValue = function(value)
    {
        if (this.treePages.currentIndex < 0) return;

        switch (this.tab) {
        case "elementScriptList":
        case "contentRequestHeaderList":
        case "contentResponseHeaderList":
        case "pageRequestHeaderList":
        case "pageResponseHeaderList":
            this.tabs[this.tab].object.setScript(value);
            this.tabs[this.tab].list.set(this.tabs[this.tab].tree.currentIndex, 99, value);
            break;
        case "pageStyleList":
        case "pageScriptList":
            this.tabs[this.tab].list.set(null, null, value);
            this.refreshTab(this.tab, null, true);
            break;
        case "pageExtensionList":
            this.tabs[this.tab].object.setValue(value);
            break;
        }
        yarip.reloadPage(null); // reset known
    }

    this.doCut = function()
    {
        if (this.treePages.currentIndex < 0) return;

        this.page = new YaripPage();
        var toList = null;
        switch (this.tab) {
        case "elementWhitelist":
            toList = this.page.elementWhitelist;
            break;
        case "elementBlacklist":
            toList = this.page.elementBlacklist;
            break;
        case "elementAttributeList":
            toList = this.page.elementAttributeList;
            break;
        case "elementScriptList":
            toList = this.page.elementScriptList;
            break;
        case "contentWhitelist":
            toList = this.page.contentWhitelist;
            break;
        case "contentBlacklist":
            toList = this.page.contentBlacklist;
            break;
        case "contentRequestHeaderList":
            toList = this.page.contentRequestHeaderList;
            break;
        case "contentResponseHeaderList":
            toList = this.page.contentResponseHeaderList;
            break;
        case "contentRedirectList":
            toList = this.page.contentRedirectList;
            break;
        case "pageRequestHeaderList":
            toList = this.page.pageRequestHeaderList;
            break;
        case "pageResponseHeaderList":
            toList = this.page.pageResponseHeaderList;
            break;
        case "pageRedirectList":
            toList = this.page.pageRedirectList;
            break;
        case "pageExtensionList":
            toList = this.page.pageExtensionList;
            break;
        default:
            return;
        }

        var fromList = this.tabs[this.tab].list;
        var selection = this.tabs[this.tab].tree.view.selection;
        var rangeCount = selection.getRangeCount();
        var keys = {};
        for (var i = 0; i < rangeCount; i++)
        {
            var start = {};
            var end = {};
            selection.getRangeAt(i, start, end);
            for (var j = start.value; j <= end.value; j++) {
                var k = fromList.get(j, -1);
                keys[k] = true;
                toList.add(fromList.obj[k].clone(), null, true);
            }
        }
        for (var k in keys) fromList.removeByKey(k);
        this.refreshTab(this.tab, true);
        this.setUserSelect();
        yarip.reloadPage(null); // reset known
    }

    this.doCopy = function()
    {
        if (this.treePages.currentIndex < 0) return;

        this.page = new YaripPage();
        var toList = null;
        switch (this.tab) {
        case "elementWhitelist":
            toList = this.page.elementWhitelist;
            break;
        case "elementBlacklist":
            toList = this.page.elementBlacklist;
            break;
        case "elementAttributeList":
            toList = this.page.elementAttributeList;
            break;
        case "elementScriptList":
            toList = this.page.elementScriptList;
            break;
        case "contentWhitelist":
            toList = this.page.contentWhitelist;
            break;
        case "contentBlacklist":
            toList = this.page.contentBlacklist;
            break;
        case "contentRequestHeaderList":
            toList = this.page.contentRequestHeaderList;
            break;
        case "contentResponseHeaderList":
            toList = this.page.contentResponseHeaderList;
            break;
        case "contentRedirectList":
            toList = this.page.contentRedirectList;
            break;
        case "pageRequestHeaderList":
            toList = this.page.pageRequestHeaderList;
            break;
        case "pageResponseHeaderList":
            toList = this.page.pageResponseHeaderList;
            break;
        case "pageRedirectList":
            toList = this.page.pageRedirectList;
            break;
        case "pageExtensionList":
            toList = this.page.pageExtensionList;
            break;
        default:
            return;
        }

        var fromList = this.tabs[this.tab].list;
        var selection = this.tabs[this.tab].tree.view.selection;
        var rangeCount = selection.getRangeCount();
        for (var i = 0; i < rangeCount; i++)
        {
            var start = {};
            var end = {};
            selection.getRangeAt(i, start, end);
            for (var j = start.value; j <= end.value; j++) {
                var k = fromList.get(j, -1);
                toList.add(fromList.obj[k].clone(), null, true);
            }
        }
        this.setUserSelect();
    }

    this.doPaste = function()
    {
        if (!this.page) return;

        var page = this.getPageByIndex();
        if (!page) return;

        page.merge(this.page);
        page.setTemporary(false);
        this.refreshTab(null, true);
        yarip.reloadPage(null); // reset known
    }

    this.gotoPage = function()
    {
        var currentIndex = this.tabs[this.tab].tree.currentIndex;
        if (currentIndex < 0) return;

        var pageName = this.tabs[this.tab].list.get(currentIndex, 0);
        this.reloadPage(pageName, false, true, true);

        switch (this.tab) {
        case "pageExtensionList":
            this.pageTabbox.selectedIndex = 5;
            break;
        case "pageExtendedByList":
            this.pageTabbox.selectedIndex = 4;
            break;
        }
    }

    this.doSelectAll = function()
    {
        if (this.treePages.currentIndex < 0) return;

        this.tabs[this.tab].tree.focus();
        this.tabs[this.tab].tree.view.selection.selectAll();
    }

    this.add = function()
    {
        var page = this.getPageByIndex();
        if (!page) return;

        switch (this.tab) {
        case "elementWhitelist":
            var obj = {
                element: new YaripElementWhitelistItem("//div[@id='example']"),
                pageName: page.getName()
            }
            window.openDialog("chrome://yarip/content/whitelistelementdialog.xul", "whitelistelementdialog", "chrome,modal,resizable", null, obj);
            if (!obj.element) return;

            page.elementWhitelist.add(obj.element);
            break;

        case "elementBlacklist":
            var obj = {
                element: new YaripElementBlacklistItem("//iframe"),
                pageName: page.getName()
            }
            window.openDialog("chrome://yarip/content/blacklistelementdialog.xul", "blacklistelementdialog", "chrome,modal,resizable", null, obj);
            if (!obj.element) return;

            page.elementBlacklist.add(obj.element);
            break;

        case "elementAttributeList":
            var obj = {
                element: new YaripElementAttributeItem("//div[@id='example']", "style", "width: auto;"),
                attrName: "style",
                attrValue: "width: auto;",
                pageName: page.getName()
            }
            window.openDialog("chrome://yarip/content/styledialog.xul", "styledialog", "chrome,modal,resizable", null, obj);
            if (!obj.element || !obj.attrName) return;

            obj.element.setName(obj.attrName);
            obj.element.setValue(obj.attrValue);
            page.elementAttributeList.add(obj.element);
            break;

        case "elementScriptList":
            var obj = {
                element: new YaripElementScriptItem("//a[starts-with(@href,'http://www.example.net/')]",
                        "function (array) {\n" +
                        "    // Do something with the elements in the array!\n" +
                        "}"),
                pageName: page.getName()
            }
            window.openDialog("chrome://yarip/content/scriptdialog.xul", "scriptdialog", "chrome,modal,resizable", null, obj);
            if (!obj.element) return;

            page.elementScriptList.add(obj.element);
            break;

        case "contentWhitelist":
            var re = yarip.getPageRegExp(page.getName());
            var obj = {
                content: new YaripContentWhitelistItem(re),
                pageName: page.getName()
            }
            window.openDialog("chrome://yarip/content/whitelistcontentdialog.xul", "whitelistcontentdialog", "chrome,modal,resizable", obj);
            if (!obj.content) return;

//            obj.content.setCreated(Date.now());
            page.contentWhitelist.add(obj.content);
            break;

        case "contentBlacklist":
            var re = yarip.getPageRegExp(page.getName());
            var obj = {
                content: new YaripContentBlacklistItem(re),
                pageName: page.getName()
            }
            window.openDialog("chrome://yarip/content/blacklistcontentdialog.xul", "blacklistcontentdialog", "chrome,modal,resizable", obj);
            if (!obj.content) return;

            page.contentBlacklist.add(obj.content);
            break;

        case "contentRequestHeaderList":
            var re = yarip.getPageRegExp(page.getName());
            var obj = {
                header: new YaripHeaderItem(re, "Cookie",
                    "function (value) {\n" +
                    "    return \"\";\n" +
                    "}", false),
                pageName: page.getName()
            }
            window.openDialog("chrome://yarip/content/headerdialog.xul", "headerdialog", "chrome,modal,resizable", obj);
            if (!obj.header) return;

            page.contentRequestHeaderList.add(obj.header);
            break;

        case "contentResponseHeaderList":
            var re = yarip.getPageRegExp(page.getName());
            var obj = {
                header: new YaripHeaderItem(re, "Set-Cookie",
                    "function (value) {\n" +
                    "    return \"\";\n" +
                    "}", false),
                pageName: page.getName()
            }
            window.openDialog("chrome://yarip/content/headerdialog.xul", "headerdialog", "chrome,modal,resizable", obj);
            if (!obj.header) return;

            page.contentResponseHeaderList.add(obj.header);
            break;

        case "contentRedirectList":
            var regExp = null;
            var newSubStr = null;
            var domain = page.getName().replace(/^(([^:/?#]+):\/+)?|[/?#].*$/g, "").match(DOMAIN_RE);
            if (domain) {
                var notIP = !domain[3];
                regExp = "^http://" + (notIP ? "([^/?#]+\\.)?" : "") + domain[0].replace(/([.*+?|()\[\]{}\\])/g, "\\$1") + "([/?#].*)";
                newSubStr = "https://" + (notIP ? "$1" : "") + domain[0] + (notIP ? "$2" : "$1");
            } else {
                regExp = "^http://([^/?#]+.)?example.net([/?#].*)";
                newSubStr = "https://$1example.net$2";
            }

            var obj = {
                redirect: new YaripRedirectItem(regExp, newSubStr),
//                redirect: new YaripRedirectItem("^http://", "https://"),
                pageName: page.getName()
            }
            window.openDialog("chrome://yarip/content/redirectdialog.xul", "redirectdialog", "chrome,modal,resizable", obj);
            if (!obj.redirect) return;

            page.contentRedirectList.add(obj.redirect);
            break;

        case "pageRequestHeaderList":
            var re = yarip.getPageRegExp(page.getName());
            var obj = {
                header: new YaripHeaderItem(re, "Cookie",
                    "function (value) {\n" +
                    "    return \"\";\n" +
                    "}", false),
                pageName: page.getName()
            }
            window.openDialog("chrome://yarip/content/headerdialog.xul", "headerdialog", "chrome,modal,resizable", obj);
            if (!obj.header) return;

            page.pageRequestHeaderList.add(obj.header);
            break;

        case "pageResponseHeaderList":
            var re = yarip.getPageRegExp(page.getName());
            var obj = {
                header: new YaripHeaderItem(re, "Set-Cookie",
                    "function (value) {\n" +
                    "    return \"\";\n" +
                    "}", false),
                pageName: page.getName()
            }
            window.openDialog("chrome://yarip/content/headerdialog.xul", "headerdialog", "chrome,modal,resizable", obj);
            if (!obj.header) return;

            page.pageResponseHeaderList.add(obj.header);
            break;

        case "pageRedirectList":
            var regExp = null;
            var newSubStr = null;
            var domain = page.getName().replace(/^(([^:/?#]+):\/+)?|[/?#].*$/g, "").match(DOMAIN_RE);
            if (domain) {
                var notIP = !domain[3];
                regExp = "^http://" + (notIP ? "([^/?#]+\\.)?" : "") + domain[0].replace(/([.*+?|()\[\]{}\\])/g, "\\$1") + "([/?#].*)";
                newSubStr = "https://" + (notIP ? "$1" : "") + domain[0] + (notIP ? "$2" : "$1");
            } else {
                regExp = "^http://([^/?#]+.)?example.net([/?#].*)";
                newSubStr = "https://$1example.net$2";
            }

            var obj = {
                redirect: new YaripRedirectItem(regExp, newSubStr),
//                redirect: new YaripRedirectItem("^http://", "https://"),
                pageName: page.getName()
            }
            window.openDialog("chrome://yarip/content/redirectdialog.xul", "redirectdialog", "chrome,modal,resizable", obj);
            if (!obj.redirect) return;

            page.pageRedirectList.add(obj.redirect);
            break;
        }
        page.setTemporary(false);
        this.refreshTab(this.tab, true);
        yarip.reloadPage(null); // reset known
    }

    this.remove = function()
    {
        var tab = this.tabs[this.tab];
        if (tab.tree.currentIndex < 0) return;

        switch (this.tab) {
        case "elementWhitelist":
        case "elementBlacklist":
        case "elementAttributeList":
        case "elementScriptList":
        case "contentWhitelist":
        case "contentBlacklist":
        case "contentRequestHeaderList":
        case "contentResponseHeaderList":
        case "contentRedirectList":
        case "pageRequestHeaderList":
        case "pageResponseHeaderList":
        case "pageRedirectList":
        case "pageExtensionList":
        case "pageExtendedByList":
            var keys = {};
            var selection = tab.tree.view.selection;
            var rangeCount = selection.getRangeCount();
            for (var i = 0; i < rangeCount; i++)
            {
                var start = {};
                var end = {};
                selection.getRangeAt(i, start, end);
                for (var j = start.value; j <= end.value; j++) keys[tab.list.get(j, -1)] = true;
            }
            for (var k in keys) tab.list.removeByKey(k);
            this.refreshTab(this.tab, true);
            yarip.reloadPage(null); // reset known
            break;
        }
    }

    this.removeAll = function()
    {
        if (this.treePages.currentIndex < 0) return;

        var tab = this.tabs[this.tab];
        if (!tab.list || tab.list.isEmpty()) return;

        tab.list.reset();
        this.refreshTab(this.tab, true);
//        this.refreshExtMenulist();
        yarip.reloadPage(null); // reset known
    }

    this.toggleExclusive = function()
    {
//        if (this.treePages.currentIndex < 0) return;
        var page = this.getPageByIndex();
        if (!page) return;

        switch (this.tab) {
        case "contentWhitelist":
//            this.tabs[this.tab].list.setExclusive(this.tabs[tab].exclMenulist.checked);
            var exclusive = this.tabs[this.tab].exclMenulist.checked;
            this.tabs[this.tab].list.setExclusive(exclusive);
            page.setTemporary(false);
            break;
        }
    }

    this.addExtPage = function()
    {
        var page = this.getPageByIndex();
        if (!page) return;

        var object = this.tabs[this.tab].object;
        if (!object) return;

        var id = object.getValue();
        if (id != "" && id != page.getId())
        {
//            var o = object.clone();
//            o.setCreated(Date.now());
            var extPage = yarip.map.getById(id);
            var hasContents = extPage.hasContents();
            var o = new YaripPageExtensionItem(id, extPage.hasElements(), hasContents, extPage.hasScripts(), extPage.hasHeaders(), extPage.hasRedirects(), hasContents /* hasLinks */, Date.now());
            this.tabs[this.tab].list.add(o);
            page.setTemporary(false);
            this.refreshTab(this.tab, true);
        }
    }

    this.removeExtPage = function()
    {
        if (this.tabs[this.tab].tree.currentIndex < 0) return;

        var keys = {};
        var rangeCount = this.tabs[this.tab].tree.view.selection.getRangeCount();
        for (var i = 0; i < rangeCount; i++)
        {
            var start = {};
            var end = {};
            this.tabs[this.tab].tree.view.selection.getRangeAt(i, start, end);
            for (var j = start.value; j <= end.value; j++) {
                keys[this.tabs[this.tab].list.get(j, -1)] = true;
            }
        }
        for (var k in keys) this.tabs[this.tab].list.removeByKey(k);
        this.refreshTab(this.tab, true);
        this.refreshExtMenulist();
    }

//    this.newPage = function()
//    {
//        this.treePages.currentIndex = -1;
//        this.renamePage(new YaripPage(yarip.getId(), "www.example.net"));
//    }

    this.copyPage = function()
    {
        var page = this.getPageByIndex();
        if (!page) return;

        this.renamePage(page);
    }

    this.checkPageName = function(pageName)
    {
        if (!pageName) return false;

        if (yarip.checkPageName(pageName)) {
            return true;
        } else {
            alert(this.sb.getString("ERR_INVALID_PAGE_NAME"));
            return false;
        }
    }

    this.selectTab = function(page)
    {
        if (!page) return;

        var selectedIndex = 0;

        // page tabbox
        if (page.pageStyleList.isEmpty()) {
            if (page.pageScriptList.isEmpty()) {
                if (page.pageRequestHeaderList.isEmpty() && page.pageResponseHeaderList.isEmpty()) {
                    if (page.pageRedirectList.isEmpty()) {
                        if (page.pageExtensionList.isEmpty()) {
                            if (page.pageExtendedByList.isEmpty()) {
                                // pageStyleList
                                this.pageTabbox.selectedIndex = 0;
                            } else { // pageExtendedByList
                                this.pageTabbox.selectedIndex = 5;
                                selectedIndex = 2;
                            }
                        } else { // pageExtensionList
                            this.pageTabbox.selectedIndex = 4;
                            selectedIndex = 2;
                        }
                    } else { // pageRedirectList
                        this.pageTabbox.selectedIndex = 3;
                        selectedIndex = 2;
                    }
                } else { // pageRequestHeaderList and pageResponseHeaderList
                    this.pageTabbox.selectedIndex = 2;
                    selectedIndex = 2;
                }
            } else { // pageScriptList
                this.pageTabbox.selectedIndex = 1;
                selectedIndex = 2;
            }
        } else { // pageStyleList
            this.pageTabbox.selectedIndex = 0;
            selectedIndex = 2;
        }

        // content tabbox
        if (page.contentWhitelist.isEmpty()) {
            if (page.contentBlacklist.isEmpty()) {
                if (page.contentRequestHeaderList.isEmpty() && page.contentResponseHeaderList.isEmpty()) {
                    if (page.contentRedirectList.isEmpty()) {
                        // contentWhitelist
                        this.contentTabbox.selectedIndex = 0;
                    } else { // contentRedirectList
                        this.contentTabbox.selectedIndex = 3;
                        selectedIndex = 1;
                    }
                } else { // contentRequestHeaderList and contentResponseHeaderList
                    this.contentTabbox.selectedIndex = 2;
                    selectedIndex = 1;
                }
            } else { // contentBlacklist
                this.contentTabbox.selectedIndex = 1;
                selectedIndex = 1;
            }
        } else { // contentWhitelist
            this.contentTabbox.selectedIndex = 0;
            selectedIndex = 1;
        }

        // element tabbox
        if (page.elementWhitelist.isEmpty()) {
            if (page.elementBlacklist.isEmpty()) {
                if (page.elementAttributeList.isEmpty()) {
                    if (page.elementScriptList.isEmpty()) {
                        // elementWhitelist
                        this.elementTabbox.selectedIndex = 0;
                    } else { // elementScriptList
                        this.elementTabbox.selectedIndex = 3;
                        selectedIndex = 0;
                    }
                } else { // elementAttributeList
                    this.elementTabbox.selectedIndex = 2;
                        selectedIndex = 0;
                }
            } else { // elementBlacklist
                this.elementTabbox.selectedIndex = 1;
                selectedIndex = 0;
            }
        } else { // elementWhitelist
            this.elementTabbox.selectedIndex = 0;
            selectedIndex = 0;
        }

        this.contentHeaderTabbox.selectedIndex = 0;
        this.pageHeaderTabbox.selectedIndex = 0;
        this.tabbox.selectedIndex = selectedIndex;
    }

    this.selectPageByIndex = function(index)
    {
        var rowCount = this.view.getRowCount();
        if (index >= rowCount) return;

        if (index < 0) {
            this.treePages.currentIndex = -1;
            this.view.selection.select(-1);
            this.view.treebox.ensureRowIsVisible(0);
        } else {
            this.treePages.currentIndex = index;
            this.view.selection.select(index);
            this.view.treebox.ensureRowIsVisible(index);
        }
    }

    this.selectPageByName = function(pageName, noReset)
    {
        if (noReset) {
            this.filterPages();
        } else {
            this.resetFilter();
        }

        this.treePages.currentIndex = -1;
        this.view.selection.select(-1);
        if (!pageName) {
            this.view.treebox.ensureRowIsVisible(0);
        } else {
            for (var i = 0; i < this.view.getRowCount(); i++)
            {
                var page = this.view.getPageByIndex(i);
                if (page.getName() == pageName) {
                    this.treePages.currentIndex = i;
                    this.view.selection.select(i);
                    this.view.treebox.ensureRowIsVisible(i);
                    return page;
                }
            }
        }

        return null;
    }

    this.renamePage = function(page)
    {
        var oldName = null;
        if (!page) {
            page = this.getPageByIndex();
            if (!page) return false;

            oldName = page.getName();
        }
        var pageName = prompt(this.sb.getString("enterNameOfPage"), page.getName());
        if (!this.checkPageName(pageName)) return false;
        if (oldName) {
            yarip.map.remove(page);
            page.setName(pageName, true);
        } else {
            page = page.clone(true, pageName);
        }
        yarip.map.add(page);
        page.setTemporary(false);

        this.refreshPages(true);
        this.selectPageByName(pageName);
        yarip.reloadPage(null); // reset known
        return true;
    }

    this.removePages = function()
    {
        var currentIndex = this.treePages.currentIndex;
        if (currentIndex < 0) return;

        var index = currentIndex;
        var pageNames = {};
        var selection = this.view.selection;
        var rangeCount = selection.getRangeCount();
        for (var i = 0; i < rangeCount; i++)
        {
            var start = {};
            var end = {};
            selection.getRangeAt(i, start, end);
            for (var j = start.value; j <= end.value; j++) {
                pageNames[this.getPageByIndex(j).getName()] = true;
            }
        }
        for (var pageName in pageNames) yarip.map.removeByName(pageName);
        this.refreshPages(true);
        yarip.reloadPage(null); // reset known
    }

    this.refreshPages = function(load, dontSelect)
    {
        if (load) {
            if (!this.view) {
                this.view = new YaripPageTreeView();
                this.treePages.view = this.view;
            } else {
                this.view.purge();
            }
            this.view.applyFilterString(this.filterString, -1);
        }
        var page = null;
        var currentIndex = this.treePages.currentIndex;
        if (load || currentIndex < 0) {
            if (currentIndex >= 0) {
                page = this.view.getPageByIndex(currentIndex);
            } else {
                page = new YaripPage("0");
            }
        } else {
            page = this.view.getPageByIndex(currentIndex);
        }
        if (!dontSelect) this.selectPageByIndex(currentIndex);

        this.dialogheaderPage.setAttribute("title", page.getName());
        this.tabs["elementWhitelist"].list = page.elementWhitelist;
        this.tabs["elementBlacklist"].list = page.elementBlacklist;
        this.tabs["elementAttributeList"].list = page.elementAttributeList;
        this.tabs["elementScriptList"].list = page.elementScriptList;
        this.tabs["contentWhitelist"].list = page.contentWhitelist;
        this.tabs["contentBlacklist"].list = page.contentBlacklist;
        this.tabs["contentRequestHeaderList"].list = page.contentRequestHeaderList;
        this.tabs["contentResponseHeaderList"].list = page.contentResponseHeaderList;
        this.tabs["contentRedirectList"].list = page.contentRedirectList;
        this.tabs["pageStyleList"].list = page.pageStyleList;
        this.tabs["pageScriptList"].list = page.pageScriptList;
        this.tabs["pageRequestHeaderList"].list = page.pageRequestHeaderList;
        this.tabs["pageResponseHeaderList"].list = page.pageResponseHeaderList;
        this.tabs["pageRedirectList"].list = page.pageRedirectList;
        this.tabs["pageExtensionList"].list = page.pageExtensionList;
        this.tabs["pageExtendedByList"].list = page.pageExtendedByList;
        this.tabs["elementWhitelist"].tab.setAttribute("label", this.sb.getFormattedString("whitelist-tab", [this.tabs["elementWhitelist"].list.length]));
        this.tabs["elementBlacklist"].tab.setAttribute("label", this.sb.getFormattedString("blacklist-tab", [this.tabs["elementBlacklist"].list.length]));
        this.tabs["elementAttributeList"].tab.setAttribute("label", this.sb.getFormattedString("elementAttributeList-tab", [this.tabs["elementAttributeList"].list.length]));
        this.tabs["elementScriptList"].tab.setAttribute("label", this.sb.getFormattedString("elementScriptList-tab", [this.tabs["elementScriptList"].list.length]));
        this.tabs["contentWhitelist"].tab.setAttribute("label", this.sb.getFormattedString("whitelist-tab", [this.tabs["contentWhitelist"].list.length]));
        this.tabs["contentBlacklist"].tab.setAttribute("label", this.sb.getFormattedString("blacklist-tab", [this.tabs["contentBlacklist"].list.length]));
        this.tabs["contentHeader"].tab.setAttribute("label", this.sb.getFormattedString("header-tab", [this.tabs["contentRequestHeaderList"].list.length + this.tabs["contentResponseHeaderList"].list.length]));
        this.tabs["contentRequestHeaderList"].tab.setAttribute("label", this.sb.getFormattedString("request-tab", [this.tabs["contentRequestHeaderList"].list.length]));
        this.tabs["contentResponseHeaderList"].tab.setAttribute("label", this.sb.getFormattedString("response-tab", [this.tabs["contentResponseHeaderList"].list.length]));
        this.tabs["contentRedirectList"].tab.setAttribute("label", this.sb.getFormattedString("redirect-tab", [this.tabs["contentRedirectList"].list.length]));
        this.tabs["pageStyleList"].tab.setAttribute("label", this.sb.getFormattedString("pageStyleList-tab", [this.tabs["pageStyleList"].list.length]));
        this.tabs["pageScriptList"].tab.setAttribute("label", this.sb.getFormattedString("pageScriptList-tab", [this.tabs["pageScriptList"].list.length]));
        this.tabs["pageHeader"].tab.setAttribute("label", this.sb.getFormattedString("header-tab", [this.tabs["pageRequestHeaderList"].list.length + this.tabs["pageResponseHeaderList"].list.length]));
        this.tabs["pageRequestHeaderList"].tab.setAttribute("label", this.sb.getFormattedString("request-tab", [this.tabs["pageRequestHeaderList"].list.length]));
        this.tabs["pageResponseHeaderList"].tab.setAttribute("label", this.sb.getFormattedString("response-tab", [this.tabs["pageResponseHeaderList"].list.length]));
        this.tabs["pageRedirectList"].tab.setAttribute("label", this.sb.getFormattedString("redirect-tab", [this.tabs["pageRedirectList"].list.length]));
        this.tabs["pageExtensionList"].tab.setAttribute("label", this.sb.getFormattedString("pageExtensionList-tab", [this.tabs["pageExtensionList"].list.length]));
        this.tabs["pageExtendedByList"].tab.setAttribute("label", this.sb.getFormattedString("pageExtendedByList-tab", [this.tabs["pageExtendedByList"].list.length]));

        if (load) {
            this.tabs["pageExtensionList"].object = new YaripPageExtensionItem();
        }

        if (this.treePages.currentIndex >= 0)
        {
            var listObj = page.pageExtensionList.obj;
            for each (var o in listObj) {
                var p = o.getPage();
                if (!p) page.pageExtensionList.removeByKey(o.getKey());
            }
            listObj = page.pageExtendedByList.obj;
            for each (var o in listObj) {
                var p = o.getPage();
                if (!p) page.pageExtendedByList.removeByKey(o.getKey());
            }
        }

        this.refreshTab(null, true);
        this.refreshExtMenulist(load);
    }

    this.importFiles = function()
    {
        var nsIFilePicker = Components.interfaces.nsIFilePicker;
        var fp = Components.classes["@mozilla.org/filepicker;1"].createInstance(nsIFilePicker);
        fp.init(window, this.sb.getString("importPages"), nsIFilePicker.modeOpenMultiple);
        fp.appendFilters(nsIFilePicker.filterXML);
        var res = fp.show();
        if (res != nsIFilePicker.returnOK) return;
        var files = fp.files;
        var pageName = null;
        var pName = null;
        while (files.hasMoreElements()) {
            pName = yarip.load(files.getNext(), true);
            if (pName) pageName = pName;
        }
//        if (pageName) {
            this.refreshPages(true);
        if (pageName) {
            this.selectTab(this.selectPageByName(pageName));
        }
    }

    this.getDateString = function()
    {
        var now = new Date();
        var year = now.getFullYear();
        var month = now.getMonth() + 1
        var day = now.getDate();
        return year + "-" + (month < 10 ? "0" : "") + month + "-" + (day < 10 ? "0" : "") + day;
    }

    this.exportPages = function()
    {
        if (this.treePages.currentIndex < 0) return;

        var addressObj = {
            obj: {},
            exclusive: false,
            found: false
        }
        var pages = [];
        var selection = this.view.selection;
        var rangeCount = selection.getRangeCount();
        for (var i = 0; i < rangeCount; i++)
        {
            var start = {};
            var end = {};
            selection.getRangeAt(i, start, end);
            for (var j = start.value; j <= end.value; j++) {
                var page = this.getPageByIndex(j);
                pages.push(page);
            }
        }
        if (pages.length <= 0) return;

        for (var i = 0; i < pages.length; i++) {
            addressObj.obj[pages[i].getName()] = [true, true, true, true];
        }
        var map = new YaripMap();
        yarip.getExtensionAddressObj(addressObj, true);
        for (var p in addressObj.obj) map.add(this.getPageByName(p).clone());
        var dateStr = this.getDateString();
        var nsIFilePicker = Components.interfaces.nsIFilePicker;
        var fp = Components.classes["@mozilla.org/filepicker;1"].createInstance(nsIFilePicker);
        fp.init(window, this.sb.getString("exportPages"), nsIFilePicker.modeSave);
        fp.appendFilters(nsIFilePicker.filterXML);
        if (pages.length == 1) fp.defaultString = dateStr + "_" + pages[0].getName().replace(/\/+$/, "").replace(/\//g, "_") + ".xml";
        else fp.defaultString = fp.defaultString = dateStr + "_yarip-pages.xml";
        var res = fp.show();
        if (res == nsIFilePicker.returnOK || res == nsIFilePicker.returnReplace)
        {
            map.purge();
            var data = map.generateXml(true);
            yarip.saveToFile(data, fp.file);
        }
    }

    this.exportAll = function()
    {
        var dateStr = this.getDateString();
        var nsIFilePicker = Components.interfaces.nsIFilePicker;
        var fp = Components.classes["@mozilla.org/filepicker;1"].createInstance(nsIFilePicker);
        fp.init(window, this.sb.getString("exportPages"), nsIFilePicker.modeSave);
        fp.appendFilters(nsIFilePicker.filterXML);
        fp.defaultString = dateStr + "_yarip.xml";
        var res = fp.show();
        if (res == nsIFilePicker.returnOK || res == nsIFilePicker.returnReplace)
        {
            var map = yarip.map.clone();
            map.purge();
            var data = map.generateXml(true);
            yarip.saveToFile(data, fp.file);
        }
    }

    this.save = function()
    {
        yarip.save();
    }

    this.refreshTab = function(tab, load, update)
    {
        if (!tab)
        {
            this.refreshTab("elementWhitelist", load);
            this.refreshTab("elementBlacklist", load);
            this.refreshTab("elementAttributeList", load);
            this.refreshTab("elementScriptList", load);
            this.refreshTab("contentWhitelist", load);
            this.refreshTab("contentBlacklist", load);
            this.refreshTab("contentRequestHeaderList", load);
            this.refreshTab("contentResponseHeaderList", load);
            this.refreshTab("contentRedirectList", load);
            this.refreshTab("pageStyleList", load);
            this.refreshTab("pageScriptList", load);
            this.refreshTab("pageRequestHeaderList", load);
            this.refreshTab("pageResponseHeaderList", load);
            this.refreshTab("pageRedirectList", load);
            this.refreshTab("pageExtensionList", load);
            this.refreshTab("pageExtendedByList", load);
            return;
        }

        var elementCount = this.tabs["elementWhitelist"].list.length
            + this.tabs["elementBlacklist"].list.length
            + this.tabs["elementAttributeList"].list.length
            + this.tabs["elementScriptList"].list.length;
        var contentCount = this.tabs["contentWhitelist"].list.length
            + this.tabs["contentBlacklist"].list.length
            + this.tabs["contentRequestHeaderList"].list.length
            + this.tabs["contentResponseHeaderList"].list.length
            + this.tabs["contentRedirectList"].list.length;
        var contentHeaderCount = this.tabs["contentRequestHeaderList"].list.length
            + this.tabs["contentResponseHeaderList"].list.length;
        var pageCount = this.tabs["pageStyleList"].list.length
            + this.tabs["pageScriptList"].list.length
            + this.tabs["pageRequestHeaderList"].list.length
            + this.tabs["pageResponseHeaderList"].list.length
            + this.tabs["pageRedirectList"].list.length
            + this.tabs["pageExtensionList"].list.length
            + this.tabs["pageExtendedByList"].list.length;
        var pageHeaderCount = this.tabs["pageRequestHeaderList"].list.length
            + this.tabs["pageResponseHeaderList"].list.length;
        var elementTab = document.getElementById("element-tab");
        var contentTab = document.getElementById("content-tab");
        var contentHeaderTab = document.getElementById("contentHeader-tab");
        var pageTab = document.getElementById("page-tab");
        var pageHeaderTab = document.getElementById("pageHeader-tab");
        elementTab.setAttribute("label", this.sb.getFormattedString("element-tab", [elementCount]));
        contentTab.setAttribute("label", this.sb.getFormattedString("content-tab", [contentCount]));
        contentHeaderTab.setAttribute("label", this.sb.getFormattedString("header-tab", [contentHeaderCount]));
        pageTab.setAttribute("label", this.sb.getFormattedString("page-tab", [pageCount]));
        pageHeaderTab.setAttribute("label", this.sb.getFormattedString("header-tab", [pageHeaderCount]));

        switch (tab) {
        case "elementWhitelist":
        case "elementBlacklist":
        case "elementAttributeList":
        case "elementScriptList":
        case "contentRequestHeaderList":
        case "contentResponseHeaderList":
        case "pageRequestHeaderList":
        case "pageResponseHeaderList":
            if (load) {
                this.tabs[tab].tree.view = new YaripListTreeView(this.tabs[tab].list);
            }
            if (load || this.tabs[tab].tree.currentIndex < 0) {
                switch (tab) {
                case "elementWhitelist": this.tabs[tab].object = new YaripElementWhitelistItem(); break;
                case "elementBlacklist": this.tabs[tab].object = new YaripElementBlacklistItem(); break;
                case "elementAttributeList": this.tabs[tab].object = new YaripElementAttributeItem(); break;
                case "elementScriptList": this.tabs[tab].object = new YaripElementScriptItem(); break;
                case "contentRequestHeaderList":
                case "contentResponseHeaderList":
                case "pageRequestHeaderList":
                case "pageResponseHeaderList":
                    this.tabs[tab].object = new YaripHeaderItem();
                    break;
                }
            } else {
                var k = this.tabs[tab].list.get(this.tabs[tab].tree.currentIndex, -1);
                this.tabs[tab].object = this.tabs[tab].list.obj[k].clone();
            }

            this.tabs[tab].tab.setAttribute("label", this.sb.getFormattedString(tab + "-tab", [this.tabs[tab].list.length]));
            switch (tab) {
            case "elementScriptList":
            case "contentRequestHeaderList":
            case "contentResponseHeaderList":
            case "pageRequestHeaderList":
            case "pageResponseHeaderList":
                if (load || update) {
                    this.tabs[tab].textbox.value = this.tabs[tab].list.get(this.tabs[tab].tree.currentIndex, 99); // get script
                }
                break;
            }
            break;
        case "contentWhitelist":
        case "contentBlacklist":
        case "contentRedirectList":
        case "pageRedirectList":
            if (load) {
                this.tabs[tab].tree.view = new YaripListTreeView(this.tabs[tab].list);
            }
            if (load || this.tabs[tab].tree.currentIndex < 0) {
                switch (tab) {
                case "contentWhitelist": this.tabs[tab].object = new YaripContentWhitelistItem(); break;
                case "contentBlacklist": this.tabs[tab].object = new YaripContentBlacklistItem(); break;
                case "contentRedirectList":
                case "pageRedirectList":
                    this.tabs[tab].object = new YaripRedirectItem();
                    break;
                }
            } else {
                var k = this.tabs[tab].list.get(this.tabs[tab].tree.currentIndex, -1);
                this.tabs[tab].object = this.tabs[tab].list.obj[k].clone();
            }

            this.tabs[tab].tab.setAttribute("label", this.sb.getFormattedString(tab + "-tab", [this.tabs[tab].list.length]));
            switch (tab) {
            case "contentWhitelist":
                this.tabs[tab].exclMenulist.checked = this.tabs[tab].list.getExclusive();
                break;
            }
            break;
        case "pageStyleList":
        case "pageScriptList":
            if (load) {
                this.tabs[tab].textbox.value = this.tabs[tab].list.get();
            }

            this.tabs[tab].tab.setAttribute("label", this.sb.getFormattedString(tab + "-tab", [this.tabs[tab].list.length]));
            break;
        case "pageExtensionList":
        case "pageExtendedByList":
            if (load) {
                this.tabs[tab].tree.view = new YaripListTreeView(this.tabs[tab].list);
            }

            this.tabs[tab].tab.setAttribute("label", this.sb.getFormattedString(tab + "-tab", [this.tabs[tab].list.length]));
            break;
        }
    }

    this.reloadPage = function(pageName, refresh, selectPage, selectTab, reset)
    {
        var page = null;
        if (refresh) {
            if (!selectPage) {
                page = this.getPageByIndex();
                if (page) {
                    pageName = page.getName();
                    selectPage = true;
                }
            }
            this.refreshPages(true);
            this.refreshExtMenulist(true);
        }

        var tab = null;
        if (selectPage) {
            page = this.getPageByName(pageName);
            if (page) {
//                this.resetFilter();
                tab = this.selectPageByName(pageName, !reset);
            } else {
                this.selectPageByName(); // unselect!
                selectTab = true;
            }
        }
        if (selectTab) {
            this.refreshTab(null, true);
        }
    }

    this.handleEvent = function(event)
    {
        if (!event) return;

        switch(event.target.id) {
        case "yarip-edit-menupopup":
            switch (event.type) {
            case "popupshowing":
                var disabled = this.tabs[this.tab].tree.view.selection.getRangeCount() <= 0;
                var rowCount = this.tabs[this.tab].tree.view.rowCount;
                var gotoHidden = this.tab != "pageExtensionList" && this.tab != "pageExtendedByList";
                document.getElementById("cmd_cut").setAttribute("disabled", "" + (disabled || this.tab == "pageExtendedByList"));
                document.getElementById("cmd_copy").setAttribute("disabled", "" + (disabled || this.tab == "pageExtendedByList"));
                document.getElementById("cmd_paste").setAttribute("disabled", "" + !this.page);
                document.getElementById("yarip-goto-menuseparator").setAttribute("hidden", "" + gotoHidden);
                document.getElementById("yarip-goto-menuitem").setAttribute("hidden", "" + gotoHidden);
                document.getElementById("yarip-goto-menuitem").setAttribute("disabled", "" + disabled);
                document.getElementById("yarip-goto-menuitem").disabled = disabled;
                document.getElementById("cmd_goto").setAttribute("disabled", "" + disabled);
                document.getElementById("cmd_selectall").setAttribute("disabled", "" + (rowCount > 0));
                break;
            }
            break;

        case "tree-pages":
            if (this.tabs[this.tab].tree) {
                this.tabs[this.tab].tree.stopEditing(false);
            }
            this.refreshPages(null, true);
            break;

        case "elementWhitelist-tree":
        case "elementBlacklist-tree":
        case "elementAttributeList-tree":
        case "contentWhitelist-tree":
        case "contentBlacklist-tree":
        case "contentRedirectList-tree":
        case "pageStyleList-textbox":
        case "pageScriptList-textbox":
        case "pageRedirectList-tree":
        case "pageExtensionList-tree":
        case "pageExtendedByList-tree":
            this.refreshTab(this.tab);
            break;

        case "elementScriptList-tree":
        case "contentRequestHeaderList-tree":
        case "contentResponseHeaderList-tree":
        case "pageRequestHeaderList-tree":
        case "pageResponseHeaderList-tree":
            this.refreshTab(this.tab, null, true);
            break;
        }
    }

    this.initTab = function(tab)
    {
        if (!tab) return;

        switch (tab) {
        case "contentHeader":
        case "pageHeader":
            this.tabs[tab] = {};
            this.tabs[tab].tab = document.getElementById(tab + "-tab");
            break;

        case "elementWhitelist":
        case "elementBlacklist":
        case "elementAttributeList":
        case "elementScriptList":
        case "contentWhitelist":
        case "contentBlacklist":
        case "contentRequestHeaderList":
        case "contentResponseHeaderList":
        case "contentRedirectList":
        case "pageRequestHeaderList":
        case "pageResponseHeaderList":
        case "pageRedirectList":
        case "pageExtensionList":
        case "pageExtendedByList":
            this.tabs[tab] = {};
            this.tabs[tab].tab = document.getElementById(tab + "-tab");
            this.tabs[tab].tree = document.getElementById(tab + "-tree");
            this.tabs[tab].tree.addEventListener("select", this, false);
            break;

        case "pageStyleList":
        case "pageScriptList":
            this.tabs[tab] = {};
            this.tabs[tab].tab = document.getElementById(tab + "-tab");
            this.tabs[tab].textbox = document.getElementById(tab + "-textbox");
            break;
        }

        switch (tab) {
        case "elementScriptList":
        case "contentRequestHeaderList":
        case "contentResponseHeaderList":
        case "pageRequestHeaderList":
        case "pageResponseHeaderList":
            this.tabs[tab].textbox = document.getElementById(tab + "-textbox");
            break;

        case "contentWhitelist":
            this.tabs[tab].exclMenulist = document.getElementById(tab + "-exclusive");
            break;

        case "pageExtensionList":
            this.tabs[tab].menulist = document.getElementById(tab + "-menulist");
            break;
        }
    }

    this.refreshExtMenulist = function(load)
    {
        if (load)
        {
            var menupopup = document.createElement("menupopup");
            yarip.map.tree.traverse(function(node) {
                var page = node.value;
                if (page) {
                    var menuitem = document.createElement("menuitem");
                    menuitem.setAttribute("label", page.getName());
                    menuitem.setAttribute("value", page.getId());
                    menupopup.appendChild(menuitem);
                }
            });
            this.tabs["pageExtensionList"].menulist.replaceChild(menupopup, this.tabs["pageExtensionList"].menulist.firstChild);
        }

        if (this.tabs["pageExtensionList"].menulist.firstChild.childNodes.length > 0) {
            if (this.tabs["pageExtensionList"].menulist.selectedIndex < 0) {
                this.tabs["pageExtensionList"].menulist.selectedIndex = 0;
            }
            this.tabs["pageExtensionList"].object.setValue(this.tabs["pageExtensionList"].menulist.selectedItem.getAttribute("value"));
        }
    }

    this.load = function()
    {
        yarip.pageDialog = this;

        var pageName = null;
        if ("arguments" in window && window.arguments.length > 0) pageName = window.arguments[0];

        this.sb = document.getElementById("page-dialog-stringbundle");
        this.treePages = document.getElementById("tree-pages");
        this.dialogheaderPage = document.getElementById("dialogheader-page");
        this.tabbox = document.getElementById("tabbox");
        this.elementTabbox = document.getElementById("element-tabbox");
        this.contentTabbox = document.getElementById("content-tabbox");
        this.contentHeaderTabbox = document.getElementById("content-header-tabbox");
        this.pageTabbox = document.getElementById("page-tabbox");
        this.pageHeaderTabbox = document.getElementById("page-header-tabbox");
        this.editMenupopup = document.getElementById("yarip-edit-menupopup");

        this.treePages.addEventListener("select", this, false);
        this.editMenupopup.addEventListener("popupshowing", this, false);

        this.tab = "elementWhitelist";
        this.tabs = {};
        this.initTab("elementWhitelist");
        this.initTab("elementBlacklist");
        this.initTab("elementAttributeList");
        this.initTab("elementScriptList");
        this.initTab("contentWhitelist");
        this.initTab("contentBlacklist");
        this.initTab("contentHeader");
        this.initTab("contentRequestHeaderList");
        this.initTab("contentResponseHeaderList");
        this.initTab("contentRedirectList");
        this.initTab("pageStyleList");
        this.initTab("pageScriptList");
        this.initTab("pageHeader");
        this.initTab("pageRequestHeaderList");
        this.initTab("pageResponseHeaderList");
        this.initTab("pageRedirectList");
        this.initTab("pageExtensionList");
        this.initTab("pageExtendedByList");

        this.refreshPages(true);
        if (pageName) this.selectTab(this.selectPageByName(pageName));
    }

    this.unload = function()
    {
        yarip.pageDialog = null;

        this.treePages.removeEventListener("select", this, false);
        this.tabs["elementWhitelist"].tree.removeEventListener("select", this, false);
        this.tabs["elementBlacklist"].tree.removeEventListener("select", this, false);
        this.tabs["elementAttributeList"].tree.removeEventListener("select", this, false);
        this.tabs["elementScriptList"].tree.removeEventListener("select", this, false);
        this.tabs["contentWhitelist"].tree.removeEventListener("select", this, false);
        this.tabs["contentBlacklist"].tree.removeEventListener("select", this, false);
        this.tabs["contentRequestHeaderList"].tree.removeEventListener("select", this, false);
        this.tabs["contentResponseHeaderList"].tree.removeEventListener("select", this, false);
        this.tabs["contentRedirectList"].tree.removeEventListener("select", this, false);
        this.tabs["pageRequestHeaderList"].tree.removeEventListener("select", this, false);
        this.tabs["pageResponseHeaderList"].tree.removeEventListener("select", this, false);
        this.tabs["pageRedirectList"].tree.removeEventListener("select", this, false);
        this.editMenupopup.removeEventListener("popupshowing", this, false);
    }

    this.resetFilter = function()
    {
        this.filterString = null;
        document.getElementById("yarip-pageFilter-textbox").value = "";
        this.filterPages();
    }

    this.filterPages = function(event)
    {
        this.filterString = event ? event.target.value : this.filterString;
        this.view.applyFilterString(this.filterString, this.treePages.currentIndex);
        this.refreshTab(null, true);
    }
}

function YaripPageTreeView()
{
    this.treebox = null;
    this.visibleData = [];
    this.filterString = null;
    this.filterRegExp = null;
    this.filterError = false;
    this.rowCount = this.visibleData.length;

    this.getRowCount = function() { return this.visibleData.length; }

    this.cycleHeader = function(col) {}
    this.getCellProperties = function(row, col, props) {}
    this.getCellText = function(row, col) {
        if (row < 0 || row >= this.visibleData.length) return "";

        var cellText = this.visibleData[row].getName();
        return cellText ? cellText : null;
    }
    this.getCellValue = function(row, col) {
        if (row < 0 || row >= this.visibleData.length) return "";

        var cellText = this.visibleData[row].getName();
        return cellText ? cellText : null;
    }
    this.getColumnProperties = function(colid, col, props) {}
    this.getImageSrc = function(row, col) { return null; }
    this.getLevel = function(row) { return 0; }
    this.getRowProperties = function(row, props) {}
    this.isContainer = function(row) { return false; }
    this.isEditable = function(row, col) { return col.element.getAttribute("editable"); }
    this.isSeparator = function(row) { return false; }
    this.isSorted = function(row) { return false; }
    this.getParentIndex = function(rowIndex) { return -1; }
    this.setCellText = function(row, col, value) { this.setCellValue(row, col, value); }
    this.setCellValue = function(row, col, value) {
        if (!yarip.checkPageName(value)) return;

        var pageName = this.visibleData[row].getName();
        var page = yarip.map.get(pageName);
        yarip.map.remove(page);
        page.setName(value, true);
        yarip.map.add(page);
        var listObj = page.pageExtendedByList.obj;
        for each (var o in listObj) {
            var p = o.getPage();
            if (p) p.pageExtendedByList.sorted = false;
        }
        yaripPageDialog.treePages.stopEditing(false);
        if (yaripPageDialog.tabs[yaripPageDialog.tab].tree) {
            yaripPageDialog.tabs[yaripPageDialog.tab].tree.stopEditing(false);
        }
        yaripPageDialog.view.purge();
        yaripPageDialog.selectPageByName(value, true);
        yarip.reloadPage(null); // reset known
    }
    this.setTree = function(treebox) { this.treebox = treebox; }

    this.getPageByIndex = function(index)
    {
        if (index < 0 || index >= this.visibleData.length) return null;

        return this.visibleData[index];
    }

    this.applyFilterString = function(value, currentIndex, treeView)
    {
        var tv = treeView ? treeView : this;
        var prevRowCount = tv.visibleData.length;
        var currentPage = currentIndex >= 0 ? tv.visibleData[currentIndex] : null;
        tv.filterString = value != "" ? value : null;
        tv.filterRegExp = null;
        tv.filterError = false;
        tv.visibleData = [];
        if (tv.filterString) {
            try {
                tv.filterRegExp = new RegExp(tv.filterString, "i");
            } catch (e) {
                tv.filterError = true;
            }
        }
        var arr = yarip.map.tree.toArray();
        for (var i = 0; i < arr.length; i++) {
            var page = arr[i];
            if (tv.allowPage(page, tv)) {
                tv.visibleData.push(page);
                page.index = tv.visibleData.length - 1;
            } else {
                page.index = -1;
            }
        }
        tv.rowCount = tv.visibleData.length;
        tv.treebox.rowCountChanged(0, tv.rowCount - prevRowCount);
        if (currentPage) {
            for (var i = currentPage.index; i < tv.rowCount; i++) {
                if (tv.visibleData[i] && tv.visibleData[i].index > -1) {
                    tv.selection.select(tv.visibleData[i].index);
                    tv.treebox.ensureRowIsVisible(tv.visibleData[i].index);
                    break;
                }
            }
        }
        tv.treebox.invalidate();
    }

    this.allowPage = function(page, treeView)
    {
        var tv = treeView ? treeView : this;
        return !tv.filterError && (!tv.filterRegExp || tv.filterRegExp.test(page.getName()));
    }

    this.purge = function()
    {
        this.applyFilterString(null, -1);
    }
}

function YaripListTreeView(list)
{
    this.rowCount = list.length;
    this.getRowCount = function() { return this.rowCount; }
    this.cycleHeader = function(col) {}
    this.getCellProperties = function(row, col, props) {}
    this.getCellText = function(row, col) { return list.get(row, col.index); }
    this.getCellValue = function(row, col) { return list.get(row, col.index); }
    this.getColumnProperties = function(colid, col, props) {}
    this.getImageSrc = function(row, col) { return null; }
    this.getLevel = function(row) { return 0; }
    this.getRowProperties = function(row, props) {}
    this.isContainer = function(row) { return false; }
    this.isEditable = function(row, col) { return col.element.getAttribute("editable"); }
    this.isSeparator = function(row) { return false; }
    this.isSorted = function(row) { return false; }
    this.setCellText = function(row, col, value) { this.setCellValue(row, col, value); }
    this.setCellValue = function(row, col, value) {
        var refresh = list.set(row, col.index, value);
        if (refresh) {
            this.treebox.invalidate();
        } else {
            this.treebox.invalidateCell(row, col);
        }
        yarip.reloadPage(null); // reset known
    }
    this.setTree = function(treebox) { this.treebox = treebox; }
}

