
/*
Copyright 2007-2011 Kim A. Brandt <kimabrandt@gmx.de>

This file is part of yarip.

Yarip is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Yarip is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with yarip; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

const EXPORTED_SYMBOLS = [
        "YaripItem",
        "YaripElementWhitelistItem",
        "YaripElementBlacklistItem",
        "YaripElementAttributeItem",
        "YaripElementScriptItem",
        "YaripContentItem",
        "YaripContentWhitelistItem",
        "YaripContentBlacklistItem",
        "YaripHeaderItem",
        "YaripRedirectItem",
        "YaripPageExtensionItem"
    ];

const Cc = Components.classes;
const Ci = Components.interfaces;
const Cr = Components.results;
const Cu = Components.utils;

Cu.import("resource://yarip/object.jsm");

function YaripItem(xpath, script, deep, force, dow, placeholder, created, lastFound, found, notFound, ignored)
//function YaripItem(xpath, deep, force, dow, placeholder, created, lastFound, found, notFound, ignored)
{
    this.xpath = "";
    this.script = "";
    this.deep = true;
    this.force = true;
    this.dow = false;
    this.placeholder = false;
    this.created = -1;
    this.lastFound = -1;
    this.found = 0;
    this.notFound = 0;
    this.ignored = 0;

    this.setXPath(xpath);
    this.setScript(script);
    this.setDeep(deep);
    this.setForce(force);
    this.setDOW(dow);
    this.setPlaceholder(placeholder);
    this.setCreated(created ? created : Date.now());
    this.setLastFound(lastFound);
    this.setFound(found);
    this.setNotFound(notFound);
    this.setIgnored(ignored);
}
YaripItem.prototype.getKey = function()
{
    return this.getXPath();
}
YaripItem.prototype.setXPath = function(value)
{
    if (!value) return;
    this.xpath = "" + value;
    this.setCreated(Date.now());
}
YaripItem.prototype.getXPath = function()
{
    return this.xpath;
}
YaripItem.prototype.setScript = function(value)
{
    if (!value) return;
    this.script = value;
}
YaripItem.prototype.getScript = function()
{
    return this.script;
}
YaripItem.prototype.setDeep = function(value)
{
    if (!value) return;
    this.deep = "" + value != "false";
}
YaripItem.prototype.getDeep = function()
{
    return this.deep;
}
YaripItem.prototype.setForce = function(value)
{
    this.force = "" + value != "false";
}
YaripItem.prototype.getForce = function()
{
    return this.force;
}
YaripItem.prototype.setDOW = function(value)
{
    this.dow = "" + value == "true";
}
YaripItem.prototype.getDOW = function()
{
    return this.dow;
}
YaripItem.prototype.setPlaceholder = function(value)
{
    this.placeholder = "" + value == "true";
}
YaripItem.prototype.getPlaceholder = function()
{
    return this.placeholder;
}
YaripItem.prototype.setCreated = function(value)
{
    if (!value) return;
    value = Number(value);
    if (typeof value != "number" || value + "" == "NaN") return;
    this.created = value;
}
YaripItem.prototype.getCreated = function()
{
    return this.created;
}
YaripItem.prototype.setLastFound = function(value)
{
    if (!value) return;
    value = Number(value);
    if (typeof value != "number" || value + "" == "NaN") return;
    this.lastFound = value;
}
YaripItem.prototype.getLastFound = function()
{
    return this.lastFound;
}
YaripItem.prototype.setFound = function(value)
{
    if (!value) return;
    value = Number(value);
    if (typeof value != "number" || value + "" == "NaN") return;
    this.found = value;
}
YaripItem.prototype.getFound = function()
{
    return this.found;
}
YaripItem.prototype.incrementFound = function()
{
    this.found++;
    this.lastFound = Date.now();
}
YaripItem.prototype.setNotFound = function(value)
{
    if (!value) return;
    value = Number(value);
    if (typeof value != "number" || value + "" == "NaN") return;
    this.notFound = value;
}
YaripItem.prototype.getNotFound = function()
{
    return this.notFound;
}
YaripItem.prototype.incrementNotFound = function()
{
    this.notFound++;
}
YaripItem.prototype.setIgnored = function(value)
{
    if (!value) return;
    value = Number(value);
    if (typeof value != "number" || value + "" == "NaN") return;
    this.ignored = value;
}
YaripItem.prototype.getIgnored = function()
{
    return this.ignored;
}
YaripItem.prototype.incrementIgnored = function()
{
    this.ignored++;
}
YaripItem.prototype.purge = function()
{
    this.created = Date.now();
    this.lastFound = -1;
    this.found = 0;
    this.notFound = 0;
    this.ignored = 0;
}
YaripItem.prototype.clone = function(purge)
{
    var tmp = new this.constructor(this.xpath, this.script, this.deep, this.force, this.dow, this.placeholder, this.created, this.lastFound, this.found, this.notFound, this.ignored);
//    var tmp = new this.constructor(this.xpath, this.deep, this.force, this.dow, this.placeholder, this.created, this.lastFound, this.found, this.notFound, this.ignored);
    if (purge) tmp.purge();
    return tmp;
}
YaripItem.prototype.merge = function(o)
{
    if (!o) return;
    this.setDeep(o.getDeep() && this.getDeep());
    this.setForce(o.getForce() && this.getForce());
    this.setDOW(o.getDOW() || this.getDOW());
    this.setPlaceholder(o.getPlaceholder() || this.getPlaceholder());
    if (this.getCreated() == -1 || o.getCreated() < this.getCreated()) this.setCreated(o.getCreated());
}
YaripItem.prototype.compare = function(b)
{
    var aKey = this.getKey();
    var bKey = b.getKey();
    if (aKey < bKey) return -1;
    if (aKey > bKey) return 1;
    return 0;
}
YaripItem.prototype.generateXml = function()
{
    if (!this.xpath) return "";
    return "\t\t\t\t" +
        "<item" +
            " deep=\"" + this.deep + "\"" +
            " force=\"" + this.force + "\"" +
            " dow=\"" + this.dow + "\"" +
            " placeholder=\"" + this.placeholder + "\"" +
            (this.created > -1 ? " created=\"" + this.created + "\"" : "") +
            (this.lastFound > -1 ? " lastFound=\"" + this.lastFound + "\"" : "") +
            (this.found > 0 ? " found=\"" + this.found + "\"" : "") +
            (this.notFound > 0 ? " notFound=\"" + this.notFound + "\"" : "") +
            (this.ignored > 0 ? " ignored=\"" + this.ignored + "\"" : "") +
        ">\n" +
        "\t\t\t\t\t<xpath><![CDATA[" + this.xpath + "]]></xpath>\n" +
//          (this.style ? "\t\t\t\t\t<style><![CDATA[" + this.style + "]]></style>\n" : "") +
        "\t\t\t\t</item>\n";
}
YaripItem.prototype.loadFromObject = function(obj)
{
    this.setXPath(obj.xpath);
    this.setScript(obj.script);
    this.setDeep(obj.deep);
    this.setForce(obj.force);
    this.setDOW(obj.dow);
    this.setPlaceholder(obj.placeholder);
    this.setCreated(obj.created);
    this.setLastFound(obj.lastFound);
    this.setFound(obj.found);
    this.setNotFound(obj.notFound);
    this.setIgnored(obj.ignored);
}

function YaripElementWhitelistItem(xpath, deep, force, created, lastFound, found, notFound, ignored)
{
    this.xpath = "";
    this.deep = true;
    this.force = false;
    this.created = -1;
    this.lastFound = -1;
    this.found = 0;
    this.notFound = 0;
    this.ignored = 0;

    this.setXPath(xpath);
    this.setDeep(deep);
    this.setForce(force);
    this.setCreated(created ? created : Date.now());
    this.setLastFound(lastFound);
    this.setFound(found);
    this.setNotFound(notFound);
    this.setIgnored(ignored);
}
YaripElementWhitelistItem.prototype = new YaripItem;
YaripElementWhitelistItem.prototype.constructor = YaripElementWhitelistItem;
YaripElementWhitelistItem.prototype.setForce = function(value)
{
    this.force = "" + value == "true";
}
YaripElementWhitelistItem.prototype.clone = function(purge)
{
    var tmp = new this.constructor(this.xpath, this.deep, this.force, this.created, this.lastFound, this.found, this.notFound, this.ignored);
    if (purge) tmp.purge();
    return tmp;
}
YaripElementWhitelistItem.prototype.merge = function(o)
{
    if (!o) return;
    this.setDeep(o.getDeep() && this.getDeep());
    this.setForce(o.getForce() || this.getForce());
    if (this.getCreated() == -1 || o.getCreated() < this.getCreated()) this.setCreated(o.getCreated());
}
YaripElementWhitelistItem.prototype.generateXml = function()
{
    if (!this.xpath) return "";
    return "\t\t\t\t" +
        "<item" +
            " deep=\"" + this.deep + "\"" +
            " force=\"" + this.force + "\"" +
            (this.created > -1 ? " created=\"" + this.created + "\"" : "") +
            (this.lastFound > -1 ? " lastFound=\"" + this.lastFound + "\"" : "") +
            (this.found > 0 ? " found=\"" + this.found + "\"" : "") +
            (this.notFound > 0 ? " notFound=\"" + this.notFound + "\"" : "") +
            (this.ignored > 0 ? " ignored=\"" + this.ignored + "\"" : "") +
        ">\n" +
        "\t\t\t\t\t<xpath><![CDATA[" + this.xpath + "]]></xpath>\n" +
        "\t\t\t\t</item>\n";
}

function YaripElementBlacklistItem(xpath, deep, force, dow, placeholder, created, lastFound, found, notFound, ignored)
{
    this.xpath = "";
    this.deep = true;
    this.force = true;
    this.dow = false;
    this.placeholder = false;
    this.created = -1;
    this.lastFound = -1;
    this.found = 0;
    this.notFound = 0;
    this.ignored = 0;

    this.setXPath(xpath);
    this.setDeep(deep);
    this.setForce(force);
    this.setDOW(dow);
    this.setPlaceholder(placeholder);
    this.setCreated(created ? created : Date.now());
    this.setLastFound(lastFound);
    this.setFound(found);
    this.setNotFound(notFound);
    this.setIgnored(ignored);
//    this.setXPath(xpath);
}
YaripElementBlacklistItem.prototype = new YaripItem;
YaripElementBlacklistItem.prototype.constructor = YaripElementBlacklistItem;
YaripElementBlacklistItem.prototype.setXPath = function(value)
{
    if (!value) return;
    this.xpath = "" + value;
    this.setCreated(Date.now());
}
YaripElementBlacklistItem.prototype.clone = function(purge)
{
//    var tmp = new this.constructor(this.xpath, this.script, this.deep, this.force, this.dow, this.placeholder, this.created, this.lastFound, this.found, this.notFound, this.ignored);
    var tmp = new this.constructor(this.xpath, this.deep, this.force, this.dow, this.placeholder, this.created, this.lastFound, this.found, this.notFound, this.ignored);
    if (purge) tmp.purge();
    return tmp;
}
YaripElementBlacklistItem.prototype.merge = function(o)
{
    if (!o) return;
    this.setDeep(o.getDeep() && this.getDeep());
    this.setForce(o.getForce() && this.getForce());
    this.setDOW(o.getDOW() || this.getDOW());
    this.setPlaceholder(o.getPlaceholder() || this.getPlaceholder());
    if (this.getCreated() == -1 || o.getCreated() < this.getCreated()) this.setCreated(o.getCreated());
}
YaripElementBlacklistItem.prototype.generateXml = function()
{
    if (!this.xpath) return "";
    return "\t\t\t\t" +
        "<item" +
            " deep=\"" + this.deep + "\"" +
            " force=\"" + this.force + "\"" +
            " dow=\"" + this.dow + "\"" +
            " placeholder=\"" + this.placeholder + "\"" +
            (this.created > -1 ? " created=\"" + this.created + "\"" : "") +
            (this.lastFound > -1 ? " lastFound=\"" + this.lastFound + "\"" : "") +
            (this.found > 0 ? " found=\"" + this.found + "\"" : "") +
            (this.notFound > 0 ? " notFound=\"" + this.notFound + "\"" : "") +
            (this.ignored > 0 ? " ignored=\"" + this.ignored + "\"" : "") +
        ">\n" +
        "\t\t\t\t\t<xpath><![CDATA[" + this.xpath + "]]></xpath>\n" +
//        (this.style ? "\t\t\t\t\t<style><![CDATA[" + this.style + "]]></style>\n" : "") +
        "\t\t\t\t</item>\n";
}

function YaripElementAttributeItem(xpath, name, value, deep, created, lastFound, found, notFound)
{
    this.xpath = "";
    this.name = "";
    this.value = "";
    this.deep = true;
    this.created = -1;
    this.lastFound = -1;
    this.found = 0;
    this.notFound = 0;

    this.setXPath(xpath);
    this.setName(name);
    this.setValue(value);
    this.setDeep(deep);
    this.setCreated(created ? created : Date.now());
    this.setLastFound(lastFound);
    this.setFound(found);
    this.setNotFound(notFound);
//    this.setXPath(xpath);
}
YaripElementAttributeItem.prototype.getKey = function()
{
    return this.getXPath() + " " + this.getName();
}
YaripElementAttributeItem.prototype.setXPath = function(value)
{
    if (!value) return;
    this.xpath = "" + value;
    this.setCreated(Date.now());
}
YaripElementAttributeItem.prototype.getXPath = function()
{
    return this.xpath;
}
YaripElementAttributeItem.prototype.setName = function(value)
{
    if (!value) return;
    this.name = value;
    this.setCreated(Date.now());
}
YaripElementAttributeItem.prototype.getName = function()
{
    return this.name;
}
YaripElementAttributeItem.prototype.setValue = function(value)
{
    if (value) this.value = value;
}
YaripElementAttributeItem.prototype.getValue = function()
{
    return this.value;
}
YaripElementAttributeItem.prototype.setDeep = function(value)
{
    if (!value) return;
    this.deep = "" + value != "false";
}
YaripElementAttributeItem.prototype.getDeep = function()
{
    return this.deep;
}
YaripElementAttributeItem.prototype.setCreated = function(value)
{
    if (!value) return;
    value = Number(value);
    if (typeof value != "number" || value + "" == "NaN") return;
    this.created = value;
}
YaripElementAttributeItem.prototype.getCreated = function()
{
    return this.created;
}
YaripElementAttributeItem.prototype.setLastFound = function(value)
{
    if (!value) return;
    value = Number(value);
    if (typeof value != "number" || value + "" == "NaN") return;
    this.lastFound = value;
}
YaripElementAttributeItem.prototype.getLastFound = function()
{
    return this.lastFound;
}
YaripElementAttributeItem.prototype.setFound = function(value)
{
    if (!value) return;
    value = Number(value);
    if (typeof value != "number" || value + "" == "NaN") return;
    this.found = value;
}
YaripElementAttributeItem.prototype.getFound = function()
{
    return this.found;
}
YaripElementAttributeItem.prototype.incrementFound = function()
{
    this.found++;
    this.lastFound = Date.now();
}
YaripElementAttributeItem.prototype.setNotFound = function(value)
{
    if (!value) return;
    value = Number(value);
    if (typeof value != "number" || value + "" == "NaN") return;
    this.notFound = value;
}
YaripElementAttributeItem.prototype.getNotFound = function()
{
    return this.notFound;
}
YaripElementAttributeItem.prototype.incrementNotFound = function()
{
    this.notFound++;
}
YaripElementAttributeItem.prototype.purge = function()
{
    this.created = Date.now();
    this.lastFound = -1;
    this.found = 0;
    this.notFound = 0;
}
YaripElementAttributeItem.prototype.clone = function(purge)
{
    var tmp = new this.constructor(this.xpath, this.name, this.value, this.deep, this.created, this.lastFound, this.found, this.notFound);
    if (purge) tmp.purge();
    return tmp;
}
YaripElementAttributeItem.prototype.merge = function(o)
{
    if (!o) return;
//        this.setName(o.getName() || this.getName());
    this.setValue(o.getValue() || this.getValue());
    this.setDeep(o.getDeep() && this.getDeep());
    if (this.getCreated() == -1 || o.getCreated() < this.getCreated()) this.setCreated(o.getCreated());
}
YaripElementAttributeItem.prototype.compare = function(b)
{
    var aKey = this.getKey();
    var bKey = b.getKey();
    if (aKey < bKey) return -1;
    if (aKey > bKey) return 1;
    return 0;
}
YaripElementAttributeItem.prototype.generateXml = function()
{
    if (!this.xpath || !this.name || !this.value) return "";
    var tmp = "\t\t\t\t" +
        "<item" +
            " deep=\"" + this.deep + "\"" +
            (this.created > -1 ? " created=\"" + this.created + "\"" : "") +
            (this.lastFound > -1 ? " lastFound=\"" + this.lastFound + "\"" : "") +
            (this.found > 0 ? " found=\"" + this.found + "\"" : "") +
            (this.notFound > 0 ? " notFound=\"" + this.notFound + "\"" : "") +
        ">\n" +
        "\t\t\t\t\t<xpath><![CDATA[" + this.xpath + "]]></xpath>\n" +
        "\t\t\t\t\t<name><![CDATA[" + this.name + "]]></name>\n" +
        "\t\t\t\t\t<value><![CDATA[" + this.value + "]]></value>\n" +
        "\t\t\t\t</item>\n";
    return tmp;
}
YaripElementAttributeItem.prototype.loadFromObject = function(obj)
{
    this.setXPath(obj.xpath);
    this.setName(obj.name);
    this.setValue(obj.value);
    this.setDeep(obj.deep);
    this.setCreated(obj.created);
    this.setLastFound(obj.lastFound);
    this.setFound(obj.found);
    this.setNotFound(obj.notFound);
}

function YaripElementScriptItem(xpath, script, deep, created, lastFound, found, notFound)
{
    this.xpath = "";
    this.script = "";
    this.deep = true;
    this.created = -1;
    this.lastFound = -1;
    this.found = 0;
    this.notFound = 0;

    this.setXPath(xpath);
    this.setScript(script);
    this.setDeep(deep);
    this.setCreated(created ? created : Date.now());
    this.setLastFound(lastFound);
    this.setFound(found);
    this.setNotFound(notFound);
}
YaripElementScriptItem.prototype = new YaripItem;
YaripElementScriptItem.prototype.constructor = YaripElementScriptItem;
YaripElementScriptItem.prototype.clone = function(purge)
{
    var tmp = new this.constructor(this.xpath, this.script, this.deep, this.created, this.lastFound, this.found, this.notFound);
    if (purge) tmp.purge();
    return tmp;
}
YaripElementScriptItem.prototype.merge = function(o)
{
    if (!o) return;
    this.setDeep(o.getDeep() && this.getDeep());
    if (this.getCreated() == -1 || o.getCreated() < this.getCreated()) this.setCreated(o.getCreated());
}
YaripElementScriptItem.prototype.generateXml = function()
{
    if (!this.xpath) return "";
    return "\t\t\t\t" +
        "<item" +
            " deep=\"" + this.deep + "\"" +
            (this.created > -1 ? " created=\"" + this.created + "\"" : "") +
            (this.lastFound > -1 ? " lastFound=\"" + this.lastFound + "\"" : "") +
            (this.found > 0 ? " found=\"" + this.found + "\"" : "") +
            (this.notFound > 0 ? " notFound=\"" + this.notFound + "\"" : "") +
        ">\n" +
        "\t\t\t\t\t<xpath><![CDATA[" + this.xpath + "]]></xpath>\n" +
        "\t\t\t\t\t<script><![CDATA[" + this.script + "]]></script>\n" +
        "\t\t\t\t</item>\n";
}

function YaripContentItem(regexp, deep, force, created, lastFound, ignored)
{
    this.regexp = "";
    this.regexpobj = null;
    this.deep = true;
    this.force = true;
    this.created = -1;
    this.lastFound = -1;
    this.ignored = 0;

    this.setRegExp(regexp);
    this.setDeep(deep);
    this.setForce(force);
    this.setCreated(created ? created : Date.now());
    this.setLastFound(lastFound);
    this.setIgnored(ignored);
}
YaripContentItem.prototype.getKey = function()
{
    return this.getRegExp();
}
YaripContentItem.prototype.setRegExp = function(value)
{
    if (!value) return;

    this.regexp = value;
    this.setRegExpObj(value);
    this.setCreated(Date.now());
}
YaripContentItem.prototype.getRegExp = function()
{
    return this.regexp;
}
YaripContentItem.prototype.setRegExpObj = function(value)
{
    if (value) this.regexpobj = new RegExp(value);
}
YaripContentItem.prototype.getRegExpObj = function()
{
    return this.regexpobj;
}
YaripContentItem.prototype.setDeep = function(value)
{
    this.deep = "" + value != "false";
}
YaripContentItem.prototype.getDeep = function()
{
    return this.deep;
}
YaripContentItem.prototype.setForce = function(value)
{
    this.force = "" + value != "false";
}
YaripContentItem.prototype.getForce = function()
{
    return this.force;
}
YaripContentItem.prototype.setCreated = function(value)
{
    if (!value) return;
    value = Number(value);
    if (typeof value != "number" || value + "" == "NaN") return;
    this.created = value;
}
YaripContentItem.prototype.getCreated = function()
{
    return this.created;
}
YaripContentItem.prototype.setLastFound = function(value)
{
    if (!value) return;
    value = Number(value);
    if (typeof value != "number" || value + "" == "NaN") return;
    this.lastFound = value;
}
YaripContentItem.prototype.getLastFound = function()
{
    return this.lastFound;
}
YaripContentItem.prototype.incrementLastFound = function()
{
    this.lastFound = Date.now();
}
YaripContentItem.prototype.setIgnored = function(value)
{
    if (!value) return;
    value = Number(value);
    if (typeof value != "number" || value + "" == "NaN") return;
    this.ignored = value;
}
YaripContentItem.prototype.getIgnored = function()
{
    return this.ignored;
}
YaripContentItem.prototype.incrementIgnored = function()
{
    this.ignored++;
}
YaripContentItem.prototype.purge = function()
{
    this.created = Date.now();
    this.lastFound = -1;
    this.ignored = 0;
}
YaripContentItem.prototype.clone = function(purge)
{
    var tmp = new this.constructor(this.regexp, this.deep, this.force, this.created, this.lastFound, this.ignored);
    if (purge) tmp.purge();
    return tmp;
}
YaripContentItem.prototype.merge = function(o)
{
    if (!o) return;
//    this.setRegExp(o.getRegExp() || this.getRegExp());
    this.setDeep(o.getDeep() && this.getDeep());
    this.setForce(o.getForce() && this.getForce());
    if (this.getCreated() == -1 || o.getCreated() < this.getCreated()) this.setCreated(o.getCreated());
}
YaripContentItem.prototype.compare = function(b)
{
    var aKey = this.getKey();
    var bKey = b.getKey();
    if (aKey < bKey) return -1;
    if (aKey > bKey) return 1;
    return 0;
}
YaripContentItem.prototype.generateXml = function()
{
    if (!this.regexp) return "";
    return "\t\t\t\t" +
        "<item" +
            " deep=\"" + this.deep + "\"" +
            " force=\"" + this.force + "\"" +
            (this.created > -1 ? " created=\"" + this.created + "\"" : "") +
            (this.lastFound > -1 ? " lastFound=\"" + this.lastFound + "\"" : "") +
            (this.ignored > 0 ? " ignored=\"" + this.ignored + "\"" : "") +
        ">\n" +
        "\t\t\t\t\t<regexp><![CDATA[" + this.regexp + "]]></regexp>\n" +
        "\t\t\t\t</item>\n";
}
YaripContentItem.prototype.loadFromObject = function(obj)
{
    this.setRegExp(obj.regexp);
    this.setDeep(obj.deep);
    this.setForce(obj.force);
    this.setCreated(obj.created);
    this.setLastFound(obj.lastFound);
    this.setIgnored(obj.ignored);
}

function YaripContentWhitelistItem(regexp, deep, force, created, lastFound)
{
    this.regexp = "";
    this.regexpobj = null;
    this.deep = true;
    this.force = false;
    this.created = -1;
    this.lastFound = -1;

    this.setRegExp(regexp);
    this.setDeep(deep);
    this.setForce(force);
    this.setCreated(created ? created : Date.now());
    this.setLastFound(lastFound);
}
YaripContentWhitelistItem.prototype = new YaripContentItem;
YaripContentWhitelistItem.prototype.constructor = YaripContentWhitelistItem;
YaripContentWhitelistItem.prototype.setForce = function(value)
{
    this.force = "" + value == "true";
}
YaripContentWhitelistItem.prototype.clone = function(purge)
{
    var tmp = new this.constructor(this.regexp, this.deep, this.force, this.created, this.lastFound);
    if (purge) tmp.purge();
    return tmp;
}
YaripContentWhitelistItem.prototype.merge = function(o)
{
    if (!o) return;
//    this.setRegExp(o.getRegExp() || this.getRegExp());
    this.setDeep(o.getDeep() && this.getDeep());
    this.setForce(o.getForce() || this.getForce());
    if (this.getCreated() == -1 || o.getCreated() < this.getCreated()) this.setCreated(o.getCreated());
}
YaripContentWhitelistItem.prototype.generateXml = function()
{
    if (!this.regexp) return "";
    return "\t\t\t\t" +
        "<item" +
            " deep=\"" + this.deep + "\"" +
            " force=\"" + this.force + "\"" +
            (this.created > -1 ? " created=\"" + this.created + "\"" : "") +
            (this.lastFound > -1 ? " lastFound=\"" + this.lastFound + "\"" : "") +
        ">\n" +
        "\t\t\t\t\t<regexp><![CDATA[" + this.regexp + "]]></regexp>\n" +
        "\t\t\t\t</item>\n";
}

function YaripContentBlacklistItem(regexp, deep, force, created, lastFound, ignored)
{
    this.regexp = "";
    this.regexpobj = null;
    this.deep = true;
    this.force = true;
    this.created = -1;
    this.lastFound = -1;
    this.ignored = 0;

    this.setRegExp(regexp);
    this.setDeep(deep);
    this.setForce(force);
    this.setCreated(created ? created : Date.now());
    this.setLastFound(lastFound);
    this.setIgnored(ignored);
}
YaripContentBlacklistItem.prototype = new YaripContentItem;
YaripContentBlacklistItem.prototype.constructor = YaripContentBlacklistItem;

function YaripHeaderItem(regexp, headerName, script, merge, deep, created, lastFound)
{
    this.regexp = "";
    this.regexpobj = null;
    this.headerName = null;
    this.script = "";
    this._merge = false;
    this.deep = true;
    this.created = -1;
    this.lastFound = -1;

    this.setRegExp(regexp);
    this.setHeaderName(headerName);
    this.setScript(script);
    this.setMerge(merge);
    this.setDeep(deep);
    this.setCreated(created ? created : Date.now());
    this.setLastFound(lastFound);
}
YaripHeaderItem.prototype.getKey = function()
{
    return this.getRegExp() + " " + this.getHeaderName();
}
YaripHeaderItem.prototype.setRegExp = function(value)
{
    if (!value) return;

    this.regexp = value;
    this.setRegExpObj(value);
    this.setCreated(Date.now());
}
YaripHeaderItem.prototype.getRegExp = function()
{
    return this.regexp;
}
YaripHeaderItem.prototype.setRegExpObj = function(value)
{
    if (value) this.regexpobj = new RegExp(value);
}
YaripHeaderItem.prototype.getRegExpObj = function()
{
    return this.regexpobj;
}
YaripHeaderItem.prototype.setHeaderName = function(value)
{
    if (!value && value != "") return;

    this.headerName = value;
    this.setCreated(Date.now());
}
YaripHeaderItem.prototype.getHeaderName = function()
{
    return this.headerName;
}
YaripHeaderItem.prototype.setScript = function(value)
{
    if (!value && value != "") return;

    this.script = value;
}
YaripHeaderItem.prototype.getScript = function()
{
    return this.script;
}
YaripHeaderItem.prototype.setMerge = function(value)
{
    this._merge = "" + value == "true";
}
YaripHeaderItem.prototype.getMerge = function()
{
    return this._merge;
}
YaripHeaderItem.prototype.setDeep = function(value)
{
    this.deep = "" + value != "false";
}
YaripHeaderItem.prototype.getDeep = function()
{
    return this.deep;
}
YaripHeaderItem.prototype.setCreated = function(value)
{
    if (!value) return;
    value = Number(value);
    if (typeof value != "number" || value + "" == "NaN") return;
    this.created = value;
}
YaripHeaderItem.prototype.getCreated = function()
{
    return this.created;
}
YaripHeaderItem.prototype.setLastFound = function(value)
{
    if (!value) return;
    value = Number(value);
    if (typeof value != "number" || value + "" == "NaN") return;
    this.lastFound = value;
}
YaripHeaderItem.prototype.getLastFound = function()
{
    return this.lastFound;
}
YaripHeaderItem.prototype.incrementLastFound = function()
{
    this.lastFound = Date.now();
}
YaripHeaderItem.prototype.purge = function()
{
    this.created = Date.now();
    this.lastFound = -1;
}
YaripHeaderItem.prototype.clone = function(purge)
{
    var tmp = new this.constructor(this.regexp, this.headerName, this.script, this._merge, this.deep, this.created, this.lastFound);
    if (purge) tmp.purge();
    return tmp;
}
YaripHeaderItem.prototype.merge = function(o)
{
    if (!o) return;
//    this.setRegExp(o.getRegExp() || this.getRegExp());
//    this.setHeaderName(o.getHeaderName() || this.getHeaderName());
    this.setMerge(o.getMerge() || this.getMerge());
    this.setDeep(o.getDeep() && this.getDeep());
    if (this.getCreated() == -1 || o.getCreated() < this.getCreated()) this.setCreated(o.getCreated());
}
YaripHeaderItem.prototype.compare = function(b)
{
    var aKey = this.getKey();
    var bKey = b.getKey();
    if (aKey < bKey) return -1;
    if (aKey > bKey) return 1;
    return 0;
}
YaripHeaderItem.prototype.generateXml = function()
{
    if (!this.regexp || !this.getHeaderName()) return "";
    return "\t\t\t\t\t" +
        "<item" +
            (this._merge ? " merge=\"" + this._merge + "\"" : "") +
            " deep=\"" + this.deep + "\"" +
            (this.created > -1 ? " created=\"" + this.created + "\"" : "") +
            (this.lastFound > -1 ? " lastFound=\"" + this.lastFound + "\"" : "") +
        ">\n" +
        "\t\t\t\t\t\t<regexp><![CDATA[" + this.regexp + "]]></regexp>\n" +
        "\t\t\t\t\t\t<name><![CDATA[" + this.headerName + "]]></name>\n" +
        "\t\t\t\t\t\t<script><![CDATA[" + this.script + "]]></script>\n" +
        "\t\t\t\t\t</item>\n";
}
YaripHeaderItem.prototype.loadFromObject = function(obj)
{
    this.setRegExp(obj.regexp);
    this.setHeaderName(obj.headerName);
    this.setScript(obj.script);
    this.setMerge(obj.merge);
    this.setDeep(obj.deep);
    this.setCreated(obj.created);
    this.setLastFound(obj.lastFound);
}

function YaripRedirectItem(regexp, newsubstr, deep, created, lastFound)
{
    this.regexp = "";
    this.regexpobj = null;
    this.newsubstr = "";
    this.deep = true;
    this.created = -1;
    this.lastFound = -1;

    this.setRegExp(regexp);
    this.setNewSubStr(newsubstr);
    this.setDeep(deep);
    this.setCreated(created ? created : Date.now());
    this.setLastFound(lastFound);
}
YaripRedirectItem.prototype.getKey = function()
{
    return this.getRegExp();
}
YaripRedirectItem.prototype.setRegExp = function(value)
{
    if (!value) return;

    this.regexp = value;
    this.setRegExpObj(value);
    this.setCreated(Date.now());
}
YaripRedirectItem.prototype.getRegExp = function()
{
    return this.regexp;
}
YaripRedirectItem.prototype.setRegExpObj = function(value)
{
    if (value) this.regexpobj = new RegExp(value);
}
YaripRedirectItem.prototype.getRegExpObj = function()
{
    return this.regexpobj;
}
YaripRedirectItem.prototype.setNewSubStr = function(value)
{
    if (!value && value != "") return;

    this.newsubstr = value;
}
YaripRedirectItem.prototype.getNewSubStr = function()
{
    return this.newsubstr;
}
YaripRedirectItem.prototype.setDeep = function(value)
{
    this.deep = "" + value != "false";
}
YaripRedirectItem.prototype.getDeep = function()
{
    return this.deep;
}
YaripRedirectItem.prototype.setCreated = function(value)
{
    if (!value) return;
    value = Number(value);
    if (typeof value != "number" || value + "" == "NaN") return;
    this.created = value;
}
YaripRedirectItem.prototype.getCreated = function()
{
    return this.created;
}
YaripRedirectItem.prototype.setLastFound = function(value)
{
    if (!value) return;
    value = Number(value);
    if (typeof value != "number" || value + "" == "NaN") return;
    this.lastFound = value;
}
YaripRedirectItem.prototype.getLastFound = function()
{
    return this.lastFound;
}
YaripRedirectItem.prototype.incrementLastFound = function()
{
    this.lastFound = Date.now();
}
YaripRedirectItem.prototype.purge = function()
{
    this.created = Date.now();
    this.lastFound = -1;
}
YaripRedirectItem.prototype.clone = function(purge)
{
    var tmp = new this.constructor(this.regexp, this.newsubstr, this.deep, this.created, this.lastFound);
    if (purge) tmp.purge();
    return tmp;
}
YaripRedirectItem.prototype.merge = function(o)
{
    if (!o) return;
//    this.setRegExp(o.getRegExp() || this.getRegExp());
    this.setNewSubStr(o.getNewSubStr() || this.getNewSubStr());
    this.setDeep(o.getDeep() && this.getDeep());
    if (this.getCreated() == -1 || o.getCreated() < this.getCreated()) this.setCreated(o.getCreated());
}
YaripRedirectItem.prototype.compare = function(b)
{
    var aKey = this.getKey();
    var bKey = b.getKey();
    if (aKey < bKey) return -1;
    if (aKey > bKey) return 1;
    return 0;
}
YaripRedirectItem.prototype.generateXml = function()
{
    if (!this.regexp) return "";
    return "\t\t\t\t" +
        "<item" +
            " deep=\"" + this.deep + "\"" +
            (this.created > -1 ? " created=\"" + this.created + "\"" : "") +
            (this.lastFound > -1 ? " lastFound=\"" + this.lastFound + "\"" : "") +
        ">\n" +
        "\t\t\t\t\t<regexp><![CDATA[" + this.regexp + "]]></regexp>\n" +
        (this.newsubstr ? "\t\t\t\t\t<newsubstr><![CDATA[" + this.newsubstr + "]]></newsubstr>\n" : "") +
        "\t\t\t\t</item>\n";
}
YaripRedirectItem.prototype.loadFromObject = function(obj)
{
    this.setRegExp(obj.regexp);
    this.setNewSubStr(obj.newsubstr);
    this.setDeep(obj.deep);
    this.setCreated(obj.created);
    this.setLastFound(obj.lastFound);
}

function YaripPageExtensionItem(value, doElements, doContents, doScripts, doHeaders, doRedirects, doLinks, created)
{
    this.value = "";
    this.doElements = false;
    this.doContents = true;
    this.doScripts = false;
    this.doHeaders = false;
    this.doRedirects = false;
//    this.doLinks = false;
    this.doLinks = true;
    this.created = -1;


    this.setValue(value);
    this.setDoElements(doElements);
    this.setDoContents(doContents);
    this.setDoScripts(doScripts);
    this.setDoHeaders(doHeaders);
    this.setDoRedirects(doRedirects);
    this.setDoLinks(doLinks);
    this.setCreated(created ? created : Date.now());
}
YaripPageExtensionItem.prototype = new YaripObject;
YaripPageExtensionItem.prototype.constructor = YaripPageExtensionItem;
YaripPageExtensionItem.prototype.getKey = function()
{
    return this.value;
}
YaripPageExtensionItem.prototype.setValue = function(value)
{
    if (!value) return;

    this.value = "" + value;
    this.setCreated(Date.now());
}
YaripPageExtensionItem.prototype.getValue = function()
{
    return this.value;
}
YaripPageExtensionItem.prototype.setDoElements = function(value)
{
    this.doElements = "" + value == "true";
}
YaripPageExtensionItem.prototype.getDoElements = function()
{
    return this.doElements;
}
YaripPageExtensionItem.prototype.setDoContents = function(value)
{
    this.doContents = "" + value != "false";
}
YaripPageExtensionItem.prototype.getDoContents = function()
{
    return this.doContents;
}
YaripPageExtensionItem.prototype.setDoScripts = function(value)
{
    this.doScripts = "" + value == "true";
}
YaripPageExtensionItem.prototype.getDoScripts = function()
{
    return this.doScripts;
}
YaripPageExtensionItem.prototype.setDoHeaders = function(value)
{
    this.doHeaders = "" + value == "true";
}
YaripPageExtensionItem.prototype.getDoHeaders = function()
{
    return this.doHeaders;
}
YaripPageExtensionItem.prototype.setDoRedirects = function(value)
{
    this.doRedirects = "" + value == "true";
}
YaripPageExtensionItem.prototype.getDoRedirects = function()
{
    return this.doRedirects;
}
YaripPageExtensionItem.prototype.setDoLinks = function(value)
{
    this.doLinks = "" + value != "false";
}
YaripPageExtensionItem.prototype.getDoLinks = function()
{
    return this.doLinks;
}
YaripPageExtensionItem.prototype.setCreated = function(value)
{
    if (!value) return;
    value = Number(value);
    if (typeof value != "number" || value + "" == "NaN") return;
    this.created = value;
}
YaripPageExtensionItem.prototype.getCreated = function()
{
    return this.created;
}

YaripPageExtensionItem.prototype.clone = function()
{
    return new this.constructor(this.value, this.doElements, this.doContents, this.doScripts, this.doHeaders, this.doRedirects, this.doLinks);
//    var tmp = new YaripPageExtensionItem(this.value, this.doElements, this.doContents, this.doScripts, this.doHeaders, this.doRedirects, this.doLinks);
//    return tmp;
//    function Clone() {}
//    Clone.prototype = this;
//    var c = new Clone();
//    c.constructor = Clone;
//    return c;
}
YaripPageExtensionItem.prototype.merge = function(o)
{
    if (!o) return;
//    this.setValue(o.getValue() || this.getValue());
    this.setDoElements(o.getDoElements() || this.getDoElements());
//    this.setDoContents(o.getDoContents() && this.getDoContents());
    this.setDoContents(o.getDoContents() || this.getDoContents());
    this.setDoScripts(o.getDoScripts() || this.getDoScripts());
    this.setDoHeaders(o.getDoHeaders() || this.getDoHeaders());
    this.setDoRedirects(o.getDoRedirects() || this.getDoRedirects());
    this.setDoLinks(o.getDoLinks() || this.getDoLinks());
    if (this.getCreated() == -1 || o.getCreated() < this.getCreated()) this.setCreated(o.getCreated());
}
YaripPageExtensionItem.prototype.getPage = function()
{
    return this.getPageById(this.getKey());
}
YaripPageExtensionItem.prototype.compare = function(b)
{
    var aPage = this.getPage();
    var bPage = b.getPage();
    return aPage.compare(bPage);
}
YaripPageExtensionItem.prototype.generateXml = function()
{
    if (!this.value) return "";
    return "\t\t\t\t" +
        "<item" +
            " doElements=\"" + this.doElements + "\"" +
            " doContents=\"" + this.doContents + "\"" +
            " doScripts=\"" + this.doScripts + "\"" +
            " doHeaders=\"" + this.doHeaders + "\"" +
            " doRedirects=\"" + this.doRedirects + "\"" +
            " doLinks=\"" + this.doLinks + "\"" +
            (this.created > -1 ? " created=\"" + this.created + "\"" : "") +
        ">" + this.value + "</item>\n";
}
YaripPageExtensionItem.prototype.loadFromObject = function(obj)
{
    this.setValue(obj.value);
    this.setDoElements(obj.doElements);
    this.setDoContents(obj.doContents);
    this.setDoScripts(obj.doScripts);
    this.setDoHeaders(obj.doHeaders);
    this.setDoRedirects(obj.doRedirects);
    this.setDoLinks(obj.doLinks);
    this.setCreated(obj.created);
}

