
/*
Copyright 2007-2011 Kim A. Brandt <kimabrandt@gmx.de>

This file is part of yarip.

Yarip is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Yarip is distributed in the hope that it will be useful;
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with yarip; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

const EXPORTED_SYMBOLS = ["Yarip"];

const Cc = Components.classes;
const Ci = Components.interfaces;
const Cr = Components.results;
const Cu = Components.utils;

Cu.import("resource://gre/modules/XPCOMUtils.jsm");
Cu.import("resource://yarip/constants.jsm");
Cu.import("resource://yarip/uri.jsm");

function Yarip() {
}
Yarip.prototype = {
    classDescription: "Yet Another Remove It Permanently",
    classID: Components.ID("{46d09504-6a0b-40c8-8ce8-d07cf370d25e}"),
    contractID: "@yarip.mozdev.org;1",
    _xpcom_categories: [],
    QueryInterface: XPCOMUtils.generateQI([
        Ci.nsISupports,
        Ci.nsIObserver,
        Ci.nsIWebProgressListener,
        Ci.nsISupportsWeakReference
    ]),

    mode: null,
    enabled: null,
    noFlicker: null,

    monitorDialogues: [],
    pageDialog: null,

    alwaysUseIndex: false,
    elementsInContext: 4,
    exclusiveOnCreation: false,
    templates: [],
    privateBrowsing: false,
    purgeInnerHTML: false,
    schemesRegExp: null,
    //contentRepeatThreshold: 10000,
    contentRecurrence: false,
    logWhenClosed: false,
    isMobile: false,

    map: null,
    undoObj: {},
    pageCreated: false,
    knownAddressObj: {},

    /*
     * nsIObserver
     */

    observe: function(subject, topic, data)
    {
        switch (topic) {
        case "quit-application":
            this.save();
            break;
        }
    },

    getLocationFromLocation: function(location)
    {
        if (!location) return null;
        if (location.isLocation) return location;

        // https://developer.mozilla.org/en/DOM/window.location
        var host = "";
        var asciiHost = "";
    //    var hostname = "";
        var port = location.port;
        var asciiHref = "";
        var pageName = "";
        try {
            host = location.host.replace(/\.+(:\d+)?$/, "$1");
            asciiHost = IDNS.convertUTF8toACE(host).replace(/\.+(:\d+)?$/, "$1");
    //        hostname = IDNS.convertUTF8toACE(location.hostname);
    //        asciiHref = location.protocol + "//" + asciiHost + location.pathname;
            pageName = location.protocol + "//" + asciiHost + location.pathname;
    //        asciiHref = location.protocol + "//" + asciiHost + location.pathname + location.search + location.hash;
            asciiHref = pageName + location.search + location.hash;
    //        asciiHref = IDNS.convertUTF8toACE(location.href);
        } catch(e) {}
        return {
    //        hash: null,
            host: host,
    //        hostname: null,
    //        href: protocol + "//" + host + pathname, // no need for search nor hash!
    //        href: location.href,
            href: location.protocol + "//" + host + location.pathname + location.search + location.hash,
            pathname: location.pathname,
            port: port,
            protocol: location.protocol,
    //        search: null,
            asciiHost: asciiHost,
            asciiHref: asciiHref,
            pageName: pageName.replace(/[/]*$/, ""),
            isLocation: true
        }
    },

    getLocationFromContentLocation: function(contentLocation)
    {
        if (!contentLocation) return null;
        if (contentLocation.isLocation) return contentLocation;

        // https://developer.mozilla.org/en/nsIURI
        // https://developer.mozilla.org/en/DOM/window.location
        var host = "";
        try { host = contentLocation.hostPort.replace(/\.+(:\d+)?$/, "$1"); } catch (e) {}
    //    try { host = contentLocation.host; } catch (e) {}
        var pathname = contentLocation.path.replace(/[?&#].*$/, "");
        var port = -1;
        try { port = contentLocation.port; } catch (e) {}
        var protocol = contentLocation.scheme + ":";
        var asciiHost = "";
        try { asciiHost = contentLocation.asciiHost.replace(/\.+$/, "") + (port > -1 && port != 80 ? ":" + port : ""); } catch (e) {}
        var pageName = "";
        return {
    //        hash: null,
            host: host,
    //        hostname: null,
            href: contentLocation.spec,
            pathname: pathname,
            port: port,
            protocol: protocol,
    //        search: null,
    //        asciiHost: IDNS.convertUTF8toACE(host),
            asciiHost: asciiHost,
            asciiHref: contentLocation.asciiSpec,
            pageName: protocol + "//" + asciiHost + pathname.replace(/[/]*$/, ""),
            isLocation: true
        }
    },

    getContentLocationFromContentLocation: function(contentLocation)
    {
        if (!contentLocation) return null;
        if (contentLocation.isContentLocation) return contentLocation;

        // https://developer.mozilla.org/en/nsIURI
        var port = -1;
        var host = "";
        var asciiHost = "";
        var hostPort = "";
        try { port = contentLocation.port; } catch (e) {}
        var userPass = contentLocation.userPass;
        try { host = contentLocation.host.replace(/\.+$/, ""); } catch (e) {}
        try { asciiHost = contentLocation.asciiHost.replace(/\.+$/, ""); } catch (e) {}
        try { hostPort = contentLocation.hostPort.replace(/\.+(:\d+)?$/, "$1"); } catch (e) {}
        return {
            scheme: contentLocation.scheme,
            port: port,
            userPass: userPass,
            host: host,
            asciiHost: asciiHost,
            hostPort: hostPort,
            path: contentLocation.path,
    //        spec: contentLocation.spec,
            spec: contentLocation.scheme + "://" + (userPass ? userPass + "@" : "") + host + (port > -1 && port != 80 ? ":" + port : "") + contentLocation.path,
    //        asciiSpec: contentLocation.asciiSpec,
            asciiSpec: contentLocation.scheme + "://" + (userPass ? userPass + "@" : "") + asciiHost + (port > -1 && port != 80 ? ":" + port : "") + contentLocation.path,
            isContentLocation: true
        }
    },

    getPageName: function(location, mode)
    {
    //    if (!location || !/^https?:$/.test(location.protocol)) return null;
        if (!location || !this.schemesRegExp.test(location.protocol.replace(/:$/, ""))) return null;

        var mode = Number(mode);
        if (typeof mode != "number" || mode + "" == "NaN") {
            mode = this.mode;
        }

        location = this.getLocationFromLocation(location);
        var address = null;
        try
        {
            switch (mode) {
            case MODE_PAGE:
    //            address = location.protocol + "//" + location.host + location.pathname.replace(/[/]*$/, "");
                address = location.protocol + "//" + location.asciiHost + location.pathname.replace(/[/]*$/, "");
    //            address = location.asciiHref.replace(/[/]*$/, "");
                break;
            case MODE_FQDN:
                address = location.asciiHost;
                break;
            case MODE_SLD:
                var sld = null;
                sld = location.asciiHost.match(FIND_SLD_RE);
                if (sld) address = sld[0];
                break;
            }
            if (!address) return null;
        } catch (e) {}
        return address;
    },

    init: function()
    {
        Cu.import("resource://yarip/map.jsm");
        Cu.import("resource://yarip/page.jsm");
        Cu.import("resource://yarip/list.jsm");
        Cu.import("resource://yarip/item.jsm");
        Cu.import("resource://yarip/replace.jsm");

        this.map = new YaripMap();
        var file = this.getFile(FILE);
        this.load(file);
        this.check();

        this.initPreferences();
        if (!this.isMobile) this.initContentPolicy();

        // DEBUG Profile functions.
//        this.profile("this.getAddressObj", this, "getAddressObj");
//        for (var f in this) {
//            if (typeof this[f] == "function") {
//                this.profile("yarip." + f, this, f);
//            }
//        }
    },

    initMobile: function()
    {
        this.isMobile = true;

        Cu.import("resource://yarip/map.jsm");
        Cu.import("resource://yarip/page.jsm");
        Cu.import("resource://yarip/list.jsm");
        Cu.import("resource://yarip/item.jsm");
        Cu.import("resource://yarip/replace.jsm");

        this.map = new YaripMap();

        this.initPreferences();
        this.initContentPolicy();
    },

    initPreferences: function()
    {
        this.toggleEnabled(this.getValue(PREF_ENABLED, true, DATA_TYPE_BOOLEAN));
        this.setMode(this.getValue(PREF_MODE, MODE_FQDN, DATA_TYPE_INTEGER));
        this.setAlwaysUseIndex(this.getValue(PREF_INDEX, false, DATA_TYPE_BOOLEAN));
        this.setElementsInContext(this.getValue(PREF_ELEMENTS, 4, DATA_TYPE_INTEGER));
        this.setPurgeInnerHTML(this.getValue(PREF_PURGE, false, DATA_TYPE_BOOLEAN));
        this.setExclusiveOnCreation(this.getValue(PREF_EXCLUSIVE, false, DATA_TYPE_BOOLEAN));
        this.setTemplates(this.getValue(PREF_TEMPLATES, "", DATA_TYPE_STRING));
        this.setSchemes(this.getValue(PREF_SCHEMES, "^https?$", DATA_TYPE_STRING));
        this.setPrivateBrowsing(this.getValue(PREF_PRIVATE, false, DATA_TYPE_BOOLEAN));
        this.setContentRecurrence(this.getValue(PREF_RECURRENCE, false, DATA_TYPE_BOOLEAN));
        this.toggleLogWhenClosed(this.getValue(PREF_LOG_WHEN_CLOSED, false, DATA_TYPE_BOOLEAN));
    },

    initContentPolicy: function()
    {
        const yaripContentPolicy = Cu.import("resource://yarip/contentPolicy.jsm", null).wrappedJSObject;
        const componentRegistrar = Components.manager.QueryInterface(Ci.nsIComponentRegistrar);
        if (componentRegistrar.isContractIDRegistered(yaripContentPolicy.contractID)) return;

        try {
            componentRegistrar.registerFactory(yaripContentPolicy.classID, yaripContentPolicy.classDescription, yaripContentPolicy.contractID, yaripContentPolicy);
        } catch (e) {
            Cu.reportError(e);
        }
        const categoryManager = XPCOMUtils.categoryManager;
        categoryManager.addCategoryEntry("content-policy", yaripContentPolicy.classDescription, yaripContentPolicy.contractID, false, true);
    }
}

Yarip.prototype.toggleEnabled = function(enabled)
{
    if (enabled == this.enabled) return false;

    if (enabled != true && enabled != false) enabled = !this.enabled;
    this.enabled = enabled;

    if (enabled) this.toggleNoFlicker(this.getValue(PREF_FLICKER, false, DATA_TYPE_BOOLEAN), null, true);
    else this.toggleNoFlicker(false, true, true);

    this.setValue(PREF_ENABLED, enabled, DATA_TYPE_BOOLEAN); // notify
    return true;
}

Yarip.prototype.toggleNoFlicker = function(noFlicker, dontNotify, force)
{
    if (noFlicker == this.noFlicker && !force) return false;

    if (noFlicker != true && noFlicker != false) noFlicker = !this.getValue(PREF_FLICKER, false, DATA_TYPE_BOOLEAN);
    this.noFlicker = noFlicker;

    this.injectCSS("chrome://yarip/skin/noflicker.css", this.noFlicker && this.enabled);
    if (!dontNotify) this.setValue(PREF_FLICKER, this.noFlicker, DATA_TYPE_BOOLEAN); // notify
    return true;
}

Yarip.prototype.setMode = function(mode)
{
    var mode = Number(mode);
    if (typeof mode != "number" || mode + "" == "NaN") return false;
    this.mode = mode;
    this.setValue(PREF_MODE, mode, DATA_TYPE_INTEGER); // notify
    return true;
}

Yarip.prototype.setAlwaysUseIndex = function(value)
{
    if (value == true || value == false) this.alwaysUseIndex = value;
}

Yarip.prototype.setElementsInContext = function(value)
{
    var value = Number(value);
    if (typeof value != "number" || value + "" == "NaN") return;
    if (value > 0 && value <= 20) this.elementsInContext = value;
}

Yarip.prototype.setPurgeInnerHTML = function(value)
{
    if (value == true || value == false) this.purgeInnerHTML = value;
}

Yarip.prototype.setExclusiveOnCreation = function(value)
{
    if (value == true || value == false) this.exclusiveOnCreation = value;
}

Yarip.prototype.setTemplates = function(value)
{
    this.templates = [];
    if (!value) return;

    this.templates = value.split(" ", 50); // 50 = max templates
}

Yarip.prototype.setSchemes = function(value)
{
    try {
        this.schemesRegExp = new RegExp(value);
    } catch (e) {}
}

Yarip.prototype.setPrivateBrowsing = function(value)
{
    if (value == true || value == false) this.privateBrowsing = value;
}

//yarip.setContentRepeatThreshold = function(value)
//{
//    var value = Number(value);
//    if (typeof value != "number" || value + "" == "NaN") return;
//    this.contentRepeatThreshold = value > 0 ? value : 0;
//}

Yarip.prototype.setContentRecurrence = function(value)
{
    if (value == true || value == false) this.contentRecurrence = value;
}

Yarip.prototype.toggleLogWhenClosed = function(logWhenClosed)
{
    if (logWhenClosed == this.logWhenClosed) return false;
    if (logWhenClosed != true && logWhenClosed != false) {
        logWhenClosed = !this.getValue(PREF_LOG_WHEN_CLOSED, false, DATA_TYPE_BOOLEAN);
    }
    this.logWhenClosed = logWhenClosed;

    /*if (!dontNotify)*/ this.setValue(PREF_LOG_WHEN_CLOSED, this.logWhenClosed, DATA_TYPE_BOOLEAN); // notify
    return true;
}

Yarip.prototype.getPageRegExp = function(pageName, protocol)
{
    if (!pageName) return "^https?://([^/?#]+\\.)?example\\.net[/?#]";

    protocol = !protocol || /^https?:$/.test(protocol) ? "https?:" : protocol;
    var domain = pageName.replace(/^(([^:/?#]+):\/+)?|[/?#].*$/g, "").match(DOMAIN_RE);
    if (domain) {
        return "^" + protocol + "//" + (!domain[3] /* not IP */ ? "([^/?#]+\\.)?" : "") + domain[0].replace(/([.*+?|()\[\]{}\\])/g, "\\$1") + "[/?#]";
    } else {
        return "^https?://([^/?#]+\\.)?example\\.net[/?#]";
    }
}

Yarip.prototype.createPage = function(location, pageName, privateBrowsing)
{
    if (!location) return null;

    location = this.getLocationFromLocation(location);
    if (!pageName) pageName = this.getPageName(location, this.mode);

    // Creating the page.
    var page = this.map.get(pageName);
    if (page) {
        page.setTemporary(false);
        return page;
    } else {
        page = new YaripPage(this.getId(), pageName);
        if (privateBrowsing) page.setTemporary(true);
    }

    // Adding the default WhitelistContent.
    if (this.exclusiveOnCreation || privateBrowsing) {
        var sld = location.asciiHost.match(FIND_SLD_RE);
        if (sld) {
            var re = this.getPageRegExp(sld[0], location.protocol);
            var content = new YaripContentWhitelistItem(re);
            page.contentWhitelist.add(content);
            page.contentWhitelist.setExclusive(true);
        }
    }

    // Merging with templates.
    var addressObj = this.getAddressObj(pageName, false);
    if (!addressObj.found && this.templates.length > 0) {
        for (var i = 0; i < this.templates.length; i++) {
            page.merge(this.map.get(this.templates[i]));
        }
    }

    // Subtracting duplicate entries.
//    for (var a in addressObj.obj) {
//        page.subtract(this.map.get(a));
//    }

    this.map.add(page);
    this.pageCreated = true;
    this.reloadPage(null); // reset known
    return page;
}

Yarip.prototype.hasAddress = function(reduceDomain)
{
    return this.getFirstAddress(reduceDomain) != null;
}

Yarip.prototype.getFirstAddress = function(reduceDomain)
{
    if (!reduceDomain) return null;

    reduceDomain = reduceDomain.replace(/[?&#].*$/, "");
    var tmpDomain = null;
    while (reduceDomain != tmpDomain)
    {
        var reducePath = reduceDomain;
        var tmpPath = null;
        while (reducePath != tmpPath)
        {
            var page = this.map.get(reducePath);
            if (page) return reducePath;

            tmpPath = reducePath;
//            reducePath = reducePath.replace(/((:\d+)?\/|[^/]*)$/, "");
//            reducePath = reducePath.replace(/(\/|[^/]*)$/, "");
            reducePath = reducePath.replace(/(\/|[^/]*)$/, "");
        }
        tmpDomain = reduceDomain;
        reduceDomain = reduceDomain.replace(/^([^:]+:\/\/|[\w-~!$&'()*+,;=:@]+\.)|:\d+$/, "");
    }

    return null;
}

Yarip.prototype.getAddressMap = function(reduceDomain, follow)
{
    var map = new YaripMap();
    if (typeof reduceDomain == "string") {
        var addressObj = this.getAddressObj(reduceDomain, follow);
        for (var a in addressObj.obj) {
            map.add(this.map.get(a).clone());
        }
    } else { // instanceof Array
        for (var i = 0; i < reduceDomain.length; i++) {
            var addressObj = this.getAddressObj(reduceDomain[i], follow);
            for (var a in addressObj.obj) {
                map.add(this.map.get(a).clone());
            }
        }
    }
    return map;
}

Yarip.prototype.getAddressObj = function(origReduceDomain, follow, reuse)
{
    if (!origReduceDomain) {
        return {
            obj: {},
            exclusive: false,
            found: false
        };
    }

    var addressObj = null;
    var key = null;
    if (!this.contentRecurrence) {
        key = !!follow + " " + origReduceDomain;
        addressObj = reuse && key in this.knownAddressObj ? this.knownAddressObj[key] : null;
//        addressObj.time = time;
        if (addressObj) return addressObj;
    }

    addressObj = {
        obj: {},
        exclusive: false,
        found: false
//        time: time
    }
    var reduceDomain = origReduceDomain.replace(/[?&#].*$/, "");
    var tmpDomain = null;
    while (reduceDomain != tmpDomain)
    {
        var reducePath = reduceDomain;
        var tmpPath = null;
        while (reducePath != tmpPath)
        {
            var page = this.map.get(reducePath);
            if (page) {
                addressObj.obj[reducePath] = [true, true, true, true, true, true, true]; // [isSelf, doElements, doContents, doScripts, doHeaders, doRedirects, doLinks]
                addressObj.found = true;
                if (page.contentWhitelist.getExclusive()) {
                    addressObj.exclusive = true;
                }
            }
            tmpPath = reducePath;
            reducePath = reducePath.replace(/(\/|[^/]*)$/, "");
        }
        tmpDomain = reduceDomain;
        reduceDomain = reduceDomain.replace(/^([^:]+:\/\/|[\w-~!$&'()*+,;=:@]+\.)|:\d+$/, "");
    }

    if (follow) {
        this.getExtensionAddressObj(addressObj, true);
    }

    if (!this.contentRecurrence && addressObj.found) {
        this.knownAddressObj[key] = addressObj;
    }

    return addressObj;
}

Yarip.prototype.getAddressObjByLocation = function(location, follow, reuse)
{
//    var pageName = this.getPageName(location, MODE_PAGE);
    var pageName = location ? location.pageName : null;
    if (/*!location ||*/ !pageName) {
        return {
            obj: {},
            exclusive: false,
            found: false,
            pageName: null
        };
    }

    var addressObj = this.getAddressObj(pageName, follow, reuse);
    if (!addressObj.found && this.privateBrowsing) {
        var page = this.createPage(location, null, true);
        this.reloadPage(page.getName());
        addressObj = this.getAddressObj(pageName, follow, reuse);
    }

    addressObj.pageName = pageName;
    return addressObj;
}

Yarip.prototype.getExtensionAddressObj = function(addressObj, append)
{
    if (append && !addressObj) return null;

    if (append) {
        for (var a in addressObj.obj) {
            this.getRecursiveAddressArray(a, addressObj, addressObj.obj[a]);
        }
    } else {
        var aTmp = {
            obj: {},
            exclusive: false,
            found: false
        }
        for (var a in addressObj.obj) {
            this.getRecursiveAddressArray(a, aTmp, addressObj.obj[a]);
        }
        return aTmp;
    }

    return null;
}

Yarip.prototype.getRecursiveAddressArray = function(parentAddress, addressObj, extMask)
{
    if (!extMask[1] && !extMask[2] && !extMask[3] && !extMask[4] && !extMask[5] && !extMask[6]) return;

    var extObj = this.map.get(parentAddress).pageExtensionList.obj;
    for (var id in extObj)
    {
        var extPage = this.map.getById(id);
        if (extPage)
        {
            var extAddress = extPage.getName();
            var extension = extObj[id];
            if (extAddress in addressObj.obj) { // already added
                if (extMask[1]) addressObj.obj[extAddress][1] = addressObj.obj[extAddress][1] || extension.getDoElements();
                if (extMask[2]) addressObj.obj[extAddress][2] = addressObj.obj[extAddress][2] || extension.getDoContents();
                if (extMask[3]) addressObj.obj[extAddress][3] = addressObj.obj[extAddress][3] || extension.getDoScripts();
                if (extMask[4]) addressObj.obj[extAddress][4] = addressObj.obj[extAddress][4] || extension.getDoHeaders();
                if (extMask[5]) addressObj.obj[extAddress][5] = addressObj.obj[extAddress][5] || extension.getDoRedirects();
                if (extMask[6]) addressObj.obj[extAddress][6] = addressObj.obj[extAddress][6] || extension.getDoLinks();
            } else { // not added
                addressObj.obj[extAddress] = [
                    false, // extended (not self)
                    extMask[1] && extension.getDoElements(),
                    extMask[2] && extension.getDoContents(),
                    extMask[3] && extension.getDoScripts(),
                    extMask[4] && extension.getDoHeaders(),
                    extMask[5] && extension.getDoRedirects(),
                    extMask[6] && extension.getDoLinks()];
                addressObj.found = true;
                if (extPage.pageExtensionList.length > 0) {
                    this.getRecursiveAddressArray(extAddress, addressObj, addressObj.obj[extAddress]);
                }
            }
        } else {
            this.map.get(parentAddress).pageExtensionList.removeByKey(id);
        }
    }
}

Yarip.prototype.resetUndo = function()
{
    this.undoObj = {};
}

Yarip.prototype.resetOnAddress = function(obj)
{
    if (!obj) return false;

    var list = null;

    switch (obj.type) {
    case TYPE_WHITELIST:
        list = this.map.get(obj.address) ? this.map.get(obj.address).elementWhitelist : null;
        break;
    case TYPE_BLACKLIST:
        list = this.map.get(obj.address) ? this.map.get(obj.address).elementBlacklist : null;
        break;
    case TYPE_ATTRIBUTE:
        list = this.map.get(obj.address) ? this.map.get(obj.address).elementAttributeList : null;
        break;
    }

    switch (obj.type) {
    case TYPE_WHITELIST:
    case TYPE_BLACKLIST:
        var elements = this.getElementsByXPath(obj.document, obj.xValue);
        if (elements)
        {
            for (var i = 0; i < elements.snapshotLength; i++)
            {
                var element = elements.snapshotItem(i);
                if (element.nodeType != 1) continue;

                switch (element.getAttribute("status")) {
                case "whitelisted":
                case "whitelisted forced":
                case "blacklisted":
                case "placeholder blacklisted":
                case "placeholder":
                    element.removeAttribute("status");
                    break;
                }
            }
        }
        break;
    }

    if (list) {
        list.removeByKey(obj.key);
        this.reloadPage(obj.address, false, true);
    }
    delete this.undoObj[obj.type + "-" + obj.key];
    return true;
}

Yarip.prototype.getIncrement = function(newIncrement, oldIncrement)
{
    return newIncrement > oldIncrement ? newIncrement : oldIncrement;
}

Yarip.prototype.reloadPage = function(pageName, selectItem, selectTab, reset)
{
    this.knownAddressObj = {};
    var refresh = this.shouldRefreshPage(pageName);
    if (this.pageDialog && pageName) {
        this.pageDialog.reloadPage(pageName, this.pageCreated || refresh, selectItem, selectTab, reset);
//        this.pageDialog.reloadPage(pageName, this.pageCreated, selectItem, selectTab);
        this.pageCreated = false;
        return true;
    } else {
        return false;
    }
}

Yarip.prototype.shouldRefreshPage = function(pageName)
{
    if (!pageName) return false;

    var page = this.map.get(pageName);
    if (!page) return false;
//    if (page.isEmpty()) {
//        this.map.remove(page);
//        return true;
//    } else {
//        return false;
//    }
    else return true;
}

Yarip.prototype.removeAllExceptWhitelisted = function(doc)
{
    this.whitelistXPath(doc, null, "/html/head/descendant-or-self::*[not(@status='blacklisted')]"); // whitelist head and descendants
//    this.blacklistXPath(doc, null, "//*[not(@status='whitelisted')]"); // remove all except whitelisted
    this.blacklistXPath(doc, null, "//*[not(@status='whitelisted') and not(@status='whitelisted forced')]"); // remove all except whitelisted
}

//yarip.whitelistXPath = function(doc, pageName, xValue, isNew, xpath, doIncrement)
Yarip.prototype.whitelistXPath = function(doc, pageName, xValue, isNew, force, xpath, doIncrement)
{
//    if (!doc || !pageName || !xValue) return false;
    if (!doc || !xValue) return false;

    var elements = this.getElementsByXPath(doc, xValue);
    if (!elements) return false;

//    var page = this.map.get(pageName);
//    var elementWhitelist = page ? page.elementWhitelist : null;
    var found = false;
    var increment = INCREMENT_NOT_FOUND;

    for (var i = 0; i < elements.snapshotLength; i++)
    {
        var element = elements.snapshotItem(i);

        switch (element.nodeType) {
        case 1: // ELEMENT
            switch (element.getAttribute("status")) {
            case "whitelisted":
            case "whitelisted forced":
            case "blacklisted":
//            case "placeholder blacklisted":
            case "placeholder":
                increment = this.getIncrement(INCREMENT_IGNORED, increment);
                continue;
            }

            if (isNew) {
                element.setAttribute("status", force ? "whitelisted forced" : "whitelisted");
            } else {
//                while (element && element.nodeType == 1 && element.getAttribute("status") != "whitelisted" && element.getAttribute("status") != "whitelisted forced") {
                while (element && element.nodeType == 1 && !/^whitelisted(\s+forced)?$/.test(element.getAttribute("status"))) {
                    element.setAttribute("status", force ? "whitelisted forced" : "whitelisted");
                    element = element.parentNode;
                }
            }
            break;
        default:
            continue;
        }

        increment = INCREMENT_FOUND;
        found = true;
    }

    if (!pageName) return found;

    var page = this.map.get(pageName);
    var elementWhitelist = page ? page.elementWhitelist : null;

    if (doIncrement && elementWhitelist && elementWhitelist.obj[xValue])
    {
        switch (increment) {
        case INCREMENT_FOUND:
            elementWhitelist.obj[xValue].incrementFound();
            break;
        case INCREMENT_NOT_FOUND:
            elementWhitelist.obj[xValue].incrementNotFound();
            break;
        case INCREMENT_IGNORED:
            elementWhitelist.obj[xValue].incrementIgnored();
            break;
        }
    }

    if (isNew && found)
    {
        var xpath = xpath ? xpath : new YaripElementWhitelistItem(xValue);
        if (page) {
            page.setTemporary(false);
        } else {
            page = this.createPage(doc.location, pageName);
        }
        page.elementWhitelist.add(xpath);
        this.reloadPage(pageName, false, true);

        var obj = {};
        obj.document = doc;
        obj.address = pageName;
        obj.xValue = xValue;
        obj.key = xpath.getKey();
        obj.type = TYPE_WHITELIST;
        obj.text = xValue;
        this.undoObj[obj.type + "-" + obj.key] = obj;
    }

    return found;
}

Yarip.prototype.blacklistXPath = function(doc, pageName, xValue, isNew, force, placeholder, xpath, doIncrement)
{
//    if (!doc || !pageName || !xValue) return false;
    if (!doc || !xValue) return false;

    var elements = this.getElementsByXPath(doc, xValue);
    if (!elements) return false;

    var found = false;
    var increment = INCREMENT_NOT_FOUND;
    var useForce = true;

    for (var i = 0; i < elements.snapshotLength; i++)
    {
        var element = elements.snapshotItem(i);
        switch (element.nodeType) {
        case 1: // ELEMENT
            switch (element.getAttribute("status")) {
            case "whitelisted":
                if (force) {
                  useForce = false; // don't force removal
                  break;
                } else {
                  increment = this.getIncrement(INCREMENT_IGNORED, increment);
                  continue;
                }
            case "blacklisted":
            case "placeholder":
            case "whitelisted forced":
              increment = this.getIncrement(INCREMENT_IGNORED, increment);
              continue;
//            case "placeholder blacklisted":
////                if (placeholder) continue;
//                if ((!force || doIncrement) /* not temporary */ && placeholder) continue;
//                break;
            }

            switch (element.localName) {
            case "html":
            case "body":
            case "frameset":
                continue;
            case "head":
                element.innerHTML =
                    "<style rel='stylesheet' type='text/css'>\n" +
                        "\t* {\n" +
                            "\t\tmargin: 0 !important;\n" +
                            "\t\tpadding: 0 !important;\n" +
                        "\t}\n" +
                        "\tdiv {\n" +
                            "\t\theight: auto !important;\n" +
                        "\t}\n" +
                        "\ttable {\n" +
                            "\t\tborder-spacing: 0 0 !important;\n" +
                            "\t\theight: auto !important;\n" +
                            "\t\twidth: 0 !important;\n" +
                        "\t}\n" +
                    "</style>";
                break;
            case "script":
                element.removeAttribute("src");
                element.innerHTML = "";
                break;
            case "frame":
                if (element.parentNode.localName == "FRAMESET") {
                    continue;
                } else {
                    element.setAttribute("src", "");
                    element.setAttribute("status", "blacklisted");
                }
                break;
            case "br":
                element.setAttribute("status", "blacklisted");
                break;
            case "audio":
            case "embed":
            case "iframe":
            case "video":
//                if (placeholder) {
                if ((!force || doIncrement) /* not temporary */ && placeholder) {
                    element.setAttribute("status", "placeholder");
                } else {
                    element.setAttribute("src", "");
                    element.setAttribute("status", "blacklisted");
                }
                break;
            default:
//                element.setAttribute("status", placeholder ? "placeholder" : "blacklisted");
//                if (this.purgeInnerHTML && !isNew && !placeholder) {
//                    element.innerHTML = "";
//                }
//                if (placeholder) {
                if ((!force || doIncrement) /* not temporary */ && placeholder) {
                    element.setAttribute("status", "placeholder");
                } else {
                    element.setAttribute("status", "blacklisted");
                    if (this.purgeInnerHTML && !isNew) {
                        element.innerHTML = "";
                    }
                }
                break;
            }
            break;
        case 2: // ATTRIBUTE
            if (element.ownerElement) element.ownerElement.removeAttribute(element.name);
            break;
        case 3: // TEXT
            if (element.parentNode) element.parentNode.removeChild(element);
            break;
        default:
            continue;
        }

        increment = INCREMENT_FOUND;
        found = true;
    }

    if (!pageName) return found;

    var page = this.map.get(pageName);
    var elementBlacklist = page ? page.elementBlacklist : null;

    if (doIncrement && elementBlacklist && elementBlacklist.obj[xValue])
    {
        switch (increment) {
        case INCREMENT_FOUND:
            elementBlacklist.obj[xValue].incrementFound();
            break;
        case INCREMENT_NOT_FOUND:
            elementBlacklist.obj[xValue].incrementNotFound();
            break;
        case INCREMENT_IGNORED:
            elementBlacklist.obj[xValue].incrementIgnored();
            break;
        }
    }

    if (isNew && found)
    {
        var xpath = xpath ? xpath : new YaripElementBlacklistItem(xValue, true, useForce, null, placeholder);
        if (page) {
            page.setTemporary(false);
        } else {
            page = this.createPage(doc.location, pageName);
        }
        page.elementBlacklist.add(xpath);
        this.reloadPage(pageName, false, true);

        var obj = {};
        obj.document = doc;
        obj.address = pageName;
        obj.xValue = xValue;
        obj.key = xpath.getKey();
        obj.type = TYPE_BLACKLIST;
        obj.text = xValue;
        this.undoObj[obj.type + "-" + obj.key] = obj;
    }

    return found;
}

Yarip.prototype.whitelistContent = function(pageName, regExpValue, content, location)
{
    if (!pageName || !regExpValue) return false;

    var tmpContent = content ? content : new YaripContentWhitelistItem(regExpValue);
    var page = this.map.get(pageName);
    if (page) {
        page.setTemporary(false);
    } else {
        page = this.createPage(location, pageName);
    }
    page.contentWhitelist.add(tmpContent);
    this.reloadPage(pageName, false, true);
    return true;
}

Yarip.prototype.blacklistContent = function(pageName, regExpValue, content, location)
{
    if (!pageName || !regExpValue) return false;

    var tmpContent = content ? content : new YaripContentBlacklistItem(regExpValue);
    var page = this.map.get(pageName);
    if (page) {
        page.setTemporary(false);
    } else {
        page = this.createPage(location, pageName);
    }
    page.contentBlacklist.add(tmpContent);
    this.reloadPage(pageName, false, true);
    return true;
}

//yarip.extendPage = function(pageName, contentLocation, contentAddress)
Yarip.prototype.extendPage = function(pageLocation, pageName, contentLocation, contentAddress)
{
    if (!pageName || !contentAddress) return false;

//    var pageOwn = this.map.get(pageName);
//    if (!pageOwn) return false;
    var pageOwn = this.createPage(pageLocation, pageName);
    var pageExt = this.createPage(contentLocation, contentAddress);
//    pageOwn.pageExtensionList.add(new YaripPageExtensionItem(pageExt.getId(), false, true, false, false, false, false)); // contents only
    pageOwn.pageExtensionList.add(new YaripPageExtensionItem(pageExt.getId(), false, true, false, false, false, true)); // contents and links
//    TODO Use getExtensionAddressObj to set doElements, doContents, ...!
//    var doElements = pageExt.hasElements();
//    var doContents = pageExt.hasContents();
//    var doScripts = pageExt.hasScripts();
//    var doHeaders = pageExt.hasHeaders();
//    var doRedirects = pageExt.hasRedirects();
//    var doLinks = pageExt.hasLinks();
//    var doSomething = doElements || doContents || doScripts || doHeaders || doRedirects || doLinks;
//    pageOwn.pageExtensionList.add(new YaripPageExtensionItem(pageExt.getId(), doElements, doSomething ? doContents : true, doScripts, doHeaders, doRedirects, doSomething ? doLinks : true));
    pageOwn.pageExtensionList.removeByKey(pageOwn.getId());
    pageOwn.setTemporary(false);
    this.reloadPage(pageExt.getName());
    this.reloadPage(pageOwn.getName());
    return true;
}

Yarip.prototype.styleElement = function(doc, pageName, xValue, attrName, attrValue, isNew, xpath, doIncrement)
{
    if (!doc || !pageName || !xValue || !attrName || !attrValue) return false;

    var elements = this.getElementsByXPath(doc, xValue);
    if (!elements) return false;

    var found = false;
    var increment = INCREMENT_NOT_FOUND;

    for (var i = 0; i < elements.snapshotLength; i++)
    {
        var element = elements.snapshotItem(i);
        switch (element.nodeType) {
        case 1: // ELEMENT
            switch (attrName) {
            case "style":
                var style = element.getAttribute(attrName);
                if (style) {
                    if (style.indexOf(attrValue) === -1) {
                        element.setAttribute(attrName, style + attrValue);
                    }
                } else {
                    element.setAttribute(attrName, attrValue);
                }
                break;
            default:
                element.setAttribute(attrName, attrValue);
                break;
            }
            break;
        default:
            continue;
        }

        increment = INCREMENT_FOUND;
        found = true;
    }

    var page = this.map.get(pageName);
    var list = page ? page.elementAttributeList : null;

    xpath = xpath ? xpath : new YaripElementAttributeItem(xValue, attrName, attrValue);
    var key = xpath.getKey();
    if (doIncrement && list && list.obj[key])
    {
        switch (increment) {
        case INCREMENT_FOUND:
            list.obj[key].incrementFound();
            break;
        case INCREMENT_NOT_FOUND:
            list.obj[key].incrementNotFound();
            break;
        }
    }

    if (isNew && found)
    {
        if (page) {
            page.setTemporary(false);
        } else {
            page = this.createPage(doc.location, pageName);
        }
        page.elementAttributeList.add(xpath);
        this.reloadPage(pageName, false, true);

        var obj = {};
        obj.document = doc;
        obj.address = pageName;
        obj.key = xpath.getKey();
        obj.type = TYPE_ATTRIBUTE;
        obj.text = attrName + ": " + attrValue;
        this.undoObj[obj.type + "-" + obj.key] = obj;
    }

    return found;
}

Yarip.prototype.scriptElement = function(doc, pageName, xValue, isNew, xpath)
{
    if (!doc || !pageName || !xValue || !xpath) return false;

    var elements = this.getElementsByXPath(doc, xValue);
    if (!elements) return false;

    var page = this.map.get(pageName);

    if (isNew)
    {
//        xpath = xpath ? xpath : new YaripElementScriptItem(xValue,
//                    "function (array) {\n" +
//                    "    // Do something with the elements in the array!\n" +
//                    "}");
        if (page) {
            page.setTemporary(false);
        } else {
            page = this.createPage(doc.location, pageName);
        }
        page.elementScriptList.add(xpath);
        this.reloadPage(pageName, false, true);
    }

    return true;
}

Yarip.prototype.calculateXPath = function(element, fullPath)
{
    if (!element || !element.localName) return null;

    if (element.nodeType == 2) {
        return this.calculateXPath(element.ownerElement, fullPath) + "/@" + element.localName; // attribute
    }

    var xValue = "";
    do
    {
        var attributes = "";
        if (element.getAttribute("id"))
        {
            // ID
            attributes += "[@id='" + element.getAttribute("id") + "']";
            if (!fullPath) {
                return "//" + element.localName + attributes + xValue; // shortest xpath
            }
        }
        else
        {
            // ATTRIBUTES
            if (element.getAttribute("name")) attributes += "@name='" + element.getAttribute("name") + "' and ";
            if (element.getAttribute("class")) attributes += "@class='" + element.getAttribute("class") + "' and ";
            if (attributes != "") attributes = "[" + attributes.substring(0, attributes.length - 5) + "]";

            // INDEX
            if (!/^(html|head|title|body|thead|tbody|tfoot|frameset)$/i.test(element.localName))
            {
                var index = 1;
                var sibling = element.previousSibling;
                while (sibling) {
                    if (sibling.localName == element.localName) ++index;
                    sibling = sibling.previousSibling;
                }

                var found = false;
                sibling = element.nextSibling;
                while (!found && sibling) {
                    if (sibling.localName == element.localName) found = true;
                    else sibling = sibling.nextSibling;
                }
                if (found || index > 1 || this.alwaysUseIndex) attributes = "[" + index + "]" + attributes;
            }
        }
        xValue = "/" + element.localName + attributes + xValue;
        element = element.parentNode;
    } while (element && element.nodeType == 1);

    return xValue != "" ? xValue : null;
}

Yarip.prototype.getElementsByXPath = function(doc, xValue)
{
    var elements = null;
    try {
        elements = doc.evaluate(xValue, doc.body, null, UNORDERED_NODE_SNAPSHOT_TYPE, null);
    } catch (e) {
        var scriptError = Cc['@mozilla.org/scripterror;1'].createInstance(Ci.nsIScriptError);
//        var message = doc.getElementById("yarip-overlay-stringbundle").getFormattedString("malformedXPath", [xValue]);
        var message = "[xpath=" + xValue + "] ";
//        var sourceName = "chrome://yarip/content/yarip.js";
        var sourceName = "";
        scriptError.init(message + e.message, sourceName, null, e.lineNumber, e.columnNumber, scriptError.errorFlag, "content javascript");
        CS.logMessage(scriptError);
    }
//    if (!elements || elements.snapshotLength == 0) return null;
    return elements;
}

Yarip.prototype.generateRegExp = function(value)
{
    if (!value) return null;
//    return "^" + value.replace(/([/.*+?|()\[\]{}\\])/g, "\\$1") + "$";
    return "^" + value.replace(/([.*+?|()\[\]{}\\])/g, "\\$1") + "$";
}

Yarip.prototype.checkPageName = function(value)
{
    return URI_SIMPLE_RE.test(value);
}

Yarip.prototype.checkXPath = function(xValue)
{
    if (!xValue) return false;
    try
    {
        var dp = Cc['@mozilla.org/xmlextras/domparser;1'].createInstance(Ci.nsIDOMParser);
        var doc = dp.parseFromString("<yarip></yarip>", "text/xml");
        doc.evaluate(xValue, doc, null, UNORDERED_NODE_SNAPSHOT_TYPE, null);
    }
    catch (e) { return false; }
    return true;
}

Yarip.prototype.checkRegExp = function(reValue)
{
    if (!reValue) return false;
    try { new RegExp(reValue).test(""); } catch (e) { return false; }
    return true;
}

Yarip.prototype.getElements = function(doc, xValue)
{
    var elements = null;
    try { elements = doc.evaluate(xValue, doc, null, UNORDERED_NODE_SNAPSHOT_TYPE, null); } catch (e) {}
    return elements;
}

Yarip.prototype.highlight = function(doc, xValue)
{
    if (!doc || !xValue) return false;
    var elements = this.getElements(doc, xValue);
    if (!elements) return false;
    if (elements.snapshotLength > 0) {
        for (var i = 0; i < elements.snapshotLength; i++) {
            var element = elements.snapshotItem(i);
            if (element.nodeType == 2 /* ATTRIBUTE */) element = element.ownerElement;
            if (element && element.getAttribute) {
                var status = element.getAttribute("status");
                if (!status) {
                    element.setAttribute("status", "highlighted");
                } else if (!/\bhighlighted\b/.test(status)) {
                    element.setAttribute("status", status + " highlighted");
                }
            }
        }
    }
    return true;
}

Yarip.prototype.unHighlight = function(doc, xValue)
{
    if (!doc || !xValue) return;
    var elements = this.getElements(doc, xValue);
    if (elements && elements.snapshotLength > 0) {
        for (var i = 0; i < elements.snapshotLength; i++) {
            var element = elements.snapshotItem(i);
            if (element.nodeType == 2 /* ATTRIBUTE */) element = element.ownerElement;
            if (element && element.getAttribute) {
                var status = element.getAttribute("status");
                if (status && /\bhighlighted\b/.test(status)) {
                    var newStatus = status.replace(/\s*highlighted\b/, "");
//                    if (newStatus == "") {
                    if (!newStatus) {
                        element.removeAttribute("status");
                    } else {
                        element.setAttribute("status", newStatus);
                    }
                }
            }

        }
    }
}

Yarip.prototype.save = function()
{
    var file = this.getFile(FILE);
    var data = this.map.generateXml();
    this.saveToFile(data, file);
}

Yarip.prototype.getValue = function(preference, defaultValue, type)
{
    try {
        switch (type) {
        case DATA_TYPE_BOOLEAN:
            return PB.getBoolPref(preference);
        case DATA_TYPE_INTEGER:
            return PB.getIntPref(preference);
        case DATA_TYPE_STRING:
            return PB.getCharPref(preference);
        }
    } catch (e) {
        if (defaultValue != false && (typeof defaultValue != "number" || defaultValue + "" == "NaN") && !defaultValue) {
            return null;
        }
        this.setValue(preference, defaultValue, type);
    }
    return this.getValue(preference, null, type);
}

Yarip.prototype.setValue = function(preference, value, type)
{
    try {
        switch (type) {
        case DATA_TYPE_BOOLEAN:
            PB.setBoolPref(preference, "" + value == "true");
            break;
        case DATA_TYPE_INTEGER:
            PB.setIntPref(preference, value);
            break;
        case DATA_TYPE_STRING:
            PB.setCharPref(preference, value);
            break;
        case DATA_TYPE_RESET:
            PB.clearUserPref(preference);
            break;
        }
    } catch (e) {}
}

Yarip.prototype.deleteBranch = function(branch)
{
    if (!branch) return;

    PB.deleteBranch(branch);
}

Yarip.prototype.injectCSS = function(uri, value)
{
    if (!uri) return;

    var uri = IOS.newURI(uri, CHARSET, null); // uri, charset, baseuri
    var sss = Cc["@mozilla.org/content/style-sheet-service;1"].
        getService(Ci.nsIStyleSheetService);
    if (value) { if (!sss.sheetRegistered(uri, sss.USER_SHEET)) sss.loadAndRegisterSheet(uri, sss.USER_SHEET); }
    else { if (sss.sheetRegistered(uri, sss.USER_SHEET)) sss.unregisterSheet(uri, sss.USER_SHEET); }
}

Yarip.prototype.check = function()
{
    var previous = this.getValue(PREF_VERSION, "", DATA_TYPE_STRING);
    if (previous && previous != VERSION)
    {
        this.setValue(PREF_VERSION, VERSION, DATA_TYPE_STRING);

        if (/^0\.(1\.[2-9]|2(\.1)?)$/.test(previous) && !/^0\.(1\.[2-9]|2(\.1)?)$/.test(VERSION)) {
            // CHANGED NOFLICKER FEATURE
            this.setValue("content.notify.ontimer", null, DATA_TYPE_RESET);
            this.setValue("content.notify.backoffcount", null, DATA_TYPE_RESET);
            this.setValue("nglayout.initialpaint.delay", null, DATA_TYPE_RESET);
        }

        if (/^0\.2(\.[1-3])?$/.test(previous) && !/^0\.2(\.[1-3])?$/.test(VERSION)) {
            Cc["@mozilla.org/satchel/form-history;1"].
                getService(Ci.nsIFormHistory2).
                removeEntriesForName("style");
        }

        if (/^0\.2\.4$/.test(previous) && !/^0\.2\.4$/.test(VERSION)) {
            this.setSchemes(this.getValue("extensions.yarip.schemesList.value", "^https?$", DATA_TYPE_STRING));
            this.deleteBranch("extensions.yarip.schemesList");
        }

        this.deleteBranch("extensions.yarip.changeType"); // >0.1.7
        this.deleteBranch("extensions.yarip.flickerType"); // >0.1.7
        this.deleteBranch("extensions.yarip.loggingEnabled"); // >0.2.1
        this.deleteBranch("extensions.yarip.deep"); // >0.2.2
        this.deleteBranch("extensions.yarip.profile"); // >0.2.2
        this.deleteBranch("extensions.yarip.reload"); // >0.2.2
        this.deleteBranch("extensions.yarip.firstrun"); // >0.2.4

        if (/^0\.(1\.[2-9]|2(\.[1-4])?)$/.test(previous) && !/^0\.(1\.[2-9]|2(\.[1-4])?)$/.test(VERSION)) {
            // TODO Remove the STYLE-file!
            //      -> "extensions/yarip@mozdev.org-dev/style.css"
        }
    }
}

Yarip.prototype.getId = function()
{
    var id = Date.now();
    while (this.map.getById(id) != undefined && this.map.getById(id) != null) id++;
    return "" + id;
}

Yarip.prototype.addMonitorDialog = function(monitorDialog)
{
    monitorDialog.id = Date.now();
    while (this.monitorDialogues["" + monitorDialog.id] != undefined) monitorDialog.id++;
    this.monitorDialogues["" + monitorDialog.id] = monitorDialog;
}

Yarip.prototype.removeMonitorDialog = function(monitorDialog)
{
    delete this.monitorDialogues["" + monitorDialog.id];
    monitorDialog.id = -1;
}

Yarip.prototype.load = function(file, imported)
{
    var doc = this.getDoc(file);
    if (!doc) {
        if (!imported) this.save();
        return null;
    } else if (doc.documentElement.nodeName == "parsererror") {
        return null;
    }

    var pageName = null;
    var map = new YaripMap();

    var iYarip = doc.evaluate("./yarip", doc, null, ORDERED_NODE_ITERATOR_TYPE, null);
    var nYarip = iYarip.iterateNext();
    if (!nYarip) return null;

    var version = nYarip.getAttribute("version");
    if (!version)
    {
        var iPage = doc.evaluate("./page", nYarip, null, ORDERED_NODE_ITERATOR_TYPE, null);
        var nPage = iPage.iterateNext();
        while (nPage)
        {
            pageName = nPage.getAttribute("name");
            var pageId = nPage.getAttribute("id");
            var page = map.obj[pageName] ? map.obj[pageName] : new YaripPage(pageId ? pageId : this.getId(), pageName);

            // XPATH WHITELIST
            var iWhitelist = doc.evaluate("./whitelist", nPage, null, ORDERED_NODE_ITERATOR_TYPE, null);
            var nWhitelist = iWhitelist.iterateNext();
            if (nWhitelist)
            {
                var iXPath = doc.evaluate("./xpath", nWhitelist, null, ORDERED_NODE_ITERATOR_TYPE, null);
                var nXPath = iXPath.iterateNext();
                while (nXPath)
                {
                    page.elementWhitelist.add(new YaripElementWhitelistItem(
                        nXPath.getAttribute("value"),
                        nXPath.getAttribute("deep"),
                        null, // force
                        pageId, // created
                        null, // lastFound
                        nXPath.getAttribute("found"),
                        nXPath.getAttribute("notFound"),
                        nXPath.getAttribute("whitelisted")));
                    nXPath = iXPath.iterateNext();
                }
            }

            // XPATH BLACKLIST
            var iBlacklist = doc.evaluate("./blacklist", nPage, null, ORDERED_NODE_ITERATOR_TYPE, null);
            var nBlacklist = iBlacklist.iterateNext();
            if (nBlacklist)
            {
                var iXPath = doc.evaluate("./xpath", nBlacklist, null, ORDERED_NODE_ITERATOR_TYPE, null);
                var nXPath = iXPath.iterateNext();
                while (nXPath)
                {
                    page.elementBlacklist.add(new YaripElementBlacklistItem(
                        nXPath.getAttribute("value"),
                        nXPath.getAttribute("deep"),
                        nXPath.getAttribute("force") == "true",
                        nXPath.getAttribute("dependsOnWhitelist"),
                        null, // placeholder
                        pageId, // created
                        null, // lastFound
                        nXPath.getAttribute("found"),
                        nXPath.getAttribute("notFound"),
                        nXPath.getAttribute("whitelisted")));
                    nXPath = iXPath.iterateNext();
                }
            }

            map.add(page);
            nPage = iPage.iterateNext();
        }
    }
    else if (/^0\.2(\.[1-3])?$/.test(version))
    {
        var iPage = doc.evaluate("./page", nYarip, null, ORDERED_NODE_ITERATOR_TYPE, null);
        var nPage = iPage.iterateNext();
        while (nPage)
        {
            pageName = nPage.getAttribute("name");
            var pageId = nPage.getAttribute("id");
            var page = map.obj[pageName] ? map.obj[pageName] : new YaripPage(pageId ? pageId : this.getId(), pageName);

            // ELEMENT BLACKLIST
            var iElementBlacklist = doc.evaluate("./elementBlacklist", nPage, null, ORDERED_NODE_ITERATOR_TYPE, null);
            var nElementBlacklist = iElementBlacklist.iterateNext();
            if (nElementBlacklist)
            {
                var iElement = doc.evaluate("./element", nElementBlacklist, null, ORDERED_NODE_ITERATOR_TYPE, null);
                var nElement = iElement.iterateNext();
                while (nElement)
                {
                    var element = new YaripElementBlacklistItem(
                        nElement.getAttribute("xpath"),
                        nElement.getAttribute("deep"),
                        nElement.getAttribute("force") == "true",
                        nElement.getAttribute("dow"),
                        null, // placeholder
                        pageId, // created
                        null, // lastFound
                        nElement.getAttribute("found"),
                        nElement.getAttribute("notFound"),
                        nElement.getAttribute("ignored"));
                    page.elementBlacklist.add(element);
                    nElement = iElement.iterateNext();
                }
            }

            // CONTENT BLACKLIST
            var iContentBlacklist = doc.evaluate("./contentBlacklist", nPage, null, ORDERED_NODE_ITERATOR_TYPE, null);
            var nContent = iContentBlacklist.iterateNext();
            if (nContent)
            {
                var iContent = doc.evaluate("./content", nContent, null, ORDERED_NODE_ITERATOR_TYPE, null);
                var nContent = iContent.iterateNext();
                while (nContent)
                {
                    page.contentBlacklist.add(new YaripContentBlacklistItem(
                        nContent.getAttribute("regexp"),
                        nContent.getAttribute("deep"),
                        nContent.getAttribute("force") == "true",
                        pageId, // created
                        null, // lastFound
                        nContent.getAttribute("ignored")));
                    nContent = iContent.iterateNext();
                }
            }

            map.add(page);
            nPage = iPage.iterateNext();
        }

        var iPage = doc.evaluate("./page", nYarip, null, ORDERED_NODE_ITERATOR_TYPE, null);
        var nPage = iPage.iterateNext();
        while (nPage)
        {
            pageName = nPage.getAttribute("name");
            var pageId = nPage.getAttribute("id");
            var page = map.obj[pageName] ? map.obj[pageName] : new YaripPage(pageId ? pageId : this.getId(), pageName);

            // ELEMENT WHITELIST
            var iElementWhitelist = doc.evaluate("./elementWhitelist", nPage, null, ORDERED_NODE_ITERATOR_TYPE, null);
            var nElementWhitelist = iElementWhitelist.iterateNext();
            if (nElementWhitelist)
            {
//                page.elementWhitelist.setExclusive(nElementWhitelist.getAttribute("exclusive"));
                var iElement = doc.evaluate("./element", nElementWhitelist, null, ORDERED_NODE_ITERATOR_TYPE, null);
                var nElement = iElement.iterateNext();
                while (nElement)
                {
                    page.elementWhitelist.add(new YaripElementWhitelistItem(
                        nElement.getAttribute("xpath"),
                        nElement.getAttribute("deep"),
                        null, // force
                        pageId, // created
                        null, // lastFound
                        nElement.getAttribute("found"),
                        nElement.getAttribute("notFound"),
                        nElement.getAttribute("ignored")));
                    nElement = iElement.iterateNext();
                }
            }

            // CONTENT WHITELIST
            var iContentWhitelist = doc.evaluate("./contentWhitelist", nPage, null, ORDERED_NODE_ITERATOR_TYPE, null);
            var nContent = iContentWhitelist.iterateNext();
            if (nContent)
            {
                page.contentWhitelist.setExclusive(nContent.getAttribute("exclusive"));
                var iContent = doc.evaluate("./content", nContent, null, ORDERED_NODE_ITERATOR_TYPE, null);
                var nContent = iContent.iterateNext();
                while (nContent)
                {
                    page.contentWhitelist.add(new YaripContentWhitelistItem(
                        nContent.getAttribute("regexp"),
                        nContent.getAttribute("deep"),
                        null, // force
                        pageId, // created
                        null)); // lastFound
                    nContent = iContent.iterateNext();
                }
            }

            // STYLES
            var iStyles = doc.evaluate("./styles", nPage, null, ORDERED_NODE_ITERATOR_TYPE, null);
            var nStyles = iStyles.iterateNext();
            if (nStyles)
            {
                var iElement = doc.evaluate("./element", nStyles, null, ORDERED_NODE_ITERATOR_TYPE, null);
                var nElement = iElement.iterateNext();
                while (nElement)
                {
                    page.elementAttributeList.add(new YaripElementAttributeItem(
                        nElement.getAttribute("xpath"),
                        "style",
                        nElement.getAttribute("style"),
                        nElement.getAttribute("deep"),
                        pageId, // created
                        null, // lastFound
                        nElement.getAttribute("found"),
                        nElement.getAttribute("notFound")));
                    nElement = iElement.iterateNext();
                }
            }

            // TODO Remove this, since it was a temporary change!
            // ATTRIBUTES
            var iAttributes = doc.evaluate("./attributes", nPage, null, ORDERED_NODE_ITERATOR_TYPE, null);
            var nAttributes = iAttributes.iterateNext();
            if (nAttributes)
            {
                var iElement = doc.evaluate("./element", nAttributes, null, ORDERED_NODE_ITERATOR_TYPE, null);
                var nElement = iElement.iterateNext();
                while (nElement)
                {
                    page.elementAttributeList.add(new YaripElementAttributeItem(
                        nElement.getAttribute("xpath"),
                        nElement.getAttribute("name"),
                        nElement.getAttribute("value"),
                        nElement.getAttribute("deep"),
                        pageId, // created
                        nElement.getAttribute("lastFound"),
                        nElement.getAttribute("found"),
                        nElement.getAttribute("notFound")));
                    nElement = iElement.iterateNext();
                }
            }

            map.add(page);
            nPage = iPage.iterateNext();
        }

        if (imported) {
            this.map.merge(map);
        } else {
            this.map = map;
        }

        var iPage = doc.evaluate("./page", nYarip, null, ORDERED_NODE_ITERATOR_TYPE, null);
        var nPage = iPage.iterateNext();
        while (nPage)
        {
            pageName = nPage.getAttribute("name");
            var pageId = nPage.getAttribute("id");
            var page = map.obj[pageName];

            // EXTENSION
            var iExtension = doc.evaluate("./extension", nPage, null, ORDERED_NODE_ITERATOR_TYPE, null);
            var nExtension = iExtension.iterateNext();
            if (nExtension)
            {
                var iItem = doc.evaluate("./item", nExtension, null, ORDERED_NODE_ITERATOR_TYPE, null);
                var nItem = iItem.iterateNext();
                while (nItem)
                {
                    var value = nItem.getAttribute("value");
                    if (value != page.getId()) {
                        page.pageExtensionList.add(new YaripPageExtensionItem(
                            nItem.getAttribute("value"),
                            nItem.getAttribute("doElements"),
                            nItem.getAttribute("doContents"),
                            nItem.getAttribute("doScripts"),
                            false,
                            false,
//                            false, // links
                            true, // links
                            pageId)); // created
                    }
                    nItem = iItem.iterateNext();
                }
                page.pageExtensionList.removeByKey(page.getId()); // remove self
            }

            nPage = iPage.iterateNext();
        }
    }
    else if (/^0\.2\.4$/.test(version))
    {
        var iPage = doc.evaluate("./page", nYarip, null, ORDERED_NODE_ITERATOR_TYPE, null);
        var nPage = iPage.iterateNext();
        while (nPage)
        {
            pageName = nPage.getAttribute("name");
            var pageId = nPage.getAttribute("id");
            var page = map.obj[pageName] ? map.obj[pageName] : new YaripPage(pageId ? pageId : this.getId(), pageName);

            // ELEMENT WHITELIST
            var iElementWhitelist = doc.evaluate("./elementWhitelist", nPage, null, ORDERED_NODE_ITERATOR_TYPE, null);
            var nElementWhitelist = iElementWhitelist.iterateNext();
            if (nElementWhitelist)
            {
                var iElement = doc.evaluate("./element", nElementWhitelist, null, ORDERED_NODE_ITERATOR_TYPE, null);
                var nElement = iElement.iterateNext();
                while (nElement)
                {
                    var iXPath = doc.evaluate("./xpath", nElement, null, ORDERED_NODE_ITERATOR_TYPE, null);
                    var nXPath = iXPath.iterateNext();
                    if (nXPath) {
                        var created = nElement.getAttribute("created");
                        page.elementWhitelist.add(new YaripElementWhitelistItem(
                            nXPath.textContent,
                            nElement.getAttribute("deep"),
                            null, // force
                            created ? created : pageId,
                            nElement.getAttribute("lastFound"),
                            nElement.getAttribute("found"),
                            nElement.getAttribute("notFound"),
                            nElement.getAttribute("ignored")));
                    }
                    nElement = iElement.iterateNext();
                }
            }

            // ELEMENT BLACKLIST
            var iElementBlacklist = doc.evaluate("./elementBlacklist", nPage, null, ORDERED_NODE_ITERATOR_TYPE, null);
            var nElementBlacklist = iElementBlacklist.iterateNext();
            if (nElementBlacklist)
            {
                var iElement = doc.evaluate("./element", nElementBlacklist, null, ORDERED_NODE_ITERATOR_TYPE, null);
                var nElement = iElement.iterateNext();
                while (nElement)
                {
                    var iXPath = doc.evaluate("./xpath", nElement, null, ORDERED_NODE_ITERATOR_TYPE, null);
                    var nXPath = iXPath.iterateNext();
                    if (nXPath) {
                        var created = nElement.getAttribute("created");
                        page.elementBlacklist.add(new YaripElementBlacklistItem(
                            nXPath.textContent,
                            nElement.getAttribute("deep"),
                            nElement.getAttribute("force"),
                            nElement.getAttribute("dow"),
                            null, // placeholder
                            created ? created : pageId,
                            nElement.getAttribute("lastFound"),
                            nElement.getAttribute("found"),
                            nElement.getAttribute("notFound"),
                            nElement.getAttribute("ignored")));
                    }
                    nElement = iElement.iterateNext();
                }
            }

            // CONTENT WHITELIST
            var iContentWhitelist = doc.evaluate("./contentWhitelist", nPage, null, ORDERED_NODE_ITERATOR_TYPE, null);
            var nContent = iContentWhitelist.iterateNext();
            if (nContent)
            {
                page.contentWhitelist.setExclusive(nContent.getAttribute("exclusive"));
                var iContent = doc.evaluate("./content", nContent, null, ORDERED_NODE_ITERATOR_TYPE, null);
                var nContent = iContent.iterateNext();
                while (nContent)
                {
                    var iRegexp = doc.evaluate("./regexp", nContent, null, ORDERED_NODE_ITERATOR_TYPE, null);
                    var nRegexp = iRegexp.iterateNext();
                    if (nRegexp) {
                        var created = nContent.getAttribute("created");
                        page.contentWhitelist.add(new YaripContentWhitelistItem(
                            nRegexp.textContent,
                            nContent.getAttribute("deep"),
                            null,
                            created ? created : pageId,
                            nContent.getAttribute("lastFound")));
                    }
                    nContent = iContent.iterateNext();
                }
            }

            // CONTENT BLACKLIST
            var iContentBlacklist = doc.evaluate("./contentBlacklist", nPage, null, ORDERED_NODE_ITERATOR_TYPE, null);
            var nContent = iContentBlacklist.iterateNext();
            if (nContent)
            {
                var iContent = doc.evaluate("./content", nContent, null, ORDERED_NODE_ITERATOR_TYPE, null);
                var nContent = iContent.iterateNext();
                while (nContent)
                {
                    var iRegexp = doc.evaluate("./regexp", nContent, null, ORDERED_NODE_ITERATOR_TYPE, null);
                    var nRegexp = iRegexp.iterateNext();
                    if (nRegexp) {
                        var created = nContent.getAttribute("created");
                        page.contentBlacklist.add(new YaripContentBlacklistItem(
                            nRegexp.textContent,
                            nContent.getAttribute("deep"),
                            nContent.getAttribute("force"),
                            created ? created : pageId,
                            nContent.getAttribute("lastFound"),
                            nContent.getAttribute("ignored")));
                    }
                    nContent = iContent.iterateNext();
                }
            }

            // ATTRIBUTES
            var iAttributes = doc.evaluate("./attributes", nPage, null, ORDERED_NODE_ITERATOR_TYPE, null);
            var nAttributes = iAttributes.iterateNext();
            if (nAttributes)
            {
                var iElement = doc.evaluate("./element", nAttributes, null, ORDERED_NODE_ITERATOR_TYPE, null);
                var nElement = iElement.iterateNext();
                while (nElement)
                {
                    var iXPath = doc.evaluate("./xpath", nElement, null, ORDERED_NODE_ITERATOR_TYPE, null);
                    var nXPath = iXPath.iterateNext();
                    var iName = doc.evaluate("./name", nElement, null, ORDERED_NODE_ITERATOR_TYPE, null);
                    var nName = iName.iterateNext();
                    var iValue = doc.evaluate("./value", nElement, null, ORDERED_NODE_ITERATOR_TYPE, null);
                    var nValue = iValue.iterateNext();
                    if (nXPath && nName && nValue) {
                        var created = nElement.getAttribute("created");
                        page.elementAttributeList.add(new YaripElementAttributeItem(
                            nXPath.textContent,
                            nName.textContent,
                            nValue.textContent,
                            nElement.getAttribute("deep"),
                            created ? created : pageId,
                            nElement.getAttribute("lastFound"),
                            nElement.getAttribute("found"),
                            nElement.getAttribute("notFound")));
                    }
                    nElement = iElement.iterateNext();
                }
            }

            // SCRIPT
            var iScript = doc.evaluate("./script", nPage, null, ORDERED_NODE_ITERATOR_TYPE, null);
            var nScript = iScript.iterateNext();
            if (nScript) {
                page.pageScriptList.setLastChanged(nScript.getAttribute("lastChanged"));
                page.pageScriptList.set(null, null, nScript.textContent);
            }

            map.add(page);
            nPage = iPage.iterateNext();
        }

        if (imported) {
            this.map.merge(map);
        } else {
            this.map = map;
        }

        var iPage = doc.evaluate("./page", nYarip, null, ORDERED_NODE_ITERATOR_TYPE, null);
        var nPage = iPage.iterateNext();
        while (nPage)
        {
            pageName = nPage.getAttribute("name");
            var pageId = nPage.getAttribute("id");
            var page = map.obj[pageName];

            // EXTENSION
            var iExtension = doc.evaluate("./extension", nPage, null, ORDERED_NODE_ITERATOR_TYPE, null);
            var nExtension = iExtension.iterateNext();
            if (nExtension)
            {
                var iPageRef = doc.evaluate("./pageReference", nExtension, null, ORDERED_NODE_ITERATOR_TYPE, null);
                var nPageRef = iPageRef.iterateNext();
                while (nPageRef)
                {
                    var pageExtId = nPageRef.textContent;
                    if (pageExtId && pageExtId != page.getId()) {
                        var created = nPageRef.getAttribute("created");
                        page.pageExtensionList.add(new YaripPageExtensionItem(
                            pageExtId,
                            nPageRef.getAttribute("doElements"),
                            nPageRef.getAttribute("doContents"),
                            nPageRef.getAttribute("doScripts"),
                            nPageRef.getAttribute("doHeaders"),
                            nPageRef.getAttribute("doRedirects"),
                            nPageRef.getAttribute("doLinks"),
                            created ? created : pageId));
                    }
                    nPageRef = iPageRef.iterateNext();
                }
                page.pageExtensionList.removeByKey(page.getId()); // remove self
            }

            nPage = iPage.iterateNext();
        }
    }
    else // if (/^(0\.2\.[5-6](\.\d+)?)|(0\.3\.1)$/.test(version))
    {
        var iPage = doc.evaluate("./page", nYarip, null, ORDERED_NODE_ITERATOR_TYPE, null);
        var nPage = iPage.iterateNext();
        while (nPage)
        {
            pageName = nPage.getAttribute("name");
            var pageId = nPage.getAttribute("id");
            var page = map.obj[pageName] ? map.obj[pageName] : new YaripPage(pageId ? pageId : this.getId(), pageName);

            var iElementChild = doc.evaluate("./element", nPage, null, ORDERED_NODE_ITERATOR_TYPE, null);
            var nElementChild = iElementChild.iterateNext();
            if (nElementChild)
            {
                // ELEMENT WHITELIST
                var iElementWhitelist = doc.evaluate("./whitelist", nElementChild, null, ORDERED_NODE_ITERATOR_TYPE, null);
                var nElementWhitelist = iElementWhitelist.iterateNext();
                if (nElementWhitelist)
                {
                    var iItem = doc.evaluate("./item", nElementWhitelist, null, ORDERED_NODE_ITERATOR_TYPE, null);
                    var nItem = iItem.iterateNext();
                    while (nItem)
                    {
                        var iXPath = doc.evaluate("./xpath", nItem, null, ORDERED_NODE_ITERATOR_TYPE, null);
                        var nXPath = iXPath.iterateNext();
                        if (nXPath) {
                            var created = nItem.getAttribute("created");
                            page.elementWhitelist.add(new YaripElementWhitelistItem(
                                nXPath.textContent,
                                nItem.getAttribute("deep"),
                                nItem.getAttribute("force"),
                                created ? created : pageId,
                                nItem.getAttribute("lastFound"),
                                nItem.getAttribute("found"),
                                nItem.getAttribute("notFound"),
                                nItem.getAttribute("ignored")));
                        }
                        nItem = iItem.iterateNext();
                    }
                }

                // ELEMENT BLACKLIST
                var iElementBlacklist = doc.evaluate("./blacklist", nElementChild, null, ORDERED_NODE_ITERATOR_TYPE, null);
                var nElementBlacklist = iElementBlacklist.iterateNext();
                if (nElementBlacklist)
                {
                    var iItem = doc.evaluate("./item", nElementBlacklist, null, ORDERED_NODE_ITERATOR_TYPE, null);
                    var nItem = iItem.iterateNext();
                    while (nItem)
                    {
                        var iXPath = doc.evaluate("./xpath", nItem, null, ORDERED_NODE_ITERATOR_TYPE, null);
                        var nXPath = iXPath.iterateNext();
                        if (nXPath) {
                            var created = nItem.getAttribute("created");
                            var element = new YaripElementBlacklistItem(
                                nXPath.textContent,
                                nItem.getAttribute("deep"),
                                nItem.getAttribute("force"),
                                nItem.getAttribute("dow"),
                                nItem.getAttribute("placeholder"),
                                created ? created : pageId,
                                nItem.getAttribute("lastFound"),
                                nItem.getAttribute("found"),
                                nItem.getAttribute("notFound"),
                                nItem.getAttribute("ignored"));
                            page.elementBlacklist.add(element);
                        }
                        nItem = iItem.iterateNext();
                    }
                }

                // ELEMENT ATTRIBUTE
                var iElementAttribute = doc.evaluate("./attribute", nElementChild, null, ORDERED_NODE_ITERATOR_TYPE, null);
                var nElementAttribute = iElementAttribute.iterateNext();
                if (nElementAttribute)
                {
                    var iItem = doc.evaluate("./item", nElementAttribute, null, ORDERED_NODE_ITERATOR_TYPE, null);
                    var nItem = iItem.iterateNext();
                    while (nItem)
                    {
                        var iXPath = doc.evaluate("./xpath", nItem, null, ORDERED_NODE_ITERATOR_TYPE, null);
                        var nXPath = iXPath.iterateNext();
                        var iName = doc.evaluate("./name", nItem, null, ORDERED_NODE_ITERATOR_TYPE, null);
                        var nName = iName.iterateNext();
                        var iValue = doc.evaluate("./value", nItem, null, ORDERED_NODE_ITERATOR_TYPE, null);
                        var nValue = iValue.iterateNext();
                        if (nXPath && nName && nValue) {
                            var created = nItem.getAttribute("created");
                            page.elementAttributeList.add(new YaripElementAttributeItem(
                                nXPath.textContent,
                                nName.textContent,
                                nValue.textContent,
                                nItem.getAttribute("deep"),
                                created ? created : pageId,
                                nItem.getAttribute("lastFound"),
                                nItem.getAttribute("found"),
                                nItem.getAttribute("notFound")));
                        }
                        nItem = iItem.iterateNext();
                    }
                }

                // ELEMENT SCRIPT
                var iElementScript = doc.evaluate("./script", nElementChild, null, ORDERED_NODE_ITERATOR_TYPE, null);
                var nElementScript = iElementScript.iterateNext();
                if (nElementScript)
                {
                    var iItem = doc.evaluate("./item", nElementScript, null, ORDERED_NODE_ITERATOR_TYPE, null);
                    var nItem = iItem.iterateNext();
                    while (nItem)
                    {
                        var iXPath = doc.evaluate("./xpath", nItem, null, ORDERED_NODE_ITERATOR_TYPE, null);
                        var nXPath = iXPath.iterateNext();
                        var iScript = doc.evaluate("./script", nItem, null, ORDERED_NODE_ITERATOR_TYPE, null);
                        var nScript = iScript.iterateNext();
                        if (nXPath) {
                            var created = nItem.getAttribute("created");
                            var element = new YaripElementScriptItem(
                                nXPath.textContent,
                                nScript ? nScript.textContent : null,
                                nItem.getAttribute("deep"),
                                created ? created : pageId,
                                nItem.getAttribute("lastFound"),
                                nItem.getAttribute("found"),
                                nItem.getAttribute("notFound"));
                            page.elementScriptList.add(element);
                        }
                        nItem = iItem.iterateNext();
                    }
                }
            }

            var iContentChild = doc.evaluate("./content", nPage, null, ORDERED_NODE_ITERATOR_TYPE, null);
            var nContentChild = iContentChild.iterateNext();
            if (nContentChild)
            {
                // CONTENT WHITELIST
                var iContentWhitelist = doc.evaluate("./whitelist", nContentChild, null, ORDERED_NODE_ITERATOR_TYPE, null);
                var nContentWhitelist = iContentWhitelist.iterateNext();
                if (nContentWhitelist)
                {
                    page.contentWhitelist.setExclusive(nContentWhitelist.getAttribute("exclusive"));
                    var iItem = doc.evaluate("./item", nContentWhitelist, null, ORDERED_NODE_ITERATOR_TYPE, null);
                    var nItem = iItem.iterateNext();
                    while (nItem)
                    {
                        var iRegexp = doc.evaluate("./regexp", nItem, null, ORDERED_NODE_ITERATOR_TYPE, null);
                        var nRegexp = iRegexp.iterateNext();
                        if (nRegexp) {
                            var created = nItem.getAttribute("created");
                            page.contentWhitelist.add(new YaripContentWhitelistItem(
                                nRegexp.textContent,
                                nItem.getAttribute("deep"),
                                nItem.getAttribute("force"),
                                created ? created : pageId,
                                nItem.getAttribute("lastFound")));
                        }
                        nItem = iItem.iterateNext();
                    }
                }

                // CONTENT BLACKLIST
                var iContentBlacklist = doc.evaluate("./blacklist", nContentChild, null, ORDERED_NODE_ITERATOR_TYPE, null);
                var nContentBlacklist = iContentBlacklist.iterateNext();
                if (nContentBlacklist)
                {
                    var iItem = doc.evaluate("./item", nContentBlacklist, null, ORDERED_NODE_ITERATOR_TYPE, null);
                    var nItem = iItem.iterateNext();
                    while (nItem)
                    {
                        var iRegexp = doc.evaluate("./regexp", nItem, null, ORDERED_NODE_ITERATOR_TYPE, null);
                        var nRegexp = iRegexp.iterateNext();
                        if (nRegexp) {
                            var created = nItem.getAttribute("created");
                            page.contentBlacklist.add(new YaripContentBlacklistItem(
                                nRegexp.textContent,
                                nItem.getAttribute("deep"),
                                nItem.getAttribute("force"),
                                created ? created : pageId,
                                nItem.getAttribute("lastFound"),
                                nItem.getAttribute("ignored")));
                        }
                        nItem = iItem.iterateNext();
                    }
                }

                var iContentHeader = doc.evaluate("./header", nContentChild, null, ORDERED_NODE_ITERATOR_TYPE, null);
                var nContentHeader = iContentHeader.iterateNext();
                if (nContentHeader)
                {
                    // CONTENT REQUEST HEADER
                    var iContentRequestHeader = doc.evaluate("./request", nContentHeader, null, ORDERED_NODE_ITERATOR_TYPE, null);
                    var nContentRequestHeader = iContentRequestHeader.iterateNext();
                    if (nContentRequestHeader)
                    {
                        var iItem = doc.evaluate("./item", nContentRequestHeader, null, ORDERED_NODE_ITERATOR_TYPE, null);
                        var nHeader = iItem.iterateNext();
                        while (nHeader)
                        {
                            var iRegexp = doc.evaluate("./regexp", nHeader, null, ORDERED_NODE_ITERATOR_TYPE, null);
                            var nRegexp = iRegexp.iterateNext();
                            var iName = doc.evaluate("./name", nHeader, null, ORDERED_NODE_ITERATOR_TYPE, null);
                            var nName = iName.iterateNext();
                            var iScript = doc.evaluate("./script", nHeader, null, ORDERED_NODE_ITERATOR_TYPE, null);
                            var nScript = iScript.iterateNext();
                            if (nRegexp) {
                                var created = nHeader.getAttribute("created");
                                page.contentRequestHeaderList.add(new YaripHeaderItem(
                                    nRegexp.textContent,
                                    nName.textContent,
                                    nScript ? nScript.textContent : null,
                                    nHeader.getAttribute("merge"),
                                    nHeader.getAttribute("deep"),
                                    created ? created : pageId,
                                    nHeader.getAttribute("lastFound")));
                            }
                            nHeader = iItem.iterateNext();
                        }
                    }

                    // CONTENT RESPONSE HEADER
                    var iContentResponseHeader = doc.evaluate("./response", nContentHeader, null, ORDERED_NODE_ITERATOR_TYPE, null);
                    var nContentResponseHeader = iContentResponseHeader.iterateNext();
                    if (nContentResponseHeader)
                    {
                        var iItem = doc.evaluate("./item", nContentResponseHeader, null, ORDERED_NODE_ITERATOR_TYPE, null);
                        var nHeader = iItem.iterateNext();
                        while (nHeader)
                        {
                            var iRegexp = doc.evaluate("./regexp", nHeader, null, ORDERED_NODE_ITERATOR_TYPE, null);
                            var nRegexp = iRegexp.iterateNext();
                            var iName = doc.evaluate("./name", nHeader, null, ORDERED_NODE_ITERATOR_TYPE, null);
                            var nName = iName.iterateNext();
                            var iScript = doc.evaluate("./script", nHeader, null, ORDERED_NODE_ITERATOR_TYPE, null);
                            var nScript = iScript.iterateNext();
                            if (nRegexp) {
                                var created = nHeader.getAttribute("created");
                                page.contentResponseHeaderList.add(new YaripHeaderItem(
                                    nRegexp.textContent,
                                    nName.textContent,
                                    nScript ? nScript.textContent : null,
                                    nHeader.getAttribute("merge"),
                                    nHeader.getAttribute("deep"),
                                    created ? created : pageId,
                                    nHeader.getAttribute("lastFound")));
                            }
                            nHeader = iItem.iterateNext();
                        }
                    }
                }

                // CONTENT REDIRECT
                var iContentRedirect = doc.evaluate("./redirect", nContentChild, null, ORDERED_NODE_ITERATOR_TYPE, null);
                var nContentRedirect = iContentRedirect.iterateNext();
                if (nContentRedirect)
                {
                    var iItem = doc.evaluate("./item", nContentRedirect, null, ORDERED_NODE_ITERATOR_TYPE, null);
                    var nItem = iItem.iterateNext();
                    while (nItem)
                    {
                        var iRegexp = doc.evaluate("./regexp", nItem, null, ORDERED_NODE_ITERATOR_TYPE, null);
                        var nRegexp = iRegexp.iterateNext();
                        var iNewSubStr = doc.evaluate("./newsubstr", nItem, null, ORDERED_NODE_ITERATOR_TYPE, null);
                        var nNewSubStr = iNewSubStr.iterateNext();
                        if (nRegexp) {
                            var created = nItem.getAttribute("created");
                            page.contentRedirectList.add(new YaripRedirectItem(
                                nRegexp.textContent,
                                nNewSubStr ? nNewSubStr.textContent : null,
                                nItem.getAttribute("deep"),
                                created ? created : pageId,
                                nItem.getAttribute("lastFound")));
                        }
                        nItem = iItem.iterateNext();
                    }
                }
            }

            var iPageChild = doc.evaluate("./page", nPage, null, ORDERED_NODE_ITERATOR_TYPE, null);
            var nPageChild = iPageChild.iterateNext();
            if (nPageChild)
            {
                // PAGE STYLE
                var iPageStyle = doc.evaluate("./style", nPageChild, null, ORDERED_NODE_ITERATOR_TYPE, null);
                var nPageStyle = iPageStyle.iterateNext();
                if (nPageStyle) {
                    page.pageStyleList.setLastChanged(nPageStyle.getAttribute("lastChanged"));
                    page.pageStyleList.set(null, null, nPageStyle.textContent);
                }

                // PAGE SCRIPT
                var iPageScript = doc.evaluate("./script", nPageChild, null, ORDERED_NODE_ITERATOR_TYPE, null);
                var nPageScript = iPageScript.iterateNext();
                if (nPageScript) {
                    page.pageScriptList.setLastChanged(nPageScript.getAttribute("lastChanged"));
                    page.pageScriptList.set(null, null, nPageScript.textContent);
                }

                var iPageHeader = doc.evaluate("./header", nPageChild, null, ORDERED_NODE_ITERATOR_TYPE, null);
                var nPageHeader = iPageHeader.iterateNext();
                if (nPageHeader)
                {
                    // PAGE REQUEST HEADER
                    var iPageRequestHeader = doc.evaluate("./request", nPageHeader, null, ORDERED_NODE_ITERATOR_TYPE, null);
                    var nPageRequestHeader = iPageRequestHeader.iterateNext();
                    if (nPageRequestHeader)
                    {
                        var iItem = doc.evaluate("./item", nPageRequestHeader, null, ORDERED_NODE_ITERATOR_TYPE, null);
                        var nItem = iItem.iterateNext();
                        while (nItem)
                        {
                            var iRegexp = doc.evaluate("./regexp", nItem, null, ORDERED_NODE_ITERATOR_TYPE, null);
                            var nRegexp = iRegexp.iterateNext();
                            var iName = doc.evaluate("./name", nItem, null, ORDERED_NODE_ITERATOR_TYPE, null);
                            var nName = iName.iterateNext();
                            var iScript = doc.evaluate("./script", nItem, null, ORDERED_NODE_ITERATOR_TYPE, null);
                            var nScript = iScript.iterateNext();
                            if (nRegexp) {
                                var created = nItem.getAttribute("created");
                                page.pageRequestHeaderList.add(new YaripHeaderItem(
                                    nRegexp.textContent,
                                    nName.textContent,
                                    nScript ? nScript.textContent : null,
                                    nItem.getAttribute("merge"),
                                    nItem.getAttribute("deep"),
                                    created ? created : pageId,
                                    nItem.getAttribute("lastFound")));
                            }
                            nItem = iItem.iterateNext();
                        }
                    }

                    // PAGE RESPONSE HEADER
                    var iPageResponseHeader = doc.evaluate("./response", nPageHeader, null, ORDERED_NODE_ITERATOR_TYPE, null);
                    var nPageResponseHeader = iPageResponseHeader.iterateNext();
                    if (nPageResponseHeader)
                    {
                        var iItem = doc.evaluate("./item", nPageResponseHeader, null, ORDERED_NODE_ITERATOR_TYPE, null);
                        var nItem = iItem.iterateNext();
                        while (nItem)
                        {
                            var iRegexp = doc.evaluate("./regexp", nItem, null, ORDERED_NODE_ITERATOR_TYPE, null);
                            var nRegexp = iRegexp.iterateNext();
                            var iName = doc.evaluate("./name", nItem, null, ORDERED_NODE_ITERATOR_TYPE, null);
                            var nName = iName.iterateNext();
                            var iScript = doc.evaluate("./script", nItem, null, ORDERED_NODE_ITERATOR_TYPE, null);
                            var nScript = iScript.iterateNext();
                            if (nRegexp) {
                                var created = nItem.getAttribute("created");
                                page.pageResponseHeaderList.add(new YaripHeaderItem(
                                    nRegexp.textContent,
                                    nName.textContent,
                                    nScript ? nScript.textContent : null,
                                    nItem.getAttribute("merge"),
                                    nItem.getAttribute("deep"),
                                    created ? created : pageId,
                                    nItem.getAttribute("lastFound")));
                            }
                            nItem = iItem.iterateNext();
                        }
                    }
                }

                // PAGE REDIRECT
                var iPageRedirect = doc.evaluate("./redirect", nPageChild, null, ORDERED_NODE_ITERATOR_TYPE, null);
                var nPageRedirect = iPageRedirect.iterateNext();
                if (nPageRedirect)
                {
                    var iItem = doc.evaluate("./item", nPageRedirect, null, ORDERED_NODE_ITERATOR_TYPE, null);
                    var nItem = iItem.iterateNext();
                    while (nItem)
                    {
                        var iRegexp = doc.evaluate("./regexp", nItem, null, ORDERED_NODE_ITERATOR_TYPE, null);
                        var nRegexp = iRegexp.iterateNext();
                        var iNewSubStr = doc.evaluate("./newsubstr", nItem, null, ORDERED_NODE_ITERATOR_TYPE, null);
                        var nNewSubStr = iNewSubStr.iterateNext();
                        if (nRegexp) {
                            var created = nItem.getAttribute("created");
                            page.pageRedirectList.add(new YaripRedirectItem(
                                nRegexp.textContent,
                                nNewSubStr ? nNewSubStr.textContent : null,
                                nItem.getAttribute("deep"),
                                created ? created : pageId,
                                nItem.getAttribute("lastFound")));
                        }
                        nItem = iItem.iterateNext();
                    }
                }
            }

            map.add(page);
            nPage = iPage.iterateNext();
        }

        if (imported) {
            this.map.merge(map);
        } else {
            this.map = map;
        }

        var iPage = doc.evaluate("./page", nYarip, null, ORDERED_NODE_ITERATOR_TYPE, null);
        var nPage = iPage.iterateNext();
        while (nPage)
        {
            pageName = nPage.getAttribute("name");
            var pageId = nPage.getAttribute("id");
//            var page = map.obj[pageName];
            var page = this.map.obj[pageName];

            var iPageChild = doc.evaluate("./page", nPage, null, ORDERED_NODE_ITERATOR_TYPE, null);
            var nPageChild = iPageChild.iterateNext();
            if (nPageChild)
            {
                // PAGE EXTENSION
                var iExtension = doc.evaluate("./extension", nPageChild, null, ORDERED_NODE_ITERATOR_TYPE, null);
                var nExtension = iExtension.iterateNext();
                if (nExtension)
                {
                    var iItem = doc.evaluate("./item", nExtension, null, ORDERED_NODE_ITERATOR_TYPE, null);
                    var nItem = iItem.iterateNext();
                    while (nItem)
                    {
                        var pageExtId = nItem.textContent;
                        if (pageExtId && pageExtId != page.getId()) {
                            var created = nItem.getAttribute("created");
                            page.pageExtensionList.add(new YaripPageExtensionItem(
                                pageExtId,
                                nItem.getAttribute("doElements"),
                                nItem.getAttribute("doContents"),
                                nItem.getAttribute("doScripts"),
                                nItem.getAttribute("doHeaders"),
                                nItem.getAttribute("doRedirects"),
                                nItem.getAttribute("doLinks"),
                                created ? created : pageId));
                        }
                        nItem = iItem.iterateNext();
                    }
                    page.pageExtensionList.removeByKey(page.getId()); // remove self
                }
            }

            nPage = iPage.iterateNext();
        }
    }
//    else {
    // TODO Log warning about unsupported version!
//    }

    if (imported) {
        this.map.merge(map);
        this.reloadPage(null); // reset known
    } else {
        this.map = map;
    }

    return pageName;
}

Yarip.prototype.getDoc = function(file)
{
    if (!file) return null;

    var data = this.getData(file);
    if (!data) return null;

    var dp = Cc['@mozilla.org/xmlextras/domparser;1'].createInstance(Ci.nsIDOMParser);
    return dp.parseFromString(data, "text/xml");
}

Yarip.prototype.getData = function(file)
{
    if (!file) return "";

    var fis = null;
    var sis = null;
    var data = null;
    try {
        fis = Cc['@mozilla.org/network/file-input-stream;1'].createInstance(Ci.nsIFileInputStream);
//        fis.init(file, 0x01, 0004, null);
        fis.init(file, -1, -1, null); // just reading
        sis = Cc['@mozilla.org/scriptableinputstream;1'].createInstance(Ci.nsIScriptableInputStream);
        sis.init(fis);
        data = sis.read(sis.available());
        var suc = Cc["@mozilla.org/intl/scriptableunicodeconverter"].createInstance(Ci.nsIScriptableUnicodeConverter);
        suc.charset = CHARSET;
        var data = suc.ConvertToUnicode(data);
    } catch (e) {
    } finally {
        if (sis) try { sis.close(); } catch (e) {}
        if (fis) try { fis.close(); } catch (e) {}
    }
    return data ? data : "";
}

Yarip.prototype.getFile = function(path)
{
    if (!path) return null;

    var file = Cc["@mozilla.org/file/directory_service;1"].
        getService(Ci.nsIProperties).
        get("ProfD", Ci.nsIFile);
    var aPath = path.split(/\/|\\/);
    for (var i = 0; i < aPath.length; i++) {
        file.append(aPath[i]);
    }
    if (!file.exists()) {
        file.create(Ci.nsIFile.NORMAL_FILE_TYPE, 0600);
    }
    if (file.isFile()) {
        return file;
    } else {
        return null;
    }
}

Yarip.prototype.saveToFile = function(data, file)
{
    if (data != "" && !data || !file) return;

    var fos = null;
    var os = null;
    try {
        fos = Cc["@mozilla.org/network/file-output-stream;1"].createInstance(Ci.nsIFileOutputStream);
        fos.init(file, 0x02 | 0x08 | 0x20, 0600, 0);
        os = Cc["@mozilla.org/intl/converter-output-stream;1"].createInstance(Ci.nsIConverterOutputStream);
        os.init(fos, CHARSET, 0, 0);
        os.writeString(data);
    } catch (e) {
    } finally {
        if (fos) try { fos.close(); } catch (e) {}
        if (os) try { os.close(); } catch (e) {}
    }
}

Yarip.prototype.logContentLocation = function(status, location, contentLocation, mimeTypeGuess)
{
    var date = new Date();
    location = this.getLocationFromLocation(location);
    contentLocation = this.getContentLocationFromContentLocation(contentLocation);
    if (!this.isMobile) {
        for each (var monitor in this.monitorDialogues) {
            monitor.logContentLocation(status, location, contentLocation, date, mimeTypeGuess);
        }
    } else {
        var hours = date.getHours();
        var minutes = date.getMinutes();
        var seconds = date.getSeconds();
        var milliseconds = date.getMilliseconds();
        var time = (hours < 10 ? "0" : "") + hours + ":" + (minutes < 10 ? "0" : "") + minutes + ":" + (seconds < 10 ? "0" : "") + seconds + "." + (milliseconds < 10 ? "00" : (milliseconds < 100 ? "0" : "")) + milliseconds;
        var state = "  ";
        switch (status) {
        case 1: state = "\u263A "; break; // whitelisted
//        case 2: state = "\u2639 "; break; // blacklisted
        case 2: state = "\u263B "; break; // blacklisted
        default: state = status + " "; break;
        }
        var page = location.href;
        var content = contentLocation.spec;
        dump("yarip: " + time + " " + state + " " + page + " " + content + "\n");
    }
}

Yarip.prototype.updateContentType = function(status, location, contentLocation, contentType, responseStatus)
{
    for each (var monitor in this.monitorDialogues) {
        monitor.updateContentType(status, location, contentLocation, contentType, responseStatus);
    }
}

//yarip.updateContentLocation = function(status, location, oldContentLocation, newContentLocation)
//{
//    for each (var monitor in this.monitorDialogues) {
////        monitor.logContentLocation(status, location, contentLocation, date, mimeTypeGuess);
//        monitor.updateContentLocation(status, location, oldContentLocation, newContentLocation);
//    }
//}

Yarip.prototype.shouldBlacklist = function(pageName, addressObj, url, defaultView, doFlag)
{
    doFlag = doFlag ? doFlag : 2; // 2 = content
    var whitelisted = false;
    var result = STATUS_UNKNOWN;

    // WHITELIST
    for (var a in addressObj.obj)
    {
        if (whitelisted) break;
        if (!addressObj.obj[a][doFlag]) continue; // not doFlag

        var contentWhitelist = this.map.get(a).contentWhitelist;
        if (contentWhitelist.length <= 0) continue;

        var isSelf = addressObj.obj[a][0];
        var list = contentWhitelist.obj;
        for each (var content in list)
        {
            if (isSelf && !content.getDeep() && !this.equalAddresses(pageName, a)) continue;
            if (!content.getRegExpObj().test(url)) continue;

            if (isSelf) {
                content.incrementLastFound();
            }
            if (defaultView) {
                defaultView.yaripStatus = "found";
            }
            if (content.getForce()) {
                return STATUS_WHITELISTED;
            } else {
                whitelisted = true;
                result = STATUS_WHITELISTED;
                break;
            }
        }
    }

    // BLACKLIST
    for (var a in addressObj.obj)
    {
        if (!addressObj.obj[a][doFlag]) continue; // not doFlag

        var contentBlacklist = this.map.get(a).contentBlacklist;
        if (contentBlacklist.length <= 0) continue;

        var isSelf = addressObj.obj[a][0];
        var list = contentBlacklist.obj;
        for each (var content in list)
        {
            if (isSelf && !content.getDeep() && !this.equalAddresses(pageName, a)) continue;
            if (!content.getRegExpObj().test(url)) continue;
            if (whitelisted && !content.getForce()) {
                if (isSelf) {
                    content.incrementIgnored();
                }
                continue;
            }

            if (isSelf) {
                content.incrementLastFound();
            }
            if (defaultView) {
                defaultView.yaripStatus = "found";
            }
            return STATUS_BLACKLISTED;
        }
    }

    return !whitelisted && addressObj.exclusive ? STATUS_BLACKLISTED : result;
}

Yarip.prototype.equalAddresses = function(pageName, address)
{
    if (pageName.indexOf(address) == -1) return true; // ignore deep (extension)

    pageName = pageName.replace(/\/+$/, "");
    address = address.replace(/\/+$/, "");
    if (NO_SCHEME_RE.test(address)) {
        pageName = pageName.replace(/^[^:/?#]+:\/{2}/, ""); // removing scheme
    }

    return pageName == address;
}

Yarip.prototype.openLocation = function(url)
{
    try {
        var wm = Cc["@mozilla.org/appshell/window-mediator;1"].
            getService(Ci.nsIWindowMediator);
        var gBrowser = wm.getMostRecentWindow("navigator:browser").gBrowser;
        gBrowser.selectedTab = gBrowser.addTab(url);
    } catch (e) {
        var uri = IOS.newURI(url, CHARSET, null);
        var eps = Cc["@mozilla.org/uriloader/external-protocol-service;1"].
            getService(Ci.nsIExternalProtocolService);
        eps.loadURI(uri);
    }
}

// DEBUG
//yarip.profileObj = {};
//yarip.profileTime = 0;
//yarip.profile = function(key, object, name)
//{
//    var obj = this.profileObj[key] = {
//        name: name,
//        count: 0,
//        totalTime: 0
//    };
//    var fun = object[name];
//    object[name] = function() {
//        obj.count++;
//        var then = Date.now();
//        var result = fun.apply(object, arguments);
//        var time = Date.now() - then;
//        obj.totalTime += time;
//        return result;
//    };
//}

Yarip.prototype.getInterface = function(channel, iid)
{
    if (!channel) return null;

    if (channel.notificationCallbacks) {
        try {
            return channel.notificationCallbacks.QueryInterface(Ci.nsIInterfaceRequestor).getInterface(iid);
        } catch(e) {}
    }

    if (channel.loadGroup) {
        if (channel.loadGroup.notificationCallbacks) {
            try {
                return channel.loadGroup.notificationCallbacks.QueryInterface(Ci.nsIInterfaceRequestor).getInterface(iid);
            } catch(e) {}
        }
        if (channel.loadGroup.groupObserver) {
            try {
                return channel.loadGroup.groupObserver.QueryInterface(Ci.nsIInterfaceRequestor).getInterface(iid);
            } catch(e) {}
        }
    }

    return null;
}

//yarip.getLocation = function(channel, doc)
//{
//    var obj = {
//        location: null,
//        isPage: false
//    };
//    try {
//        var newURI = IOS.newURI(channel.loadGroup.defaultLoadRequest.name, channel.URI.originCharset, null);
//        obj.location = this.getLocationFromContentLocation(newURI);
//        return obj;
//    } catch (e) {
//        if (LOAD_DOCUMENT_URI & channel.loadFlags) {
//            obj.location = this.getLocationFromContentLocation(channel.URI);
//            obj.isPage = true;
//            return obj;
//        } else if (doc) {
//            obj.location = this.getLocationFromLocation(doc.location);
//            return obj;
//        } else {
//            return null;
//        }
//    }
//}
Yarip.prototype.getLocation = function(channel, doc)
{
    var obj = {
        location: null,
        isPage: false
    };
    if (!LOAD_REPLACE & channel.loadFlags) {
        try {
            var newURI = IOS.newURI(channel.loadGroup.defaultLoadRequest.name, channel.URI.originCharset, null);
            obj.location = this.getLocationFromContentLocation(newURI);
            return obj;
        } catch (e) {}
    }
    if (LOAD_DOCUMENT_URI & channel.loadFlags) {
        obj.location = this.getLocationFromContentLocation(channel.URI);
        obj.isPage = true;
        return obj;
    } else if (doc) {
        obj.location = this.getLocationFromLocation(doc.location);
        return obj;
    } else {
        return null;
    }
}

Yarip.prototype.showLinkNotification = function(doc, contentLocation)
{
    var wm = Cc["@mozilla.org/appshell/window-mediator;1"].
        getService(Ci.nsIWindowMediator);
    var browserEnum = wm.getEnumerator("navigator:browser");
    while (browserEnum.hasMoreElements()) {
        var browserWin = browserEnum.getNext();
        var tabbrowser = browserWin.gBrowser;
        var index = tabbrowser.getBrowserIndexForDocument(doc);
        if (index >= 0) {
            var browser = tabbrowser.getBrowserAtIndex(index);
            var message = "Open link `" + contentLocation.spec + "'?";
            var nb = tabbrowser.getNotificationBox(browser);
            var openLocation = this.openLocation;
            var buttons = [{
//                    accessKey: "W",
//                    callback: function() {},
//                    label: "Whitelist",
//                    popup: null
//                },
//                {
//                    accessKey: "B",
//                    callback: function() {},
//                    label: "Blacklist",
//                    popup: null
//                },
//                {
                    accessKey: "O",
                    callback: function() { openLocation(contentLocation.spec); },
                    label: "Open",
                    popup: null
                }];
            nb.removeAllNotifications(true);
//            nb.appendNotification(message, "yarip-open-link", "chrome://yarip/skin/icon.png", nb.PRIORITY_INFO_HIGH, buttons);
            nb.appendNotification(message, "yarip-open-link", "chrome://browser/skin/Info.png", nb.PRIORITY_INFO_HIGH, buttons);
            return;
        }
    }
}

var wrappedJSObject = new Yarip();
//wrappedJSObject.init();

